; BALLYROM.ASM - VERSION 1.5  - JAN. 24, 2004
;                               CHANGES ALL $+(NAME-$) TO NAME
;                        1.4  - MATCHES EXACTLY WITH 3159 ROM
;                        1.32 - DEC. 26, 2003
;                               - ASSEMBLED WITH NO ERRORS
;                        1.3  - DEC. 25, 2003
;                               - FINISHED TYPING
;                        1.15 - MAR. 1 2002
;
; zmac -i -m -o ballyrom.bin -x ballyrom.lst ballyrom.asm 
;
INCLUDE "HVGLIB.H"        ; HOME VIDEO GAME LIBRARY 
  ; **********
  ; * HVGSYS *  HOME VIDEO GAME SYSTEM
  ; **********
SPACE 2
PFUG    EQU 08H           ; POT FUDGE FACTOR
GFSTRT  EQU 17DEH         ; GUNFIGHT START ADDRESS
CMSTRT  EQU 1328H         ; CHECKMATE START ADDRESS
CALCST  EQU 1020H         ; CALCULATOR START ADDRESS
SCBST   EQU 0E19H         ; SCRIBBLING START ADDRESS
SPACE 2
  ; ********************
  ; * POWER UP RESTART *
  ; ********************
        ORG 0
        NOP               ; WAIT FOR THINGS TO SETTLE DOW
        DI
        XOR  A
        OUT  (CONCM),A    ; *** SET CONSUMER MODE ***
        JP   PWRUP
SPACE 2
        ORG 8
  ; TRANSFER CONTROL TO RESTART HANDLER
        JP 2007H          ; VECTOR OUT
SPACE 2
NUMBAS  DB   1CH
        DB   3CH
        DB   1CH
        DB   20H
;
        DB   0FFH ; FILL EMPTY SPACE WITH FFH (TO MATCH 3159 ROM)
;        
SPACE 2
        ORG 16
        JP  200AH         ; RESTART 2
MENUCL  DB   06H          ; MENU COLORS
        DB   0FAH         ; CHANGED TO MATCH 3159 ROM
        DB   07H
        DB   62H          ; CHANGED TO MATCH 3159 ROM
;
        DB   0FFH ; FILL EMPTY SPACE WITH FFH (TO MATCH 3159 ROM)
;        
SPACE 2
        ORG 24
        JP 200DH          ; RESTART 3
EJECT
; NAME         PAUSE
; PURPOSE      HALT # OF INTERUPTS
; INPUT        B = # OF INTERRUPTS
MPAUSE  EI
        HALT
        DJNZ ($-1)  
        RET
EJECT
        ORG 32
        JP  2010H         ; RESTART 4
SPACE 2
; NAME: SET WORD
 ; (HL)=DE
MSETW   LD   (HL),E
        INC  HL
        LD   (HL),D
        RET
;
        DB   0FFH ; FILL EMPTY SPACE WITH FFH (TO MATCH 3159 ROM)
;        
SPACE 2
        ORG  40
        JP   2013H        ; RESTART 5
SPACE 2
CONC2   LD   HL,0         ; ZERO OUT HL
        RET
;
        DB   0FFH ; FILL EMPTY SPACE WITH FFH (TO MATCH 3159 ROM)
;        
SPACE 2
        ORG 48
        JP  2016H         ; RESTART 6
SPACE 2
CKSUM   DB   0            ; CHECKSUM
SPACE 2
ITAB    DW   MACTIN       ; INTERRUPT TRANSFER
        DB   1            ; ** SYSTEM REVISION LEVEL
;
        DB   0FFH ; FILL EMPTY SPACE WITH FFH (TO MATCH 3159 ROM)
;        
SPACE 2
        ORG 56
 ; NAME:         USER PROGRAM INTERFACE
 ; PURPOSE:      TRANSFER OF CONTROL FROM USER TO SYSTEM
 ; INPUT:        ROUTINE # FOLLOWS INLINE AFTER RST INSTR
 ;               IF L.O. BIT SET, LOAD ARGUMENTS INLINE F
 ; OUTPUT:       NONE
 ; STACK USE:    18 BYTES TOTAL, 16 BYTES ON EXIT
 ; SIDE EFFECTS: REGISTERS AF,BC,DE,HL,IX, AND OLD IY SAV   
 ; EXPLANATION
 ;  REGISTERS AF,BC,DE,HL,IX, AND PREVIOUS IY ARE PUSHED
 ; THE NUMBER FOLLOWING THE RST 56 INSTRUCTION IS USED TO
 ; INDEX A JUMP VECTOR GIVING THE STARTING ADDRESS OF THE
 ; SYSTEM ROUTINE TO CALL.  IF OPTIONED, INLINE ARGUMENTS
 ; ARE COPIED INTO THE CONTEXT AREA.  FOR ARGUMENT ORDERIN
 ; SEE INTERPRETER DOCUMENTATION AND APPROP. TABLES
 ; A DUMMY RETURN IS INSERTED WHICH, WHEN RETURNED TO BY
 ; SYSTEM ROUTINE, WILL RESTORE THE REGISTER CONTENTS AND
 ; RETURN TO THE USER PROGRAM
 ;
 ;   *** THE UPI HAS BEEN EXTENDED TO SUPPORT USER SUPPLI
 ;   ROUTINES.  IF THE CALL INDEX PROVIDED IS NEGATIVE
 ;   THEN THE USERS DISPATCH TABLE POINTER (USERTB) IS US
 ;   NOTE THAT THE SIGN BIT ISN'T ZAPPED BEFORE BEING
 ;   USED AS AN INDEX, THIS MEANS THAT THE USERS DISPATCH
 ;   TABLE POINTER SHOULD POINT 128 BYTES BEFORE THE FIRS
        EX   (SP),HL      ; RETURN ADDRESS TO HL
        PUSH AF           ; CREATE CONTEXT
        PUSH BC
        PUSH DE
        PUSH IX
        PUSH IY
        LD   IY,0         ; POINT IY AT CONTEXT
        ADD  IY,SP
        LD   A,(HL)       ; LOAD OPCODE
        INC  HL
        LD   DE,RETN_     ; DE = RETURN POINT
        RRA               ; SUCK WANTED
        JR   C,MINT0      ; JUMP IF YES
INTPE   PUSH HL           ; SAVE PC
        PUSH DE           ; SAVE DUMMY RETURN
        LD   HL,SYSDPT
        RLCA
        LD   E,A
        LD   D,0
        RLA               ; USER TABLE WANTED?
        JR   NC,PUSH1
        LD   HL,(USERTB)  ; YES    LOAD IT
PUSH1   ADD  HL,DE
        LD   E,(HL)
        INC  HL
        LD   D,(HL)
        PUSH DE
        LD   H,(IY+CBH)
        LD   L,(IY+CBL)
RELD    LD   D,(IY+CBIXH)
        LD   E,(IY+CBIXL)
        PUSH DE
        POP  IX
        LD   A,(IY+CBA)
DELOAD  LD   D,(IY+CBD)
        LD   E,(IY+CBE)
        RET               ; CALL VIA RETURN
EJECT
 ; NAME:         MACRO INTERPRETER
 ; PURPOSE:      INTERPRETING SEQUENCES OF SYSTEM CALLS
 ; INPUT:        ADDRESS OF STRING 0 TO INTERPRET PASSED ON
 ; STACK USE:    NO INCREASE IN DEPTH
 ; EXPLANATION:  IF OPTIONED (BIT 0 OF CALL INDEX SET) THE
 ; ARGUMENT TABLE (MRARGT) IS INDEXED GIVING A MASK WHICH
 ; SPECIFIES HOW TO TRANSFER INLINE ARGUMENTS INTO THE COL
 ; BLOCK.  THIS MASK IS FORMATTED AS FOLLOWS:
 ;
 ; 
 ; **********************************
 ; * 7 * 6 * 5 * 4  * 3 * 2 * 1 * 0 *
 ; **********************************
 ; * H * L * A * IX * B * C * D * E *
 ; **********************************
 ; ARGUMENTS MUST FOLLOW THE CALL INDEX IN THE FOLLOWING
 ; (OMITTING UNUSED ARGUMENTS, OF COURSE)
 ; (INDEX),IXL,IXH,E,D,C,B,A,L,H
 ;
 ;       THE SIMULATED PC IS SAVED AND A DUMMY RETURN IS
 ; INSERTED ON THE STACK.  THE UPI DISPATCHING ROUTINE IS
 ; THEN ENTERED AT 'INTPE', WHICH EFFECTS A CONTROL TRANS
 ; TO THE CALLED ROUTINE.  WHEN THE CALLED ROUTINE RETURN
 ; IT WILL COME BACK HERE TO INTERPRET THE NEXT MACRO INS
 ; NOTE THAT THIS ROUTINE IS REENTRANT, THEREFORE THE CAL
 ; ROUTINE MAY RECUR BACK THRU HERE, IF IT FEELS LIKE IT.
 ; ** THE UPI HAS BEEN EXTENDED TO PROVIDE USER PROVIDED
 ; SYSTEM ROUTINES.  IF A NEGATIVE CALL INDEX IS ENCOUNTER
 ; BY THE INTERPRETER, AND 'SUCK INLINE' IS OPTIONED, THE
 ; USER MACRO ROUTINE ARGUMENT TABLE IS INDEXED FOR A
 ; PARAMETER MASK.  THE ADDRESS OF THIS TABLE IS ASSUMED
 ; TO BE IN (UMARGT),(UMARGT+1).  THIS POINTER SHOULD
 ; POINT 64 BYTES BEFORE THE FIRST REAL ENTRY.
 ; I.E. LD      HL,USERMT-64   ; WHERE USERMT POINTS AT
 ;      LD      (UMARGT),HL
MINTPC  POP  DE           ; DISCARD DUMMY RETURN FROM UPI
RENTER 
        POP HL            ; POP OFF PC
 SPACE 2
 ; NAME:        MCALL
 ; PURPOSE:     CALL INTERPRETER SUBROUTINE
 ; INPUT:       HL = ROUTINE ADDRESS
 ; NOTES:       ROUTINE MAY BE CALLED FROM MACHINE LANGUAGE
 ;              ANOTHER INTERPRETED SEQUENCE
 ;              STACK DEPTH INCREASED BY 4 BY CALL
MMCALL  LD   A,(HL)       ; GET OPCODE
        INC  HL
        SRL  A
        LD   DE,RENTER    ; LOAD INTERPRETER DUMMY RETURN
MINT0   PUSH DE           ; SAVE DUMMY RETURN
        LD   C,A          ; INDEX TO C
        JR   NC,MINT2     ; JUMP IF NO LOAD WANTED
        EX   DE,HL
        LD   B,0
        LD   HL,MRARGT    ; LOAD SYSTEM ARG TABLE
        BIT  6,A          ; USE USER TABLE?
        JR   Z,MINT1      ; JUMP IS NO
        LD   HL,(UMARGT)
MINT1   ADD  HL,BC        ; INDEX TABLE
        LD   B,(HL)
        CALL MSUCK1       ; CALL SUCK ROUTINE
MINT2   POP  DE           ; DUMMY RETURN TO DE, HL = PC
        LD   A,C          ; GET CALL INDEX BACK
        LD   B,(IY+CBB)   ; RESTORE CLOBBERED REGISTERS
        LD   C,(IY+CBC)
        JR   INTPE        ; JOIN NORMAL UPI DISPATCH SEQU
SPACE
 ; NAME:         SUCK INLINE ARGUMENTS
 ; PURPOSE:      TRANSFER OF INLINE ARGS INTO CONTEXT BLO
 ; INPUT:        B = ARG LOAD MASK (SEE INTERPRETER COMME
 ; OUTPUT:       HL = UPDATED PC
 ; EXPLANATION: THIS ROUTINE IMPLEMENTS A MACRO LOAD INST
 ; IT IS USED BY THE INTERPRETER AS WELL.  A ONE BIT IN T
 ; INLINE LOAD MASK MEANS TRANSFER THE NEXT INLINE BYTE I
 ; A ZERO BIT MEANS 'ADVANCE CONTEXT BLOCK POINTER'
 ; TWO ENTRY POINTS ARE DEFINED, ONE FOR THE SUCK MACRO I
 ; THE OTHER FOR THE INTERPRETER TO USE
 ; SUCK MACRO ENTRY
MSUCK   POP  HL           ; RETURN ADDRESS TO HL
        POP  DE           ; POP OFF PC
  ; BYTE SAVING TRICK *** REPLACE WITH LD HL,REENTRY
        INC  HL           ; ADVANCE TO REENTRY (MINT0)
        PUSH HL
  ; CALL INTO...
MSUCK1  BIT  4,B          ; IX LOAD WANTED?
        JR   Z,MSUCK2     ; MSUCK2 IF NOT
        LD   A,(DE)
        INC  DE
        LD   (IY+CBIXL),A
        LD   A,(DE)
        INC  DE
        LD   (IY+CBIXH),A
MSUCK2  PUSH IY           ; LET HL = IY
        POP  HL
        INC  HL           ; + 4
        INC  HL
        INC  HL
        INC  HL
        RES  4,B          ; KILL IX BIT
; SUCK IN LOOP
MSUCK3  SRL  B
        JR   NC,MSUCK5    ; MSUCK5 IF NOT THIS TIME
        LD   A,(DE)       ; GET INLINE BYTE
        INC  DE
        LD   (HL),A       ; STUFF INTO CB
MSUCK5  INC  HL           ; BUMP CB POINTER
  ; ** THIS CODE ASSUMES THAT STATUS OF 'SRL' IS PRESERVED
        JR   NZ,MSUCK3    ; JUMP BACK IF MORE TO DO
        EX   DE,HL        ; HL = PC
        RET               ; THEN QUIT
EJECT
  ; *****************************
  ; * UPI ROUTINE ADDRESS TABLE *
  ; *****************************
SYSDPT  DW   MINTPC
        DW   MXINTC
        DW   MRCALL
        DW   MMCALL
        DW   MMRET
        DW   MMJUMP
        DW   MSUCK
        DW   MACTIN
        DW   TIMEY
        DW   MUZSET
        DW   MUZSTP
        DW   MSETUP
        DW   MCOLOR
        DW   MFILL
        DW   MPAINT
        DW   MVWRIT
        DW   MWRITR
        DW   MWRITP
        DW   MWRIT
        DW   MWRITA
        DW   MVBLAN
        DW   MBLANK
        DW   MSAVE
        DW   MREST
        DW   MSCROL
        DW   DISPCH
        DW   STRNEW
        DW   BCDISP
        DW   MRELAB
        DW   MRELA1       ; RELAB1
        DW   MVECTC
        DW   MVECT
        DW   MKCTAS
        DW   MENTRY       ; SENTRY
        DW   MDOIT        ; DOIT
        DW   MDOITB
        DW   MPIZBK       ; PIZBRK
        DW   MMENU
        DW   MGETP
        DW   MGETN
        DW   MPAUSE       ; PAUSE
        DW   MDISTI       ; DISPLAY TIME
        DW   MINCSC       ; INC SCORE
        DW   INXNIB       ; INDEXN
        DW   PUTNIB       ; STOREN
        DW   MINDW        ; INDEXW
        DW   MINDB        ; INDEXB
        DW   MMOVE        ; MOVE
        DW   MSHFTU
        DW   BCDAD
        DW   BCDSB
        DW   BCDML
        DW   BCDDV
        DW   BCDCS
        DW   BCDNG
        DW   SDADD
        DW   SDSMG
        DW   SDABS
        DW   SNEGT
        DW   MRANGE
        DW   MQUIT
        DW   MSETB
        DW   MSETW
        DW   MMTD
SPACE 4
 ; MACRO ROUTINES ARGUMENT MASK TABLE
 ; FORMAT:
 ; *********************************
 ; * 7 * 6 * 5 * 4 * 3 * 2 * 1 * 0 *
 ; *********************************
 ; * H * L * A * IX* B * C * D * E *
 ; *********************************
 ; ARGUMENTS MUST FOLLOW THE CALL INDEX IN THE FOLLOWING
 ; (OMITTING UNUSED ARGUMENTS, OF COURSE)
 ; (INDEX), IXL, IXH, E,D,C,B,A,L,H
MRARGT  DB   0            ; INTPC
        DB   0            ; XINTC
        DB   11000000B    ; RCALL
        DB   11000000B    ; MCALL
        DB   0            ; MRET
        DB   11000000B    ; MJUMP
        DB   00001000B    ; SUCK
        DB   0            ; ACTINT
        DB   00000100B    ; DECCTS
        DB   11110000B    ; BMUSIC
        DB   0            ; EMUSIC
        DB   00101010B    ; SETOUT
        DB   11000000B    ; COLSET
        DB   00101111B    ; FILL
        DB   00101111B    ; RECTAN
        DB   11010000B    ; VWRITR
        DB   11100011B    ; WRITR
        DB   11100011B    ; WRITP
        DB   11101111B    ; WRIT
        DB   11101111B    ; WRITA
        DB   00010011B    ; VBLANK
        DB   11001011B    ; BLANK
        DB   11001111B    ; SAVE
        DB   11000011B    ; RESTORE
        DB   11001111B    ; SCROLL
        DB   00100111B    ; NEW DISCHR
        DB   11000111B    ; NEW DISSTR
        DB   11001111B    ; DISNUM
        DB   00100000B    ; RELABS
        DB   00100000B    ; RELAB1
        DB   11010100B    ; VECTC
        DB   11010000B    ; VECT
        DB   0            ; KCTASC
        DB   00000011B    ; SENTRY
        DB   11000000B    ; DOIT
        DB   11000000B    ; DOITB
        DB   0            ; PIZBRK
        DB   11000011B    ; MENU
        DB   11101100B    ; GET PARAMETER
        DB   11001111B    ; GET NUMBER
        DB   00001000B    ; PAUSE
        DB   00000111B    ; DISTIM
        DB   11000000B    ; INCSCR
        DB   11000000B    ; INDEXN
        DB   11000000B    ; STOREN
        DB   11000000B    ; INDEXW
        DB   11000000B    ; INDEXB
        DB   11001111B    ; MOVE
        DB   11001000B    ; SHIFTU
        DB   11001011B    ; BCDADD
        DB   11001011B    ; BCDSUB
        DB   11001011B    ; BCDMUL
        DB   11001011B    ; BCDDIV
        DB   11001000B    ; BCDCHS
        DB   00001011B    ; BCDNEG
        DB   11001011B    ; DADD
        DB   00001011B    ; DSMG
        DB   00001011B    ; DABS
        DB   11001000B    ; NEGT
        DB   00100000B    ; RANGED
        DB   00000000B    ; QUIT
        DB   11100000B    ; SET BYTE
        DB   11000011B    ; SET WORD
        DB   11000111B    ; MASK TO DELTAS
SPACE 4
 ; DOES 4 60TH SEC COUNTERS IN CT0-3
MACTIN  DI                ; MAKE SURE INTERRUPT IS DISABL
        PUSH AF
        PUSH BC
        PUSH DE
        PUSH HL
        IM   2
        LD   A,ITAB SHR 8 
        LD   I,A
        LD   A,200
        OUT  (INLIN),A
        LD   A,ITAB & 0FFH
        OUT  (INFBK),A
        CALL TIMEZ        ; UPDATE TIMEOUT,MUSIC AND SECON
        LD   C,0FH        ; USE CT0-3
        CALL TIMEY        ; DEC CT0-3
        POP  HL
        POP  DE
        POP  BC
        POP  AF
        EI
        RET
SPACE 4
; ROUTINE: SENTRY
; PURPOSE: TO WAIT FOR CHANGE OF PROGRAM STATUS
; IN EITHER THE PORTS OR THE TIMER-COUNTERS.
; IN ADDITION IT CHECKS TIMOUT FOR LONG, PERIODS OF IN-
; ACTIVITY.
; ** IS VECTOR OUT FLAG SET??
MENTRY  LD   A,(SENFLG)
        CP   0AAH
        JP   Z,2019H      ; YES - JUMP OUT
        LD   A,(TIMOUT)   ; CHECK IF TIME TO BLAKOUT
        OR   A
        JR   NZ,TTEST
MPIZBK  XOR  A            ; TIME TO SHUT DOWN
        DI
        OUT  (VOLC),A     ; TURN OFF SOUNDS
        OUT  (VOLAB),A
        LD   BC,COLBX+8*256
        OUT  (C),A        ; PAINT IT BLACK
        DJNZ ($-2)
PBLP    LD   DE,AKEYS
        CALL FINDL3       ; CALL STORE DE INTO CONTEXT RO
        CALL TTEST        ; WAIT FOR SOMETHING TO HAPPEN
        INC  A
        JR   NZ,MPIZBK
        LD   (IY+CBA),0
        EI
        LD   HL,(COLLST)  ; GET SAVED COLORS
MCOLOR  LD   (COLLST),HL  ; SAVE COLORS FOR FUTURE
        LD   BC,800H+COLBX
        OTIR              ; RESET THE COLORS
        XOR  A
        RET
TTEST   CALL TRCHK
        LD   (IY+CBA),A
        LD   (IY+CBB),B
        CP   SKYD
        RET  C
        CP   POT0
        RET  NC
        LD   A,$FF
        LD   (TIMOUT),A
        RET
SPACE 2
CALCL   DW   SCBL
        DW   PNCALC
        DW   CALCST       ; START OF CALCULATOR
SPACE 4
  ; SYSTEM ROUTINES JUMP VECTOR
;        ORG  200H
        JP   TIMEZ        ; DO TIMER & MUSIC
        JP   TIMEX        ; DECTMR

SYSFNT  DB   20H
        DB   8
        DB   8
        DB   1
        DB   7
        DW   LRGCHR

SMLFNT  DB   0A0H
        DB   4
        DB   6
        DB   1
        DB   5
        DW   SMLCHR

; ALL KEYS MASK
AKEYS   DB   3FH
        DB   3FH
        DB   3FH
        DB   3FH

; HEAD OF ONBOARD MENU
GUNLNK  DW   CML
        DW   PNGF
        DW   GFSTRT
        DEFM 'MAX SCORE'
        DB   0
        DEFM '# OF PLAYERS'
        DB   0
        DEFM '# OF GAMES'
        DB   0
EJECT
; NAME:         CONVERT MASK TO DELTAS
; INPUT:        B = JOYSTICK MASK
;               C = FLOP STATUS (MR FLOP BIT SET IF FLOP
;               DE = X POSITIVE DELTA 
;               HL = Y POSITIVE DELTA
MMTD    CALL CONCPL       ; HANDLE Y
        EX   DE,HL
        BIT  MRFLOP,C      ; FLOP SET?
        JR   Z,MMTD2       ; YES - DOIT
        LD   A,B           ; NO - GET MASK
        AND  3
        JR   Z,MMTD1       ;INVERT IF NOT ZERO
        CPL
MMTD1   LD   B,A   
MMTD2   CALL CONCPL        ; PROCESS X
        EX   DE,HL
        JP   STHLDE        ; STORE HL,DE AND QUIT
SPACE 2
; SUBROUTINE TO CONDITIONALLY COMPLEMENT OR ZERO HL
CONCPL  RRC  B
        JR   NC,CONC1      ; JUMP IF NOT UP
        LD   A,L
        CPL
        LD   L,A
        LD   A,H
        CPL
        LD   H,A
        INC  HL
        RRC  B
        RET
CONC1   RRC  B            ; DOWN SET?
        RET  C            ; QUIT IF SO
        JP   CONC2        ; JUMP TO ZERO OUT
SPACE 2
; NAME:         SCROLL MEMORY BLOCK
; INPUT:        B  = NUMBER OF LINES TO SCROLL
;               C  = NUMBER OF BYTES ON LINE TO SCROLL
;               DE = LINE INCREMENT
;               HL = FIRST LINE TO SCROLL
MSCROL  XOR  A
MSCRL1  PUSH BC           ; SAVE COUNTERS
        PUSH DE
        LD   B,A
        EX   DE,HL
        ADD  HL,DE        ; ADD INCREMENT TO LINE
        PUSH HL
        LDIR
        POP  HL
        POP  DE
        POP  BC
        DJNZ MSCRL1
        RET
SPACE 2
; NAME:         MACRO INTERPRETER EXIT WITH CONTEXT REST
; PURPOSE:      QUIT INTERPRETING AND GO HOME
MXINTC: POP  HL           ; THROW OUT DUMMY RETURN
; NAME:         RETURN FROM SYSTEM CALL
; PURPOSE:      RETURNING TO USER AND RESTORATION OF REG
RETN_   POP  HL           ; RETURN ADDRESS TO HL
        POP  IY
        POP  IX
        POP  DE
        POP  BC
        POP  AF
        EX   (SP),HL       ;STK=RETURN, HL=OLD HL
        RET
SPACE 2
; NAME:         BCD DIVIDE
;
BCDDV   CALL GNACC         ; GENERATE ACCUMULATOR
        EX   (SP),HL       ; HL = ACC, TOP = ARG2
        PUSH BC
        LD   B,0
        LD   A,C
        SRL  C
        ADD  HL,BC
        LD   C,A
        EX   DE,HL         ; HL = ARG1, DE = ACC
        LDIR               ; HL=ARG1 FLAG+1
        POP  BC
        POP  DE
        DEC  HL            ;
        EX   (SP),HL       ; HL =ARG2, TOP = ARG1 FLAG
        PUSH BC
        LD   B,0
        ADD  HL,BC         ; HL = ACC+SIZE/2
        POP  BC
        DEC  C             ; DECREMENT SIZE
        EX   DE,HL         ; HL = ARG2, DE=ACC, TOP=ARG
        DEC  DE
DIV1    DEC  DE
        XOR  A
        SYSTEM NEGT        ; ARG2 = -ARG2 (10S COMP)
DIV2    SYSTEM DADD        ; SUBTRACT UNTIL BORROW
        JR   C,DIV3
        INC  A             ; OR UNTIL LOOP COUNT > 99
        DAA
        JR   NZ,DIV2
        POP  HL
        LD   (HL),$FF
        POP  BC
        JR   MULT6
DIV3    SYSTEM NEGT
        SYSTEM DADD
        EX   (SP),HL       ; HL = ARG1
        DEC  HL
        LD   (HL),A        ; SAVE ANSWER IN ARG1
        EX   (SP),HL
        DEC  C
        JR   NZ,DIV1
        POP  HL
        POP  BC
        JR   DIV4
; SUBROUTINE TO GENERATE ACCUMULATOR ON THE STACK
GNACC   POP  IX
        XOR  A
        LD   C,A
        SYSTEM DABS        ; ARG1= ABS VALUE
        EX   DE,HL
        SYSTEM DABS        ; ARG2= ABS VALUE
        EX   DE,HL      ; FLAG=1 IF NEG ANS, ELSE POS
        LD   H,A
        LD   L,A
        LD   A,B
MULT1   PUSH HL            ; GENERATE ACC ON STACK
        DJNZ MULT1
        LD   B,A           ; RESTORE SIZE
        ADD  HL,SP
        PUSH BC            ; SAVE SIGN
        PUSH HL            ; SAVE STACK POINTER
        PUSH HL            ; SAVE ACC POINTER
        LD   H,(IY+CBH)    ; RESTORE ARG2 POINTER
        LD   L,(IY+CBL)
        LD   C,B
        JP   (IX)
        ; DECIMAL MULTIPLY
        ; GIVEN:    DE>ARG1, HL>ARG2, B=SIZE/2
        ;           (SIZE/2-1 ASSUMED EVEN)
        ; RETURNED: ARG1=ANSWER, C>0 ON OVERFLOW
        ;
        ;
BCDML   CALL GNACC         ; GENERATE ACCUM
MULT2   LD   A,(HL)        ; A=MULT LOOP COUNT
        INC  HL
        EX   (SP),HL       ; HL>DEC ACC
        AND  A             ; IF A=0, SKIP MULT LOOP
        JR   Z,MULT4
        EX   DE,HL
MULT3   SYSTEM DADD        ; ELSE MULTIPLY
        AND  A             ; CLEAR THE CARRY BIT
        DEC  A             ; DECIMAL DECREMENT
        DAA
        JR   NZ,MULT3
        EX   DE,HL
MULT4   INC  HL            ; INCREMENT DECIMAL ACC
        EX   (SP),HL       ; HL>ARG2
        DEC  C
        JR   NZ,MULT2
        POP  HL            ; RESTORE STACK POINTER
        POP  HL            ; RESTORE SIGN
        POP  BC
        PUSH DE
        PUSH BC
        LD   C,B
        LD   B,0
        SRL  C
        ADD  HL,BC
        SLA  C
        LDIR
        POP  BC
        PUSH BC            ; CHECK FOR OVERFLOW
        SRL  B
        XOR  A
MULT5   OR   (HL)
        INC  HL
        DJNZ MULT5
        AND  A             ; SET FLAGS
        JR   Z,MULT7
        LD   A,$FF
        LD   (DE),A
MULT7   POP  BC            ; CHECK SIGN AND
        POP  HL
DIV4    BIT  0,C           ; NEGATE ARG1 IF NECESSARY
        JR   Z,MULT6
        SYSTEM BCDCHS
MULT6   POP  HL         ; RESTORE ORIGINAL STACK POINTER
        DJNZ MULT6
        RET
        ; BCD SUBTRACT & ADD
        ;
        ; GIVEN:    DE>ARG1, HL>ARG2
        ;           B=SIZE/2+1
        ; RETURNED: ARG1=ANSWER
BCDSB   SYSTEM BCDCHS
BCDAD   SYSTEM BCDNEG
        EX   DE,HL
        SYSTEM BCDNEG
        EX   DE,HL
        SYSTEM DADD
; AND FALL INTO
        ;
        ;
        ; DECIMAL SIGNED MAGNITUDE
        ;
        ; GIVEN:    DE>ARG (10'S COMPLEMENT)
        ;           B=SIZE/2+1
        ; RETURN:   ARG (SIGNED MAGNITUDE)
        ;
SDSMG   LD   L,B           ; HL>ARG3-1 (SIGN BYTE)
        DEC  L
        LD   H,0
        ADD  HL,DE
        LD   A,(HL)        ; IF POS (SIGN NIBBLE<5)
        CP   $50
        RET  C             ; EXIT
        EX   DE,HL
SDSMG1  LD   A,0           ; ELSE 10'S COMPLIMENT
        SBC  A,(HL)
        DAA
        LD   (HL),A
        INC  HL
        DJNZ SDSMG1
        DEC  HL            ; AND SET SIGN BIT
        LD   A,(HL)
        OR   $80
        LD   (HL),A
        RET
        ;
        ;
        ; BCD NEGATE
        ;
        ; GIVEN:    DE>ARG (SIGNED MAGNITUDE)
        ;           B=SIZE/2+1
        ; RETURNED: ARG (10'S COMPLEMENT)
        ;
BCDNG   LD   L,B           ; HL>ARG+B-1 (SIGN BYTE)
        DEC  L
        LD   H,0
        ADD  HL,DE
        BIT  7,(HL)        ; EXIT IF POS
        RET  Z
        LD   (HL),0        ; CLEAR SIGN BYTE
        EX   DE,HL
SNEGT   XOR  A             ; CLEAR CARRY
BCDNG1  LD   A,0           ; ELSE 10'S COMPLEMENT
        SBC  A,(HL)
        DAA
        LD   (HL),A
        INC  HL
        DJNZ BCDNG1
        RET
        ;
        ;
        ; DECIMAL ABSOLUTE
        ;
        ; GIVEN:    DE>ARG (SIGNED MAGNITUDE)
        ;           B=SIZE/2+1
        ; RETURNED: C=C+1 IF SIGN BIT CLEARED
        ;
SDABS   LD   L,B
        LD   H,0
        DEC  L
        ADD  HL,DE
        BIT  7,(HL)
        RET  Z
        LD   (HL),0
        INC  (IY+CBC)
        RET
        ;
        ;
        ; BCD CHANGE SIGN
        ;
        ; GIVEN:    HL>ARG B=SIZE/2+1
        ;           (SIGNED MAGNITUDE)
        ; RETURNED: ARG SIGN BIT COMPLEMENTED
        ;
BCDCS   LD   C,B
        LD   B,0
        DEC  C
        ADD  HL,BC
        LD   A,(HL)
        XOR  $80
 ; NAME:            SET BYTE
MSETB   LD   (HL),A
        RET
        ;
        ;
        ; DECIMAL ADD
        ;
        ;
        ; GIVEN:    DE>ARG1  HL>ARG2 (10'S COMPLEMENT)
        ;           B=SIZE/2+1
        ; RETURNED: ARG1=ANSWER (10'S COMPLEMENT)
        ;
SDADD   XOR  A
SDADD1  LD   A,(DE)
        ADC  A,(HL)
        DAA
        LD   (DE),A
        INC  DE
        INC  HL
        DJNZ SDADD1
        CP   99H
        RLA
        CPL
        LD   (IY+CBFLAG),A ; SEND BACK STATUS FROM DADD
        RET
SPACE 2
; NAME:             RANGED RANDOM NUMBER
; INPUT:            A = RANGE
; OUTPUT:           A = RANDOM NUMBER (0 TO RANGE-1)
MRANGE  PUSH AF
        LD   HL,(RANSHT)
        CALL SHIFTR
        LD   BC,23
        ADD  HL,BC
        ADC  A,D
        LD   (RANSHT),HL
        LD   HL,(RANSHT+2)
        LD   E,A
        CALL SHIFTR
        ADD  HL,DE
        LD   (RANSHT+2),HL
        LD   E,D
        EX   DE,HL
        POP  AF
        AND  A
        LD   C,A
        LD   A,D
        JR   Z,R3
        XOR  A
R1      ADD  HL,DE
        JR   NC,R2
        INC  A
R2      DEC  C
        JR   NZ,R1
R3      JP   QFROG
SHIFTR  LD   B,H
        LD   C,L
        XOR  A
        LD   D,7
SH1     ADD  HL,HL
        RLA
        DEC  D
        JR   NZ,SH1
        ADD  HL,BC
        ADC  A,D
        RET
SPACE 2
; NAME:          SAVE AREA
; INPUT:         HL = SCREEN ADDRESS
;                DE = SAVE AREA ADDRESS
;                BC = Y,X SIZE OF AREA TO SAVE
; NOTES:         THE SIZES OF THE OBJECT ARE SAVED IN THE
;                FIRST TWO BYTES OF THE SAVE AREA
MSAVE   EX   DE,HL
        LD   (HL),C        ; SET X SIZE
        INC  HL
        LD   (HL),B        ; SET Y SIZE
        INC  HL
        XOR  A
        EX   DE,HL
        SET  6,H           ; SET NON-MAGIC ADDRESS
MSAVE1  PUSH BC
        PUSH HL
        LD   B,A
        LDIR
        POP  HL
        LD   C,BYTEPL
        ADD  HL,BC
        POP  BC
        DJNZ MSAVE1
        RET
SPACE 2
; NAME:          PREGAME OUTPUT PORT SETUP
; PURPOSE:       TO SET CONCOM, VERBL, ETC
; INPUTS:        B=HORCB,  D=VERBL, A=INMOD
MSETUP: LD   C,HORCB       ; GET BASE PORT NUMBER
        OUT  (C),B         ; HORBL
        INC  C             ;
        OUT  (C),D         ; VERBL
        OUT  (INMOD),A
        RET
SPACE 2
; NAME:          TEST FOR TRANSITIONS
; FUNCTION:      TO LOOK FOR CHANGES IN THE PORTS %TC.
; RETURNS:       A=0 NO CHANGE
; 1-8 COUNTER TIMER #N HIT 0
; 9-C = POT0-3 CHANGED
; D = A SECONDS UP
; E= KEYBOARD CHANGED (B=0-24)
; F-16 : TRIG0!JOY0 - T3!J3
; RETURNS NEW VALUE IN B
CTLP    LD   E,(HL)
        LD   BC,$801
CCTLP   LD   A,C          ; GET MASK
        RRCA
        LD   C,A
        AND  E            ; CHECK IF CT BIT =1
        JR   NZ,CCT1
        DJNZ CCTLP
        RET
CCT1    XOR  E            ; MASK OUT BIT IN QUESTION
        LD   (HL),A       ; PUT BACK THE CTFLAGS OR SEMI4
        LD   A,B
        ADD  A,D
        POP  HL           ; OLD RETURN ADDRESS
        RET
TRCHK   JR   Z,TSEX       ; SKIP COUNTER-TIMERS AND POTS?
        LD   HL,CNT       ; GET COUNTER  TIMERS STATUS
        LD   D,0
        CALL CTLP         ; COUNTER TIMERS
        LD   D,8
        INC  HL
        CALL CTLP         ; SEMI4S
        LD   BC,$400+POT0
TPLOP   INC  HL           ; -> MPOT0
        IN   A,(C)
        LD   E,(HL)       ; GET 0POT
        SUB  E
        JR   C,PHOT       ; NEW LINE LESS THAN OLD
        SUB  PFUG         ; FUDGE BOUNCE FACTOR
        JR   C,EPLOP      ; NEW MORE THAN OLD+4
        INC  A
PHOT    ADD  A,E
        LD   (HL),A
        LD   B,A
        LD   A,C
        RET
EPLOP   INC  C
        DJNZ TPLOP
; NOW TEST SECONDS
TSEX:   LD   HL,KEYSEX     ; HL = KEYSEX
        LD   A,(HL)
        BIT  7,A
        JR   Z,TKEYS
        RES  7,A
        LD   (HL),A
        LD   A,SSEC        ; SECS
        RET
; NOW TEST KEYBOARD
TKEYS   PUSH HL
        CALL DELOAD
        EX   DE,HL
        LD   BC,$400+KEY3
        LD   DE,$FF00      ; SET BIT COUNTER+COLUMNN
MSK1    IN   A,(C)
        AND  (HL)
        JR   NZ,MSENK2
        DEC  C             ; NEXT PORT
        INC  E             ; AND COLUMN
        INC  HL            ; AND MASK
        DJNZ MSK1
        LD   A,B           ; NOTHING DOWN
        LD   E,SKYU
        JR   MSENKE
MSENK2  INC  D             ; BIT COUNTER
        RRCA
        JR   NC,MSENK2
        LD   A,D
        RLCA               ; KEY=BIT*4
        RLCA
        ADD  A,E           ; + COLUMN
        INC  A             ; PLUS 1
        LD   E,SKYD
MSENKE  POP  HL
        XOR  (HL)          ; KEY=0KEY?
        AND  $7F
        JR   Z,HANDLE
        XOR  (HL)
        LD   (HL),A
        AND  $7F
        LD   B,A
        LD   A,E           ; KEYBOARD RETURN CODE
        RET
; NOW TEST HANDLES
HANDLE  LD   BC,$400+SW0
SWLOP   INC  HL            ; -> OSW0
        IN   A,(C)
        XOR  (HL)          ; COMPARE THE TWO
        JR   NZ,SWHIT      ; NO CHANGE
        INC  C
        DJNZ SWLOP
        LD   A,B           ; RETURN 0
        RET
SWHIT   BIT  4,A           ; TEST TRIGGER
        JR   Z,JOYS        ; NO TRIG MUST BE JOYSTICK
        AND  $10           ; FILTER OUT TRIGGER
        XOR  (HL)          ; UPDATE VALUE
        LD   (HL),A
        AND   $10
        LD    B,A
        LD    A,C          ; GET PORT NUMBER
        RLCA               ; *2
        SUB   $C
        RET
JOYS    XOR   (HL)
        LD    (HL),A      ; NO CHANGE IN TRIG SO STORE ST
        AND   $F          ; TAKE OFF TRIGGER
        LD    B,A
        LD    A,C
        RLCA              ; *2
        SUB   $B
        RET
SPACE 2
; TIMEX
; INPUTS HL-> TIME BASE IN RAM
; B=TIME BASE MODULUS
; C=MASK AS IN DECCTS
; PURPOSE: TO DECR TIMEBASE AND IF 0 RESET IF AND DECR
;  COUNTER      TIMERS
TIMEX   DEC  (HL)          ; DEC TIMEBASE
        RET  NZ
        LD   (HL),B        ; RESET TIMEBASE
SPACE 2
; NAME: DECREMENT COUNTER        TIMERS
; INPUTS: C=MASK
; USED BY ACTINT AND DECCTS TO DECREMENTS CTS UNDER MASK
; MASK = *76543210* , IF BIT=1 THEN DEC CORRESPONDING
; CT# , IF BIT=0 LEAVE CT# ALONE
; NOTE: ALL COUNTERS ARE RUN IN BCD FOR EASY DISPLAY
TIMEY   LD   B,8          ; NO OF BITS
        LD   HL,CT0       ; -> TO COUNTER TIMERS
        LD   D,0          ; RESULTS
TIMLP   SRL  C            ; CHANGE THIS TIMER?
        JR   NC,ETLP
        LD   A,(HL)       ; GET THE TIMER
        OR   A            ; IS IT ZERO ALREADY?
        JR   Z,ETLP
        DEC  A
        DAA
        JR   NZ,($+3)
        SCF
        LD   (HL),A       ; STORE NEW VALUE
ETLP    INC  HL
        RR   D            ; ROTATES IN CARRY FLAG
        DJNZ TIMLP
        LD   A,(CNT)    ; COUNTER UPDATE & NUMBER TRACKER
        OR   D
        LD   (CNT),A
        RET
SPACE 2
; NAME: TIMER ROUTINE
; PURPOSE: TO UPDATE GAME TIME, TIMEOUT AND MUSIC
; INPUTS:      OUTPUTS: NONE
; NOTE: PUSH YOUR REGISTERS (AF,BC,DE,HL)
TIMEZ 
        LD   HL,PRIOR     ; ASSUMES YOU PUSH DA REGS
        BIT  1,(HL)       ; CHECK IF TICKS OVERRUN
        RET  NZ
        SET  1,(HL)
        EX   DE,HL
  ;  *SIXTIETH OF A SECOND INTERRUPT*
        LD   HL,DURAT     ; NOTE TIMER
        LD   A,(HL)       ; =0 SKIP
        OR   A
        JR   Z,SIXY
        DEC  (HL)
        JR   NZ,STAK0
        PUSH HL
        PUSH IX
        CALL MUZCPU       ; =0 DO NEXT NOTE
        POP  IX
        POP  HL
        JR   SIXY
STAK0   EX   DE,HL
        BIT  7,(HL)
        EX   DE,HL
        JR   NZ,SIXY
        DEC  A
        DEC  A            ; =1 QUIET NOTE
        JR   NZ,SIXY
; A=0
        OUT  (VOLAB),A
        OUT  (VOLC),A
SIXY    INC  HL
        DEC  (HL)         ; IF(--TMR60<0)
        JP   P,GOUT       ; ELZ ONWARD
        LD   (HL),59      ; THEN TMR60=59
        INC  HL           ; -> TIMOUT
        EX   DE,HL
        LD   HL,KEYSEX    ; SET SECONDS UP
        SET  7,(HL)
        EX   DE,HL
        LD   A,(HL)       ; CHECK IF ZERO
        OR   A
        JR   Z,GTIMER
        DEC  (HL)         ; DEC TIMOUT
  ; *GAME TIMER ONCE A SECOND ROUTINE*
  ; IF (SEC != 0 & MIN !=0)
  ;    IF (SEC ==0)
  ;       SEC=59;--MIN
  ;    ELSE --SEC
  ; ELSE GAMETIMEUP=1
GTIMER  INC  HL            ; ->GTSECS
        LD   A,(HL)        ; IF (SEC=0
        INC  HL            ; ->GTMINS
        OR   (HL)          ; & MIN!=0
        JR   Z,GT02
        DEC  HL            ; ->GTSECS AGAIN
        LD   A,(HL)        ; IF (SEC ==0)
        OR   A
        JR   NZ,GT01
        LD   (HL),$59      ; THEN SEC=59BCD
        INC  HL            ; ->GTMINS AGAIN
        LD   A,(HL)        ; --MIN
        DEC  A
        DAA
        LD   (HL),A
        JR   GOUT
GT01    DEC  A             ; ELSE --SEC
        DAA
        LD   (HL),A
        JR   GOUT
GT02    LD   HL,GAMSTB     ; ELSE GAMETIMEUP=1
        BIT  GSBTIM,(HL)
        JR   Z,GOUT
        SET  GSBEND,(HL)
GOUT    LD   HL,PRIOR
        RES  1,(HL)
        RET               ; RETURN TO BAKGRND OR LO LEVEL
 ;
 ; NAME: START MUZCPU
 ; PURPOSE: TO START MUSIC PLAYING (ALSO NOISES)
 ; INPUTS: HL -> SCORE
 ; A=VOICES
 ; NOTE: YOU SHOULD LOAD MUZSP IF YOU DO CALLS
MUZSET  LD   (VOICES),A
        LD   (MUZSP),IX
        CALL MUZSTP
        JR   MUZCP1
 ; NAME: MUZCPU
 ; PURPOSE: PLAYING MUSIC AND NOISES
 ; NOTE: DURAT=0 WHEN CALLED
 ; OUTPUT: NONE
  ; *MUSIC PROCESSOR*
  ;  FETCH OPCODE
  ;  IF (OPCODE < 80H)
  ;      SET NOTE DURATION ETC
  ;  ELSE
  ;  SWITCH (OPCODE < 80H)
  ;  CASE 80H:
  ;      IF (MASK=8  STUFF SNDBK PC=PC+9
  ;      ELSE OUTPUT(MASK)=DATA
  ;  CASE 90H:
  ;      VOICES=DATA
  ;  CASE A0H:
  ;      (--SP)=DATA IN NIBBLE OF OP + 1
  ;  CASE B0H:
  ;       SET VOLUMES = DATA,DATA
  ;  CASE C0H:
  ;      SWITCH (MASK)
  ;          CASE 9: MPCL=(MSP++); MPCH=(MSP++); BREAK
  ;          CASE D: (--MSP)=MPCH; (--MSP)=MPCL
  ;          CASE 0: IF --(SP)==0; THEN SP++
  ;          CASE 3: MPC=DATA16
  ;  CASE D0H: CALL RELATIVE
  ;  CASE E0: DURAT=DATA
  ;  CASE F0: VOICES=0,PORTS=0
MUZCPU  LD   HL,(MUZPC)   ; LOOK LIKE NORMAL LOOP RETURN
MUZCP1  LD   IX,(MUZSP)   ; FETCH STACK POINTER
OPLOOP  LD   A,(HL)       ; OPCODE FETCH
        INC  HL           ;->OPERAND,DATA
        OR   A            ; TEST FOR 80H OR MORE
        JP   M,MO0
   ; NORMAL NOTE OPERATOR 
        LD   (DURAT),A
        LD   A,(VOICES)
        LD   BC,800H+SNDBX
        SRL  A            ; SET NOISE
        JR   NC,($+4)
        OUTI
        LD   B,5          ; -> VIBRATO
        SRL  A
        JR   NC,($+4)
        OUTI              ; SET VIBRATO
        LD   B,4          ; -> NOTEC
M81:    SRL  A            ; CHECK C,B,A
        JR   NC,M82
        OUTI
M815    SRL  A            ; CHECK IF INC PC WAS ON
        JR   C,M83
        DEC  HL           ; RESTORE PC
        JR   M83
M82     DEC  B
        INC  HL
        JR   M815
M83     OR   A
        JR   NZ,M81
; PLAY NOTE
        LD   A,(PVOLAB)
        OUT  (VOLAB),A
        LD   A,(PVOLMC)
        OUT  (VOLC),A
        JP   MUZ999
MO0:    CP   90H
        JR   NC,MO1
   ; STUFF PORT OR SOUND BLOCK
        BIT  3,A          ; IF (STUFF SNDBLK)
        JR   Z,MO01
        LD   A,B          ; SAVE B (VSN)
        LD   BC,8*256+SNDBX ; B=8 C=SNDBX
        OTIR              ; HL->NEXT OPCODE WHEN DONE
        JR   OPLOOP
MO01    AND  7            ; ISOLATE PORT NUMBER
        OR   10H          ; PORTS 10H-17H
        LD   C,A          ; SET PORT REGISTER
        OUTI
        JR   OPLOOP
MO1:    JR   NZ,MO2
        LD   A,(HL)       ; GET NEW VOICES
        INC  HL
        LD   (VOICES),A
        JR   OPLOOP
MO2:    CP   0B0H
        JR   NC,MO3
        AND  0FH
        LD   E,A
        INC  E
        JR   MO45
MO3:    CP   0C0H         ; SET VOL ETC
        JR   NC,MO4
; LOAD PVOLS
        LD   DE,PVOLAB
        LDI               ; DONT CARE ABOUT BC
        LDI
OPLP2   JR   OPLOOP
MO4     JR   NZ,MO40
        DEC  (IX+0)       ; DEC STACK TOP
        JR   NZ,MO41
        INC  IX
        INC  HL
        INC  HL
        JR   OPLP2
MO40    CP   0D0H         ; PC SP STUFF
        JR   NC,MO5
MO41    AND  0FH          ; ISOLATE MASK
        CP   9            ; RETURN
        JR   NZ,MO43
        LD   L,(IX+0)
        INC  IX
        LD   H,(IX+0)
        INC  IX
        JR   OPLP2
MO43:   LD   E,(HL)       ; PCL=
        INC  HL
        LD   D,(HL)       ; PCH=
        INC  HL
        EX   DE,HL        ; SET THE PC
        CP   4            ; IS IT A JMP?
        JR   C,OPLP2      ; IT IS
MO44    DEC  IX           ; IT'S A CALL
        LD   (IX+0),D     ; (--SP)=PHL
MO45    DEC  IX
        LD   (IX+0),E     ; (--SP)=PCL
        JR   OPLP2
MO5     CP   0E0H
        JR   NC,M06
        AND  0FH
        LD   B,0
        LD   C,A
        LD   D,H
        LD   E,L
        ADD  HL,BC
        JR   MO44           ; CALL
M06     JR   NZ,M061
        LD   A,(PRIOR)      ; LEGSTA
        XOR  80H
        LD   (PRIOR),A
        JR   OPLP2
M061    CP   0F0H         ; REST VOICE (OR SUSTAIN)
        JR   Z,MUZSTP
        LD   A,(HL)
        LD   (DURAT),A    ; SET DURATION OR QUIET
        INC  HL
        XOR  A
        OUT  (VOLAB),A
        OUT  (VOLC),A
   ; END OF MUSIC PROCESSOR
MUZ999  LD   (MUZPC),HL   ; SAVE THE PC
        LD   (MUZSP),IX   ; SAVE THE STACK POINTER
        RET
 ; NAME MUZSTP
 ; PURPOSE: STOP MUZCPU, SET PORT TO 0
MUZSTP: XOR  A
        LD   (DURAT),A
        LD   (PRIOR),A
        LD   BC,800H+SNDBX
        OUT  (C),A
        DJNZ ($-2)
        RET
;
; NAME: DO IT
; PURPOSE:      TRANSFER CONTROL TO USER STATE TRANSITION
; INPUT:        A = RETURN CODE FROM SENTRY ROUTINE
;       HL = DO IT TABLE ADDRESS
; OUTPUT:
; DESCRIPTION:  THIS ROUTINE IS USED WITH THE SENTRY ROUT
;       IT IS USED FOR DISPATCHING TO A STATE TRANSITION
;       ROUTINE.  THE RETURN CODE FROM SENTRY IS USED TO
;       SEARCH THE DOIT TABLE.  IF A MATCH IS FOUND, CONT
;       TRANSFERRED.  IF NOT MATCH IS FOUND, THE ROUTINE RE
;       THE DOIT TABLE IS MADE UP OF THREE BYTE ENTRYS:
;       BYTE 0 BIT 7: IF SET - DO A MCALL TO THIS HANDLER
;       BYTE 0 BIT 6: IF SET - DO A RCALL TO THIS HANDLER
;       BYTE 0 BITS 5-0: RETURNCODE THIS ROUTINE IS TO PR
;       BYTE 1 AND 2: THE ADDRESS TO TRANSFER TO.
;       THE LIST IS TERMINATED BY A BYTE WHICH IS .GE.  OC
MDOITB: LD   A,B
MDOIT:  PUSH DE
        LD   D,A
MDOIT0: LD   A,(HL)       ; GET RETURN CODE FOR THIS ENTR
        LD   C,A          ; C = CURRENT ENTRY
        CP   0C0H         ; LIST TERMINATOR?
        JR   C,MDOIT1     ; NO - JUMP
        POP  DE           ; YES  RETURN
        RET
MDOIT1: INC  HL
        AND  3FH
        CP   D            ; NORMAL MATCH?
        JR   Z,MDOIT2     ; JUMP IF SO
MDO1A:  INC  HL           ; NO MATCH - SKIP OVER
        INC  HL           ; GO TO ADDRESS
        JR   MDOIT0
MDOIT2: POP  DE
MDOIT3: LD   E,(HL)       ; DE = GOTO ADDR
        INC  HL
        LD   D,(HL)
        EX   DE,HL
        BIT  7,C          ; MCALL?
        JP   NZ,MMCALL    ; JUMP IF SO
        BIT  6,C          ; RCALL?
        JR   NZ,MRCALL
        POP  DE           ; MUST BE JUMP
        POP  AF
        PUSH HL
        EX   DE,HL
; RCALL ROUTINE
MRCALL: JP   (HL)
 ; **********************
 ; * VECTORING ROUTINES *
 ; **********************
 ; NAME:        VECTOR X AND Y COORDINATES
 ; PURPOSE:     UPDATE X,Y COORDINATES AND LIMIT CHECK
 ; INPUT:       IX = VECTOR PACKET
 ;              HL = LIMITS TABLE
 ; OUTPUT:      C = TIME BASE USED
 ;              NONZERO STATUS SET IF OBJECT MOVED
 ; NOTES:
 ;   THIS ROUTINE WORKS WITH A 'VECTOR PACKET', WHICH LOO
 ; ********************************
 ; *BYTE*    CONTENTS    * NAME   *
 ; ********************************
 ; * 00 * MAGIC REGISTER * VBMR   *
 ; ********************************
 ; * 01 * VECTOR STATUS  * VBSTAT *
 ; ********************************
 ; * 02 * TIME BASE      * VBTIMB *
 ; ********************************
 ; * 03 * DELTA X        * VBDXL  *
 ; * 04 *                * VBDXH  *
 ; ********************************
 ; * 05 * X COORDINATE   * VBXL   *
 ; * 06 *                * VBXH   *
 ; ********************************
 ; * 07 * X CHECKS MASK  * VBXCHK *
 ; ********************************
 ; * 08 * DELTA Y        * VBDYL  *
 ; * 09 *                * VBDYH  *
 ; ********************************
 ; * 0A * Y COORDINATE   * VBYL   *
 ; * OB *                * VBYH   *
 ; ********************************
 ; * 0C * Y CHECKS MASK  * VBYCHK *
 ; ********************************
 ;
 ; OPTIONS BYTE:
 ; BIT  MEANING
 ; ---  -------
 ;  7   VECTOR IS ACTIVE
 ;
 ; CHECKS BYTE:
 ; BIT  MEANING
 ; ---  -------
 ;  0   DO LIMIT CHECKS
 ;  1   REVERSE COORDINATES ON LIMIT ATTAINMENT
 ;  3   TARGET ATTAINED (OUTPUT)
 ;   IF THE VECTOR IS ACTIVE, AND THE TIME BASE IS NONZER
 ; THEN UPDATE COORDINATE ROUTINE IS CALLED FOR THE X
 ; AND Y PORTIONS OF THE PACKET.
MVECT   SET  PSWZRO,(IY+CBFLAG) ; SET ZERO FLAG
        BIT  VBSACT,(IX+VBSTAT) ; IS VECTOR ACTIVE?
        LD   C,(IX+VBTIMB)      ; TIME BASE TO C
        LD   (IX+VBTIMB),0      ; ZERO TIME BASE
        LD   (IY+CBC),C         ; PASS BACK TIME BASE
        RET  Z
        LD   A,C
        AND  A                  ; IS TIME BASE ZERO?
        RET  Z                  ; QUIT IF SO
        LD   DE,VBDXL           ; ADVANCE TO FIRST
        ADD  IX,DE
        CALL MVECTC             ; UPDATE FIRST COORDINATE
        LD   DE,VBDYL-VBDXL     ; TO Y
        ADD  IX,DE
; AND FALL INTO...
 ; NAME:        VECTOR COORDINATE
 ; PURPOSE:     UPDATE OF SINGLE COORDINATE
 ; INPUT:       IX = POINTER TO L.O. DELTA BYTE OF VECTOR
 ;              C = TIME BASE
 ;              HL = LIMITS PACKET (IF USED)
 ; OUTPUT:      NONZERO STATUS SET IF MOTION IS OCCURED
 ;              (SHOULD BE SET ON CALL, SINCE IT IS NOT S
 ; NOTES:
 ;   THIS ROUTINE OPERATES ON A SUBSET OF THE VECTOR PACK
 ; (BETWEEN L.O. DELTA BYTE AND CHECKS BYTE).
 ;   THE DELTA IS ADDED TO THE COORDINATE TIME-BASE TIMES
 ; IF OPTIONED, LIMIT CHECKING IS DONE.  IF HE CHECK FAI
 ; THE COORDINATE IS SET TO THE LIMIT.
 ; WHEN THIS HAPPENS, THE LIMIT ATTAINED BIT IS SET
MVECTC  PUSH HL
        LD   D,(IX+VBDCH) ; LOAD DELTA
        LD   E,(IX+VBDCL)
        LD   H,(IX+VBCH)  ; LOAD COORDINATE
        LD   L,(IX+VBCL)
        LD   A,H          ; SAVE OLD COORDINATE FOR MOTIO
        LD   B,C
MVECT1  ADD  HL,DE        ; ADD DELTA TO COORDINATE
        DJNZ MVECT1       ; TIME-BASE TIMES
  ; HAS MOTION OCCURRED?
        CP   H
        JR   Z,MVCT1A     ; JUMP TO SKIP TEST IF SO
        RES  PSWZRO,(IY+CBFLAG) ; SET MOVE STATUS
  ; IS LIMIT CHECK WANTED?
MVCT1A  BIT  VBCLMT,(IX+VBCCHK)
        JR   Z,MVECT6     ; MVECT6 IF NOT
  ; PERFORM LIMIT CHECK
        LD   A,H
        EX   (SP),HL
        LD   B,(HL)       ; LIMIT TO B
        INC  HL
  ; HANDLE SLIGHTLY LESS THAN ZERO CASE
        CP   207             ; MIDPOINT BETWEEN 160 AND 0
        JR   NC,MVECT2       ; JUMP TO FAIL IF >207
        CP   B               ; DO COMPARE
        JR   C,MVECT2        ; JUMP ON FAIL
        LD   B,(HL)          ; UPPER LIMIT CHECK
        CP   B
        JR   C,MVECT3        ; JUMP ON PASS
MVECT2  INC  HL
; A LIMIT WAS EXCEEDED - SET COORDINATE AT LIMIT
        LD   (IX+VBCH),B
        LD   (IX+VBCL),0
        SET  VBCLAT,(IX+VBCCHK) ; SET LIMIT ATTAINED
; IS REVERSE DELTA SET
        POP  AF           ; CLEAN UP STACK
        BIT  VBCREV,(IX+VBCCHK)
        RET  Z            ; QUIT IF NOT
; REVERSE THE BIMBO
        LD   A,D
        CPL
        LD   D,A
        LD   A,E
        CPL
        LD   E,A
        INC  DE
        LD   (IX+VBDCL),E ; STORE STACK
        LD   (IX+VBDCH),D
        RET
MVECT3  INC  HL           ; STEP PAST LIMIT
        EX   (SP),HL      ; HL = COORDINATE AGAIN
MVECT6  LD   (IX+VBCL),L  ; STORE BACK COORDINATES
        LD   (IX+VBCH),H
        POP  HL           ; RESTORE LIMITS POINTER
        RES  VBCLAT,(IX+VBCCHK) ; CLEAR ATTAINED BIT
        RET
;
; ***************************
; * PAINT RECTANGLE ROUTINE *
; ***************************
; NAME:         PAINT RECTANGLE
; INPUT:        A = COLOR MASK TO WRITE
;               B = Y SIZE
;               C = X SIZE
;               D = Y COORDINATE
;               E = X COORDINATE
MPAINT  XOR  A
        CALL RELTA1
        EX   DE,HL
        SET  6,H          ; UNMAGIC THE ADDRESS
        OUT  (MAGIC),A
;        XOR  A
        LD   E,(IY+CBA)
        LD   A,C
        RRCA
        RRCA
        AND  3FH
        INC  A
        LD   D,A
MPT1    DEC  D
        JR   Z,MPT2
        LD   A,0FFH
        CALL STRIPE
        JR   MPT1
MPT2    LD   A,C
        AND  03H
        INC  A
        LD   C,A
        XOR  A
MPT3    DEC  C
        JR   Z,MPT4
        RRCA
        RRCA
        ADD  A,11000000B
        JR   MPT3
MPT4    CALL STRIPE
        XOR  A
; AND FALL INTO...
; STRIPE PAINTER
; HL = ADDRESS OF STRIPE A = DATA E = MASK B = ITERATIONS
; OUT HL=HL+1 A = CLOBBERED
STRIPE  PUSH HL
        PUSH BC
        LD   (WASTE),A
        LD   A,(WASTE+4000H)
        LD   C,A
STRP1   LD   A,E
        XOR  (HL)
        AND  C
        XOR  (HL)
        LD   (HL),A
        LD   A,L
        ADD  A,BYTEPL
        LD   L,A
        LD   A,H
        ADC  A,0
        LD   H,A
        DJNZ STRP1
        POP  BC
        POP  HL
        INC  HL
        RET
;
 ; ******************
 ; * WRITE ROUTINES *
 ; ******************
 ; NOTES:        THE GENERAL CALLING SEQUENCE FOR THE WRI
 ; INPUT:        HL = PATTERN ADDRESS
 ;               D = Y COORDINATE
 ;               E = X COORDINATE
 ;               B = Y SIZE
 ;               C = X SIZE
 ;               A = MAGIC REGISTER
 ; OUTPUT:       DE = SCREEN ADDRESS USED
 ;                 THESE ROUTINES ARE NESTED, FOR EXAMPLE
 ;               WRITP, WHICH FALLS INTO WRIT, WHICH FALL
 ; ENTRY:        WRITE FROM VECTOR
 ; INPUT:        HL = PATTERN ADDRESS
 ;               IX = VECTOR ADDRESS
 ; OUTPUT:       DE,A
 ; SIDE EFFECTS: BLANK BIT SET IN VECTOR STATUS BYTE
MVWRIT  LD   A,(IX+VBMR)  ; LOAD MR
        LD   D,(IX+VBYH)  ; LOAD Y
        LD   E,(IX+VBXH)  ; LOAD X
        SET  VBBLNK,(IX+VBSTAT) ; SET BLANK BIT
 ; ENTRY:        WRITE RELATIVE
 ; PURPOSE:      WRITING RELATIVE PATTERNS
 ; INPUT:        HL,DE,A
 ; OUTPUT:       DE
 ; NOTES:        PATTERN IS PRECEEDED BY RELATIVE DISPLAC
 ;               (X FIRST, THEN Y, AND PATTERN SIZE
MWRITR  PUSH  AF          ; SAVE MR
        LD    A,(HL)      ; GET REL X
        INC   HL
        ADD   A,E         ; ADD TO SUPERIOR X
        LD    E,A
        LD    A,(HL)      ; SAME STORY FOR Y
        INC   HL
        ADD   A,D
        LD    D,A
        POP   AF
 ; ENTRY:        WRITE WITH PATTERN SIZE SCARE-UP
 ; PURPOSE:      WRITING VARIABLE SIZED PATTERNS
 ; INPUT:        HL,DE,A
 ; OUTPUT:       DE
 ; NOTES:        FIRST TWO BYTES POINTED AT BY HL ARE TAK
 ;               TO BE PATTERN SIZES (X SIZE FIRST)
MWRITP  LD   C,(HL)       ; GET X SIZE
        INC  HL
        LD   B,(HL)       ; AND Y
        INC  HL
 ; ENTRY:        WRITE WITH COORDINATE CONVERSION
 ; INPUT:        HL,DE,BC,A
 ; OUTPUT:       DE
MWRIT   CALL MRELAB       ; DO CONVERSION
 ; ENTRY:        WRITE ABSOLUTE
 ; INPUT:        HL,BC,A AS ABOVE
 ;               DE = ABSOLUTE SCREEN ADDRESS
MWRITA  BIT  MRFLOP,A     ; FLOP WRITE WANTED?
        JR   NZ,MWRTFL    ; MWRTFL IS SO
        BIT  MRXPND,A     ; EXPAND WANTED?
        JR   NZ,MWX       ; JUMP IF SO
  ; DO NORMAL? WRITE
        XOR  A
MWRT    PUSH BC
        PUSH DE
        LD   B,A          ; ZERO REGISTER B
        LDIR              ; WRITE A LINE
        LD   (DE),A       ; CLEAR THE SHIFTER
        POP  DE
        EX   DE,HL        ; ADVANCE TO NEXT LINE
        LD   C,BYTEPL
        ADD  HL,BC
        EX   DE,HL
        POP  BC
        DJNZ MWRT         ; LOOP IF MORE GOODIES
        RET
; WRITE EXPANDED
MWX     EX   DE,HL
MWX1    PUSH BC
        PUSH HL
        LD   B,C
MWX2    LD   A,(DE)
        INC  DE
        LD   (HL),A
        INC  HL
        LD   (HL),A
        INC  HL
        DJNZ MWX2
        LD   (HL),B
        INC  HL
        LD   (HL),B
        POP  HL
        LD   C,BYTEPL
        ADD  HL,BC
        POP  BC
        DJNZ MWX1
        RET
  ; ROUTINE TO HANDLE FLOPPED CASE
MWRTFL  BIT  MRXPND,A     ; EXPANDED FLOPPED WRITE WANTED
        JR   NZ,MWXF      ; JUMP IF YEP
        XOR  A
WRFL1   PUSH BC
        PUSH DE
        LD   B,A
WRFL2   LDI
        DEC  DE
        DEC  DE
        JP   PE,WRFL2
        LD   (DE),A
        POP  DE
        EX   DE,HL        ; SAME AS NORMAL NOW ON
        LD   C,BYTEPL
        ADD  HL,BC
        EX   DE,HL
        POP  BC
        DJNZ WRFL1
        RET
; WRITE EXPANDED FLOPPED ROUTINE
MWXF    EX   DE,HL
MWXF1   PUSH BC
        PUSH HL
        LD   B,C
MWXF2   LD   A,(DE)
        INC  DE
        LD   (HL),A
        DEC  HL
        LD   (HL),A
        DEC  HL
        DJNZ MWXF2
        LD   (HL),B
        DEC  HL
        LD   (HL),B
        POP  HL
        LD   C,BYTEPL
        ADD  HL,BC
        POP  BC
        DJNZ MWXF1
        RET
 ; NAME:         BLANK FROM VECTOR
 ; PURPOSE:      BLANK WITH INFO LOAD FROM VECTOR
 ; INPUT:        IX = VECTOR
 ;               E = X SIZE
 ;               D = Y SIZE
 ; NOTES:        THIS ROUTINE BLANKS TO 00
 ;               THIS ROUTINE INTERROGATES THE BLANK BIT
 ;               AND REFRAINS FROM BLANKING IF NOT SET
 ;               IF IT WAS SET, IT IS THEN RESET
MVBLAN  BIT  VBBLNK,(IX+VBSTAT) ; IS BLANK BIT SET?
        RET  Z            ; QUIT IF NOT
        RES  VBBLNK,(IX+VBSTAT) ; KILL BLANK BIT
        LD   H,(IX+VBOAH) ; LOAD BLANK ADDRESS
        LD   L,(IX+VBOAL)
        BIT  MRFLOP,(IX+VBMR) ; IS FLOP SET?
        JR   Z,MVBLA1         ; JUMP IF NOT
        LD   A,E              ; X SIZE TO A
        NEG                   ; TWOS COMPLEMENT AND ADD 1
        INC  A
        LD   C,A
        LD   B,0FFH
        ADD  HL,BC        ; USE TO BACK UP SCREEN ADDRESS
 ; UNMAGIC THE BLANK ADDRESS
MVBLA1
        SET  6,H
        LD   B,0          ; ASSUME BLANK TO ZERO
 ; NAME:        BLANK AREA
 ; PURPOSE:     SETTING N X M REGION TO CONSTANT
 ; INPUT:       HL = BLANK ADDRESS
 ;              E = X SIZE
 ;              D = Y SIZE
 ;              B = DATA TO FILL WITH
MBLANK  LD   A,BYTEPL     ; COMPUTE LINE INCREMENT
        SUB  E
        LD   C,A
        LD   A,B          ; A = DATA TO FILL WITH
MBLAN1  LD   B,E
MBLAN2  LD   (HL),A
        INC  HL
        DJNZ MBLAN2
        ADD  HL,BC
        DEC  D
        JR   NZ,MBLAN1
        RET
; NAME:         RESTORE AREA
; INPUT:        HL = SCREEN ADDRESS TO RESTORE TO
;               DE = SAVE AREA ADDRESS
; NOTE:         SIZES ARE LOADED FROM THE SAVE AREA
MREST   EX   DE,HL
        LD   C,(HL)
        INC  HL
        LD   B,(HL)
        INC  HL
        SET  6,D          ; MAKE SURE WE ARE NONMAGIC
        XOR  A
MREST1  PUSH BC
        PUSH DE
        LD   B,A
        LDIR
        EX   DE,HL
        POP  HL
        LD   C,BYTEPL
        ADD  HL,BC
        EX   DE,HL
        POP  BC
        DJNZ MREST1
        RET
 ;
 ; ******************************
 ; * CHARACTER DISPLAY ROUTINES *
 ; ******************************
 ; NAME:         DISPLAY STRING
 ; PURPOSE:      MESSAGE DISPLAY
 ; INPUT:        E,D = X, Y COORDINATES
 ;               HL = STRING ADDRESS
 ;               IX = FONT DESCRIPTOR
 ; OUTPUT:       D,E ALTERED AS IN DISPLAY CHARACTER
 ; STACK USE:    4 BYTES (EXCLUDING USE BY SYSPCH)
 ; EXPLANATION:  AS EACH CHARACTER IS BROUGHT IN, IT
 ; IS TESTED FOR BEING A LIST TERMINATOR ( CHAR = 0)
 ; IF IT ISN'T, DISPLAY CHARACTER IS CALLED AND THE
 ; TEST IS REPEATED FOR THE NEXT CHARACTER.  THUS
 ; A NULL STRING IS HANDLED PROPERLY.
STRNEW  LD   A,(HL)       ; GET CHARACTER
        AND  A            ; BE IT A TERMINATOR
        RET  Z            ; QUIT IF SO
        JP   M,STRD1      ; DISPLAY IF ALT FONT
        CP   64H          ; SUCK IN STRING?
        JR   NC,STRD2     ; JUMP IF YES
STRD1   CALL DISPCH       ; SHOW CHAR
        INC  HL           ; ADVANCE TO NEXT CHAR
        JR   STRNEW       ; AND LOOP
STRD2   AND  10111B       ; MAKE SUCK MASK
        LD   B,A
        INC  HL
        EX   DE,HL
        CALL MSUCK1
        CALL RELD
        JR   STRNEW       ; GO AFTER NEXT CHARACTER
; *****************************
; * CHARACTER DISPLAY ROUTINE *
; *****************************
; INPUT:        A = CHARACTER
;               C = OPTIONS
;               D = Y COORDINATE
;               E = X COORDINATE
;               IX = FONT DESCRIPTOR
;               (ONLY IF ALTERNATE FONT SET USED)
; OUTPUT:       DE UPDATED TO POINT AT NEXT CHARACTER FRA
; NOTES:        THE OPTION BYTE IS FORMATTED AS FOLLOWS:
;       BITS    CONTENTS
;       ----    --------
;       0-1     OFF COLOR FOR EXPANSION
;       2-3     ON COLOR FOR EXPANSION
;       4       OR OPTION
;       5       XOR OPTION
;       6-7     ENLARGMENT FACTOR (N+1)X
;
; CHARACTERS BETWEEN 1 AND 1FH, AND BETWEEN 81H AND 9FH
; ARE INTERPRETED AS TAB CHARACTERS.  THEY CAUSE THE
; CURSOR REPRESENTED BY D AND E TO BE SPACED OVER N
; CHARACTER POSITIONS, WHERE N = CHAR.AND.7FH
; CHARACTERS BETWEEN 20H AND 7FH ARE TAKEN AS REFERENCES
; THE SYSTEM STANDARD 5 X 7 CHARACTER FONT.  CHARACTERS
; BETWEEN 0A0H AND 0FFH REFER TO THE USER SUPPLIED ALTERN
; CHARACTER FONT.  THIS FONT IS DESCRIBED BY A FONT
; DESCRIPTOR TABLE OF THE FOLLOWING FORMAT:
; ******************************
; * 0 * BASE CHARACTER VALUE   *
; ******************************
; * 1 * X FRAME SIZE           *
; ******************************
; * 2 * Y FRAME SIZE           *
; ******************************
; * 3 * X PATTERN SIZE (BYTES) *
; ******************************
; * 4 * Y PATTERN SIZE         *
; ******************************
; * 5 *   PATTERN TABLE        *
; * 6 *     ADDRESS            *
; ******************************
DISPCH  PUSH BC
        PUSH HL
        PUSH IX
        AND  A
        JP   M,DISCH1     ; JUMP IF YES
        LD   IX,SYSFNT
DISCH1  CP   20H          ; IS CHAR < 20H?
        JR   NC,DISC1B    ; JUMP IF NOT
DISC1A  PUSH AF           ; LOOP TO SPACE OVER
        CALL NXTFRM
        CALL FINDL3       ; STORE IT BACK
        POP  AF
        DEC  A
        JR   NZ,DISC1A
        JR   DISCH5       ; JUMP TO EXIT
DISC1B  SUB  (IX+FTBASE)  ; SUBTRACT BASE CHAR
        LD   E,A
        LD   D,0
        LD   HL,0
        LD   C,(IX+FTBYTE) ; MULTIPLY CHARACTER
DISCH2  LD   B,(IX+FTYSIZ) ; BY PATTERN SIZE
DISCH3  ADD  HL,DE
        DJNZ DISCH3
        DEC  C
        JR   NZ,DISCH2
        LD   D,(IX+FTPTH) ; ADD TO TABLE START
        LD   E,(IX+FTPTL)
        ADD  HL,DE
; COMPUTE POSITION WHERE NEXT CHARACTER WOULD GO
; AND SAVE
        CALL NXTFRM       ; STEP COORDINATES TO NEXT FRAM
        PUSH DE           ; SAVE
        LD   B,(IX+FTYSIZ)
DISCH4  PUSH BC
        PUSH HL
        CALL WRTLIN
        POP  HL
        LD   C,(IX+FTBYTE) ; STEP TO NEXT LINE OF PATTERN
        ADD  HL,BC
        POP  BC
        LD   A,(IY+CBD)    ; ADVANCE Y COORDINATE
        ADD  A,C
        LD   (IY+CBD),A
        DJNZ  DISCH4
        POP   DE          ; RESTORE NEW POSITION
        CALL  FINDL3      ; STUFF DE BACK INTO CONTEXT
DISCH5  POP   IX
        POP   HL
        POP   BC
        RET
; SUBROUTINE TO CONVERT ENLARGEMENT FACTOR TO ITERATION C
; INPUT:        MODE BYTE FROM CONTEXT SAVE AREA
; OUTPUT:       B,A = ITERATION COUNT
DCLCTB  LD   A,(IY+CBC)   ; GET MODE BYTE
        RLCA
        RLCA
        AND  03           ; ISOLATE ENLARGEMENT FACTOR
        INC  A
        LD   B,A
        XOR  A
        SCF
DCLCT1  ADC  A,A
        DJNZ DCLCT1
        LD   B,A
        RET
; SUBROUTINE TO UPDATE COORDINATES TO POINT AT NEXT CHARA
; FRAME:
; INPUT:        COORDINATES TAKEN FROM CBD, CBE IN CONTEXT
; OUTPUT:       UPDATED COORDINATES RETURNED IN D AND E
;               A,B = CLOBBERED  C= ENLARGE FACTOR CONVERT
NXTFRM  CALL DCLCTB       ; GET ITERATION COUNT
        LD   C,B          ; SAVE
        LD   D,(IY+CBD)   ; GET Y COORD
        LD   A,(IY+CBE)   ; GET X COORD
NXTFR1  ADD  A,(IX+FTFSX) ; ADD X FRAME SIZE
        DJNZ NXTFR1       ; 2**ENLARGE TIMES
        CP   160          ; PAST RIGHT EDGE OF SCREEN?
        JR   C,NXTFR3
        LD   A,D
        LD   B,C
NXTFR2  ADD  A,(IX+FTFSY) ; YEP - ADVANCE VERTICAL
        DJNZ NXTFR2
        LD   D,A
        XOR  A
NXTFR3  LD   E,A
        RET
; SUBROUTINE TO WRITE ONE LINE OF A PATTERN WITH ENLARGE
; AND EXPAND
; ENTRY: HL = SOURCE  IX = FONT TABLE
WRTLIN  LD   C,(IX+FTBYTE)
        LD   B,0
        PUSH IX           ; CAPTURE STACK POINTER
        LD   IX,0
        ADD  IX,SP
        PUSH IX           ; SAVE CAPTURED STACK
        POP  DE           ; DE = CAPTURED STACK
        LD   A,0CH        ; SET EXPAND TO 00,11
        OUT  (XPAND),A
        LD   A,08H        ; SET EXPAND BIT
        OUT  (MAGIC),A
        LD   A,(IY+CBC)   ; GET CONTROL BYTE
        AND  0C0H         ; ISOLATE ENLARGE AMOUNT
        JR   Z,WRTL3      ; JUMP IF ZERO
        RLCA
        RLCA
WRTL1   EX   DE,HL
        AND  A            ; CLEAR CARRY BIT
        SBC  HL,BC        ; COMPUTE STOCK FRAME SIZE
        SBC  HL,BC
        LD   SP,HL        ; SEIZE STACK SPACE
        RES  6,H          ; MAGNIFY THE ADDRESS
        PUSH AF
        LD   B,C
WRTL2   LD   A,(DE)       ; GET SOURCE BYTE
        INC  DE
        LD   (HL),A       ; EXPAND IT
        INC  HL
        LD   (HL),A
        INC  HL
        DJNZ WRTL2
        SLA  C
        POP  AF
        LD   HL,0         ; CAPTURE STACK TOP AGAIN
        ADD  HL,SP
        LD   D,H          ; SET DE=HL
        LD   E,L          ; FOR NEXT DEST COMBO
        DEC  A
        JR   NZ,WRTL1
; NOW DO WRITE TO SCREEN
WRTL3   CALL DCLCTB       ; GET ITERATION COUNTER
        CALL DELOAD
        LD   A,(IY+CBC)
        OUT  (XPAND),A
        AND  030H
        OR   8
        CALL RELTA
        EX   DE,HL
WRTL4   PUSH AF
        PUSH BC
        PUSH DE
        PUSH HL
        LD   B,C
WRTL5   LD   A,(DE)
        INC  DE
        LD   (HL),A
        INC  HL
        LD   (HL),A
        INC  HL
        DJNZ WRTL5
        LD   A,(IY+CBE)
        AND  03
        JR   Z,WRTL6
        LD   (HL),B
WRTL6   POP  HL           ; STEP TO NEXT LINE
        LD   C,BYTEPL
        ADD  HL,BC
        POP  DE
        POP  BC
        POP  AF
        OUT  (MAGIC),A
        DJNZ WRTL4
        LD   SP,IX        ; RESTORE STACK
        POP  IX
        RET
;
  ; MACRO TO GENERATE CHARACTER PATTERN TABLE ENTRY
DEFCHR  MACRO AA, BB, CC, DD, EE, FF, GG
        DB   AA
        DB   BB
        DB   CC
        DB   DD
        DB   EE
        DB   FF
        DB   GG
        ENDM
;
 ; LARGE CHARACTER SET (8 X 8)
LRGCHR
        DEFCHR 000H, 000H, 000H, 000H, 000H, 000H, 000H ; SPACE
        DEFCHR 020H, 020H, 020H, 020H, 020H, 000H, 020H ; !
        DEFCHR 050H, 050H, 050H, 000H, 000H, 000H, 000H ; "
        DEFCHR 048H, 048H, 0FCH, 048H, 0FCH, 048H, 048H ; #
        DEFCHR 020H, 078H, 080H, 070H, 008H, 0F0H, 020H ; $
        DEFCHR 0C0H, 0C8H, 010H, 020H, 040H, 098H, 018H ; %
        DEFCHR 060H, 090H, 0A0H, 040H, 0A8H, 090H, 068H ; &
        DEFCHR 060H, 060H, 060H, 000H, 000H, 000H, 000H ; '
        DEFCHR 010H, 020H, 020H, 020H, 020H, 020H, 010H ; (
        DEFCHR 040H, 020H, 020H, 020H, 020H, 020H, 040H ; )
        DEFCHR 000H, 0A8H, 070H, 0D8H, 070H, 0A8H, 000H ; *
        DEFCHR 000H, 020H, 020H, 0F8H, 020H, 020H, 000H ; +
        DEFCHR 000H, 000H, 000H, 060H, 060H, 020H, 040H ; ,
        DEFCHR 000H, 000H, 000H, 0F8H, 000H, 000H, 000H ; -
        DEFCHR 000H, 000H, 000H, 000H, 000H, 060H, 060H ; .
        DEFCHR 000H, 008H, 010H, 020H, 040H, 080H, 000H ; /
        DEFCHR 070H, 088H, 088H, 088H, 088H, 088H, 070H ; 0
        DEFCHR 020H, 060H, 020H, 020H, 020H, 020H, 070H ; 1
        DEFCHR 070H, 088H, 008H, 070H, 080H, 080H, 0F8H ; 2
        DEFCHR 070H, 088H, 008H, 030H, 008H, 088H, 070H ; 3
        DEFCHR 010H, 030H, 050H, 090H, 0F8H, 010H, 010H ; 4
        DEFCHR 0F8H, 080H, 0F0H, 008H, 008H, 088H, 070H ; 5
        DEFCHR 030H, 040H, 080H, 0F0H, 088H, 088H, 070H ; 6
        DEFCHR 0F8H, 008H, 010H, 020H, 040H, 040H, 040H ; 7
        DEFCHR 070H, 088H, 088H, 070H, 088H, 088H, 070H ; 8
        DEFCHR 070H, 088H, 088H, 078H, 008H, 010H, 060H ; 9
        DEFCHR 000H, 060H, 060H, 000H, 060H, 060H, 000H ; :
        DEFCHR 060H, 060H, 000H, 060H, 060H, 020H, 040H ; ;
        DEFCHR 010H, 020H, 040H, 080H, 040H, 020H, 010H ; <
        DEFCHR 000H, 000H, 0F8H, 000H, 0F8H, 000H, 000H ; =
        DEFCHR 040H, 020H, 010H, 008H, 010H, 020H, 040H ; >
        DEFCHR 070H, 088H, 008H, 010H, 020H, 000H, 020H ; ?
        DEFCHR 070H, 088H, 0B8H, 0A8H, 0B8H, 080H, 078H ; @
        DEFCHR 070H, 088H, 088H, 0F8H, 088H, 088H, 088H ; A
        DEFCHR 0F0H, 088H, 088H, 0F0H, 088H, 088H, 0F0H ; B
        DEFCHR 070H, 088H, 080H, 080H, 080H, 088H, 070H ; C
        DEFCHR 0F0H, 088H, 088H, 088H, 088H, 088H, 0F0H ; D
        DEFCHR 0F8H, 080H, 080H, 0E0H, 080H, 080H, 0F8H ; E
        DEFCHR 0F8H, 080H, 080H, 0E0H, 080H, 080H, 080H ; F
        DEFCHR 070H, 088H, 080H, 080H, 098H, 088H, 078H ; G
        DEFCHR 088H, 088H, 088H, 0F8H, 088H, 088H, 088H ; H
        DEFCHR 070H, 020H, 020H, 020H, 020H, 020H, 070H ; I
        DEFCHR 008H, 008H, 008H, 008H, 008H, 088H, 070H ; J
        DEFCHR 088H, 090H, 0A0H, 0C0H, 0A0H, 090H, 088H ; K
        DEFCHR 080H, 080H, 080H, 080H, 080H, 080H, 0F8H ; L
        DEFCHR 088H, 0D8H, 0A8H, 0A8H, 088H, 088H, 088H ; M
        DEFCHR 088H, 0C8H, 0A8H, 098H, 088H, 088H, 088H ; N
        DEFCHR 0F8H, 088H, 088H, 088H, 088H, 088H, 0F8H ; O
        DEFCHR 0F0H, 088H, 088H, 0F0H, 080H, 080H, 080H ; P
        DEFCHR 070H, 088H, 088H, 088H, 0A8H, 090H, 068H ; Q
        DEFCHR 0F0H, 088H, 088H, 0F0H, 0A0H, 090H, 088H ; R
        DEFCHR 070H, 088H, 080H, 070H, 008H, 088H, 070H ; S
        DEFCHR 0F8H, 020H, 020H, 020H, 020H, 020H, 020H ; T
        DEFCHR 088H, 088H, 088H, 088H, 088H, 088H, 070H ; U
        DEFCHR 088H, 088H, 088H, 050H, 050H, 020H, 020H ; V
        DEFCHR 088H, 088H, 088H, 0A8H, 0A8H, 0D8H, 088H ; W
        DEFCHR 088H, 088H, 050H, 020H, 050H, 088H, 088H ; X
        DEFCHR 088H, 088H, 050H, 020H, 020H, 020H, 020H ; Y
        DEFCHR 0F8H, 008H, 010H, 020H, 040H, 080H, 0F8H ; Z
        DEFCHR 070H, 040H, 040H, 040H, 040H, 040H, 070H ; [
        DEFCHR 000H, 080H, 040H, 020H, 010H, 008H, 000H ; \
        DEFCHR 070H, 010H, 010H, 010H, 010H, 010H, 070H ; ]
        DEFCHR 020H, 070H, 0A8H, 020H, 020H, 020H, 020H ; ^
        DEFCHR 000H, 020H, 040H, 0F8H, 040H, 020H, 000H ; <-
        DEFCHR 020H, 020H, 020H, 020H, 0A8H, 070H, 020H ; DOWN
        DEFCHR 000H, 020H, 010H, 0F8H, 010H, 020H, 000H ; RIGHT
        DEFCHR 000H, 088H, 050H, 020H, 050H, 088H, 000H ; MULTI
        DB   0
        DB   20H
        DB   0
        DB   0F8H
        DB   0
        DB   20H
; ** LAST BYTE OF DIVIDE IS ZERO, WHICH HAPPENS TO BE FIRST
;    BYTE OF...
 ; SMALL CHARACTERS (4 X 6)
SMLCHR
        DEF5 000H, 000H, 000H, 000H, 000H ; SPACE
;
MMJUMP  POP  IX
        EX   (SP),HL
        JP   (IX)
;
; NAME:    CONVERT KEY CODE TO ASCII
; PURPOSE: SAME
; INPUT:   A=KEY CODE
; OUTPUT:  A=ASCII EQUIVALENT
; HOW:     TABLE LOOKUP
MKCTAS
        LD   C,B
        LD   B,0
        LD   HL,KCTATB
        ADD  HL,BC
        LD   A,(HL)
QFROG   LD   (IY+CBA),A
        RET
;
KCTATB
        DB   ' '          ; SPACE
        DB   'C'          ; BULLET
        DB   5EH          ; UP ARROW
        DB   5CH          ; DOWN ARROW
        DB   '%'
        DB   'R'          ; RECALL
        DB   'S'          ; STORE
        DB   ';'          ; PLUS MINUS
        DB   '/'          ; DIVIDE
        DB   '7'
        DB   '8'
        DB   '9'
        DB   '*'          ; TIMES
        DB   '4'
        DB   '5'
        DB   '6'
        DB   '-'          ; MINUS
        DB   '1'
        DB   '2'
        DB   '3'
        DB   '+'          ; PLUS
        DB   '&'          ; CE
        DB   '0'
        DB   '.'          ; POINT
        DB   '='          ; EQUALS
;
 ; NAME:         FILL AREA
 ; PURPOSE:      SET REGION OF SCREEN TO CONSTANT VALUE
 ; INPUT:        A = DATA TO FILL WITH
 ;               BC = NUMBER OF BYTES TO FILL
 ;               DE = STARTING ADDRESS OF REGION TO FILL
MFILL   EX   DE,HL
MFILL1  LD   (HL),A       ; STUFF BYTE
        CPI               ; BUMP HL, DEC BC
        JP   PE,MFILL1
        RET
;
 ; NAME:     RELATIVE TO ABSOLUTE
 ; PURPOSE:  COORDINATE CONVERSION
 ; INPUT:    E = X COORDINATE
 ;           D = Y COORDINATE
 ;           A = MAGIC REGISTER
 ; OUTPUT:   DE = ABSOLUTE ADDRESS
 ;           A = MAGIC REGISTER TO USE
; MAGIC ENTRY POINT
MRELAB  CALL RELTA
        JR   MRELA2
; NONMAGIC ENTRY POINT
MRELA1  CALL RELTA1
        SET  6,D          ; NONMAGIC THE ADDRESS
MRELA2  LD   (IY+CBE),E   ; UPDATE CB DE
        LD   (IY+CBD),D
MFROG   JR   QFROG
; MAGIC ENTRY POINT
RELTA   CALL RELTA1
        OUT  (MAGIC),A
        RET
CKSUM2  DB   0            ; *** CHECKSUM ***
        DEF5 0E0H, 0A0H, 0A0H, 0A0H, 0E0H ; 0
        DEF5 040H, 040H, 040H, 040H, 040H ; 1
        DEF5 0E0H, 020H, 0E0H, 080H, 0E0H ; 2
        DEF5 0E0H, 020H, 060H, 020H, 0E0H ; 3
        DEF5 0A0H, 0A0H, 0E0H, 020H, 020H ; 4
        DEF5 0E0H, 080H, 0E0H, 020H, 0E0H ; 5
        DEF5 0E0H, 080H, 0E0H, 0A0H, 0E0H ; 6
        DEF5 0E0H, 020H, 020H, 020H, 020H ; 7
        DEF5 0E0H, 0A0H, 0E0H, 0A0H, 0E0H ; 8
        DEF5 0E0H, 0A0H, 0E0H, 020H, 0E0H ; 9
        DEF5 000H, 040H, 000H, 040H, 000H ; :
        DEF5 040H, 0E0H, 0E0H, 0E0H, 0E0H ; BULLET
;
; MOVE ROUTINE
MMOVE   LDIR
        RET
;
 ; SYSTEM ENTRY POINT FOR NONMAGIC ADDRESS
RELTA1  PUSH HL
        AND  0FCH         ; TOSS OUT SHIFT AMOUNT
        LD   L,A          ; SAVE
        LD   A,E          ; GET X
        AND  03H          ; ISOLATE SHIFT AMOUNT
        OR   L            ; COMBINE WITH MR
RELTA2  PUSH AF
        AND  040H         ; IS FLOPPED BIT SET?
        LD   A,E
        JR   Z,RELTA3     ; JUMP IF NOT
        CPL               ; YEP - UNFLOP THE COORDINATE
        ADD  A,160
RELTA3  LD   L,D          ; HL = Y
        LD   H,0
        ADD  HL,HL        ; SET HL = Y * 8
        ADD  HL,HL
        ADD  HL,HL
        LD   D,H
        LD   E,L
        ADD  HL,HL        ; SET HL = Y * 32
        ADD  HL,HL
        ADD  HL,DE        ; SET HL = Y * 40
        SRL  A            ; A = X 4
        SRL  A
        LD   E,A
        LD   D,0
        ADD  HL,DE        ; HL = Y * 40 + X        4
        IF   NWHDWR-1
        ENDIF
        EX   DE,HL
;
; NAME:         RETURN FROM MACRO SUBROUTINE
; PURPOSE:      RETURN CONTROL TO CALLER
; THIS CODE WAS 'STOLEN' FROM RELABS SINCE
; IT DOES THE STACK CLEANUP THAT MRET DOES
MMRET   POP  AF
        POP  HL
        RET
;
; ENTRY FOR USER
INXNIB  CALL XNIB
        JR   MFROG
;
; NAME:         INDEX NIBBLE
; PURPOSE       LOAD OF SPECIFIED NIBBLE RELATIVE TO BASE
;               C = NIBBLE NUMBER
;               HL = BASE ADDRESS
; OUTPUT:       NIBBLE RETURNED RIGHT JUSTIFIED IN A
; DESCRIPTION:  BYTE = NIBBLE# 
;  THE LOW ORDER NIBBLE OF A GIVEN BYTE IS ADDRESSED
; BY AN EVEN NIBBLE NUMBER.
XNIB    PUSH HL
        PUSH BC
        LD   B,0
        SRL  C
        ADD  HL,BC
        LD   A,(HL)
        POP  BC
        BIT  0,C
        JR   Z,XNIB1
        RRCA
        RRCA
        RRCA
        RRCA
XNIB1   AND  0FH
        POP  HL
        RET
;
; NAME:         STORE NIBBLE
; PURPOSE:      NIBBLE STORING (!)
; INPUT:        A = NIBBLE TO STORE
;               C = NIBBLE NUMBER (AS IN XNIB)
;       HL = BASE ADDRESS
PUTNIB  PUSH HL
        PUSH BC
        LD   B,0
        SRL  C
        ADD  HL,BC
        POP  BC
        BIT  0,C
        JR   Z,PUTNB1
; L.O. CASE - SHIFT IT
        RLCA
        RLCA
        RLCA
        RLCA
        XOR  (HL)
        AND  0F0H
        JR   PUTNB2
PUTNB1  XOR  (HL)         ; L.O. CASE
        AND  0FH
PUTNB2  XOR  (HL)
        LD   (HL),A
        POP  HL
        RET
;
; NAME:         INDEX WORD TABLE (WORD INDEX)
; PURPOSE:      TO INDEX AN ARRAY OF DEFW'S
; INPUTS:       A = INDEX NUMBER (0-255)
;               HL -> TABLE ENTRY 0
; OUTPUTS:      DE = ENTRY LOOKED UP
;               HL = POINTER TO ENTRY IN TABLE
MINDW   LD   E,A
        LD   D,0
        SLA  E
        RL   D            ; DE*2
        ADD  HL,DE
        LD   E,(HL)
        INC  HL
        LD   D,(HL)
        DEC  HL
STHLDE  CALL FINDL3
        JR   MINDB1       ; JOIN STORE IN INDEX BYTE
;
; NAME:         INDEX BYTE TABLE
; PURPOSE:      TABLE LOOKUP
; INPUTS:       A = INDEX NUMBER
; OUTPUT:       A = VALUE OF BYTE
;               HL = POINTER TO TABLE ENTRY
MINDB   LD   E,A
        LD   D,0
        ADD  HL,DE
        LD   A,(HL)
        LD   (IY+CBA),A
MINDB1  LD   (IY+CBH),H
        LD   (IY+CBL),L
        RET
;
; NAME:    DISPLAY TIME
; PURPOSE: DISPLAY TIME ON SCREEN
; INPUTS:  E = X COORD
;          D = Y COORD
;          C = SAME AS DISCHR OPTIOS EXCEPT BIT 7 = 1
;              TO DISPLAY COLON AND SECONDS
; OUTPUTS: NONE
MDISTI
        LD   IX,SMLFNT
        LD   B,42H
        LD   HL,GTMINS
        PUSH BC
        RES  7,(IY+CBC)
        CALL BCDISP
        POP  BC
        BIT  7,C
        RET  Z
        LD   A,80H+3AH
        CALL DISPCH
        LD   B,42H
        LD   HL,GTSECS
; AND FALL INTO...
;
; NAME:         DISPLAY BCD NUMBER
; INPUT:        B = NUMBER DISPLAY OPTIONS
;               C = CHARACTER DISPLAY OPTIONS
;               DE = Y,X COORDINATES
;               HL = NUMBER ADDRESS (POINTS AT LO BYTE)
;               IX = ALTERNATE FONT (IF USED)
; OUTPUT:       DE UPDATED
; DESCRIPTION:  THIS ROUTINE CONVERTS EACH NIBBLE INTO
; ASCII AND DISPLAYS IT.  THE NORMALLY ILLEGAL BCD
; VALUES ARE DISPLAYED AS CODES 2A THRU 2F RESPECTIVELY.
; THE NUMBER DISPLAY OPTIONS BYTE IS FORMATED AS FOLLOWS:
; BIT 7         SET IF LEADING ZERO SUPPRESSION WANTED
; BIT 6         SET IF USE OF ALTERNATE FONT WANTED
; BITS 5-0      NUMBER OF DIGITS TO DISPLAY (NOT NUMBER 0
BCDISP  LD   A,B            ; GET OPTIONS BYTE
        AND  3FH            ; ISOLATE NUMBER OF DIGITS
BCDD0   DEC  A
        RET  M              ; QUIT IF NULL OR NO MORE
        LD   C,A            ; SAVE
        CALL XNIB           ; GET NEXT DIGIT
        JR   NZ,BCDD1       ; JUMP IF NONZERO
        BIT  7,B            ; IS ZERO SUPPRESS ON?
        JR   Z,BCDD1        ; JUMP IF NOT
        OR   C              ; LAST DIGIT?
        JR   NZ,BCDD4
BCDD1   RES  7,B            ; CLEAR LEADING ZERO FLAG
        ADD  A,6
        AND  0FH
        ADD  A,2AH
BCDD2   BIT  6,B            ; ALTERNATE FONT?
        JR   Z,BCDD3        ; JUMP IF SO
        OR   80H            ; YEA, SET THE BIT
BCDD3   CALL DISPCH         ; DISPLAY THE CHAR
        LD   A,C            ; GET LOOP COUNTER IN A
        JR   BCDD0          ; AND GO FOR NEXT
BCDD4   LD   A,' '          ; LEADING ZERO - WRITE A SPACE
        JR   BCDD2
;
; NAME:    INCREMENT SCORE
; PURPOSE: INCREMENT SCORE AND COMPARE TO END SCORE
; INPUTS:  HL -> PLAYER SCORE LOW ADDR OF 3 BYTES
; OUTPUTS: GSBEND OF GAMSTB SET IF MAX SCORE REACHED
MINCSC  LD   B,3
        PUSH HL
INCLOP  LD   A,(HL)
        ADD  A,1
        DAA
        LD   (HL),A
        JR   NZ,CMPIT
        INC  HL
        DJNZ INCLOP
CMPIT   POP  HL
        INC  HL
        INC  HL
        LD   A,(GAMSTB)
        BIT  GSBSCR,A
        RET  Z
        LD   DE,ENDSCR+2
        LD   B,3
CMPLOP  LD   A,(DE)
        CP   (HL)
        JR   Z,REPEAT       ; ENDSCR = SCORE
        RET  NC             ; ENDSCR > SCORE
SETEND  LD   HL,GAMSTB      ; ENDSCR < SCORE
        SET  GSBEND,(HL)
        RET
REPEAT  DEC  DE
        DEC  HL
        DJNZ CMPLOP
        JR   SETEND
;
; NAME:         QUIT
; PURPOSE:      HOLD PRESENT GAME SCORE UNTIL KEY HIT OR
; SAY GAME OVER
MQUIT   SYSSUK STRDIS
        DB   48
        DB   24
        DB   01001100B
        DW   GMOVR
        SYSTEM ACTINT       ; ACTIVATE INTERRUPTS
MQUIT1  SYSSUK SENTRY       ; WAIT FOR SOMETHING TO HAPPEN
        DW   AKEYS
        CP   ST0
        JR   Z,MQUIT2       ; TRIGGER CHANGE?
        CP   SKYD           ; KEY HIT?
        JR   NZ,MQUIT1      ; NO - KEEP GOING
MQUIT2  RST  0              ; YES, RESET
GMOVR   DB   'GAME'
        DB   6
        DB   'OVER'
        DB   0
;
; *****************
; * MENU ROUTINES *
; *****************
NOLINE  EQU  96           ; NUMBER OF DISPLAYED LINES
MNNL    EQU  0            ; NEXT FIELD
MNNH    EQU  1
MNSAL   EQU  2            ; STRING ADDRESS
MNSAH   EQU  3
MNGL    EQU  4            ; GO TO ADDRESS
MNGH    EQU  5
;
; SYSTEM POWER UP ROUTINE
PWRUP   LD   A,(FIRSTC)   ; GET FIRST CASSETTE LOCATION
        CP   0C3H         ; IS IT A JUMP?
        JP   Z,FIRSTC     ; JUMP TO IT IF SO
        LD   SP,BEGRAM
        SYSSUK FILL       ; CLEAR SYSTEM RAM
        DW   BEGRAM
        DW   50
        DB   0
        LD   (WASTE),A    ; CLEAR SHIFTER
        DEC  A
        LD   (TIMOUT),A   ; CLEAR TIMEOUT WATCHDOG
        SYSTEM INTPC
        DO   EMUSIC
        DO   SETOUT
        DB   (NOLINE*2)-1
        DB   41
        DB   8
        DO   COLSET
        DW   MENUCL
        DO   ACTINT
        EXIT
        LD   DE,GAMSTR    ; 'SELECT GAME' AS TITLE
        LD   HL,FIRSTC    ; ASSUME MENU STARTS IN CASSETT
        LD   A,(HL)       ; GET FIRST CASSETTE BYTE
        INC  HL
        CP   55H           ; IS SENTINEL THERE?
        JR   Z,PWRUP1      ; YEP - JUMP
        LD   HL,GUNLNK     ; WRONG - USE ONBOARD ONLY
PWRUP1  SYSTEM MENU        ; DISPLAY THE MENU
;
; NAME:         DISPLAY MENU AND BRANCH ON CHOICE
; INPUT:        HL = MENU LIST
;               DE = MENU TITLE
; OUTPUT:       DE = TITLE OF SELECTION MADE
; DESCRIPTION:
;       THE MENU LIST IS A LINKED LIST OF THE FOLLOWING F
; *************************
; * 0 * NEXT ENTRY        *
; * 1 *                   *
; *************************
; * 2 * STRING ADDRESS    *
; * 3 *                   *
; *************************
; * 4 * BRANCH TO ADDRESS *
; * 5 *                   *
; *************************
; THIS LIST IS TERMINATED BY A NEXT ENTRY FIELD OF ZEROS
; A MAXIMUM OF EIGHT ENTRYS MAY BE DISPLAYED.
MMENU   PUSH HL
        PUSH HL
        CALL MNCLR        ; CLEAR SCREEN AND THROWUP TITLE
        XYRELL DE, 16, 12
        LD   BC, 109H     ; INITIALIZE ENTRY # AND COLOR
MMENU1  POP  IX           ; FIRST ENTRY TO IX
        LD   A,B          ; SELECTION NUMBER TO A
        ADD  A,'0'        ; MAKE IT ASCII
        SYSTEM CHRDIS     ; AND SHOW IT
        LD   A,'-'        ; DISPLAY DASH
        SYSTEM CHRDIS
        LD   H,(IX+MNSAH) ; HL = STRING ADDRESS
        LD   L,(IX+MNSAL)
        SYSTEM STRDIS     ; DISPLAY SELECTION
        LD   A,8
        ADD  A,D          ; TO NEXT LINE
        LD   D,A
        LD   E,16
        INC  B            ; BUMP ENTRY #
        LD   H,(IX+MNNH)  ; HL = NEXT ENTRY ADDR
        LD   L,(IX+MNNL)
        PUSH HL
        LD   A,H
        OR   L
        JR   NZ,MMENU1    ; NO - JUMP BACK
; AT THIS POINT HL = 0 (SP) = 0
        ADD  HL,SP        ; HL = STACK POINTER
MMENU3  PUSH BC
        LD   BC,0101H
        XYRELL DE, 16, 77 ; FEEDBACK ADDRESS
        SYSTEM GETNUM     ; GET NUMBA
        POP  BC
        LD   A,(HL)       ; HOW DOES SHE LOOK?
        AND  A            ; ZERO ENTERED?
        JR   Z,MMENU5     ; JUMP IF SO
        CP   B            ; IN RANGE?
        JR   C,MMENU6     ; JUMP IF SO
MMENU5  LD   A,'?'        ; DUD ENTRY - SHOW ?
        SYSTEM CHRDIS
        JR   MMENU3       ; GO BACK FOR NEXT TRY
MMENU6  POP  HL           ; THROW OUT ENTRY AREA
        POP  DE           ; RESTORE HEAD OF MENU LIST
        LD   B,A          ; NUMBER ENTERED TO B
MMENU7  EX   DE,HL        ; HL - ENTRY POINTER
        LD   E,(HL)       ; DE - NEXT
        INC  HL
        LD   D,(HL)
        DJNZ MMENU7       ; COUNT DOWN TO ENTRY
        INC  HL
        LD   E,(HL)       ; STRING TO DE
        INC  HL
        LD   D,(HL)
        INC  HL
        LD   C,(HL)       ; GO TO ADDRESS TO BC
        INC  HL
        LD   B,(HL)
        POP  HL           ; HL = RETURN TO PLACE
        POP  AF           ; THROW OUT OLD PC
        PUSH BC           ; PUT NEW PC ON STACK
        PUSH HL           ; AND PUT BACK DUMMY RETURN
FINDL3  LD   (IY+CBE),E   ; PASS BACK TITLE ADDRESS
        LD   (IY+CBD),D
        RET               ; AND GO BACK
;
; NAME:         GET PARAMETER
; PURPOSE:      INPUT OF PROGRAM OPTIONS
; INPUT:        A = NUMBER OF DIGITS
;               BC = PROMPT STRING ADDRESS
;               DE = FRAME TITLE ADDRESS
;               HL = PARAMETER ADDRESS
; DESCRIPTION:
;       THIS ROUTINE ASKS THE USER TO ENTER A NUMBER
; FIRST A MENU FRAME IS CREATED, USING THE STRING
; POINTED AT BY DE AS A TITLE.  THE STRING 'ENTER'
; IS DISPLAYED, FOLLOWED BY THE PROMPT STRING.
; GETNUM IS THEN CALLED TO INPUT THE NUMBER.  FEEDBACK
; IS PROVIDED IN DOUBLE SIZED CHARACTERS.
; NOTE: ** THIS ROUTINE USES TWO SYSTEM LEVELS AND THE AL
MGETP   PUSH AF           ; SAVE NUMBER OF DIGITS
        PUSH HL
        PUSH BC
        CALL MNCLR
        SYSSUK STRDIS     ; DISPLAY 'ENTER'
        DB   8
        DB   32
        DB   1001B
        DW   ENTSTG
        POP  HL
        SYSTEM STRDIS     ; DISPLAY WHAT TO ENTER
        POP  HL
        POP  AF
        LD   B,A
        SET  6,C          ; SET LARGE CHARS
        XYRELL DE, 48, 48 ; LOAD FEEDBACK ADDRESS
        SYSTEM GETNUM     ; GET NUMBER
        SYSSUK PAWS       ; LET USER READ IT
        DB   15
        RET
; SUBROUTINE TO CLEAR SCREEN FOR MENU AND THROWUP TITLE
MNCLR   PUSH DE
        SYSSUK FILL
        DW   NORMEM
        DW   11*BYTEPL
        DB   0
        SYSSUK FILL
        DW   NORMEM + (11*BYTEPL)
        DW   (NOLINE-11)*BYTEPL
        DB   55H
        POP  HL
        XYRELL DE, 24, 0  ; TITLE
        LD   C,0100B
        SYSTEM STRDIS
        RET
;
; NAME:         GET NUMBER
; INPUT:        B = DISPLAY OPTIONS
;               C = CHRDIS OPTIONS FOR FEEDBACK
;               DE = COORDINATE OF FEEDBACK AREA
;               HL = ADDRESS OF WHERE TO STASH NUMBER
; DESCRIPTION:  THIS ROUTINE CAN INPUT A NUMBER FROM
;       EITHER THE KEYBOARD OR THE HAND CONTROL.  KEYBOARD
;       ENTRY PROCEEDS CONVENTIONALY.  GETNUM EXITS
;       WHEN THE EQUALS KEY IS PRESSED OR THE REQUIRED NUM
;       OF DIGITS IS ENTERED.
;               PLAYER ONE HAND CONTROL MAY ALSO BE USED TO
;       ENTER A NUMBER.  TO USE THIS OPTION, PULL THE TRIG
;       THE ROTATE THE POT UNTIL THE NUMBER YOU WISH TO
;       ENTER IS SHOWN IN THE FEEDBACK AREA.  PULL THE TRIG
;       AGAIN TO REGISTER THE ENTRY.  IF DURING THE PROC
;       THE KEYBOARD IS USED - KEYBOARD INPUT WILL OVERRIDE
MGETN   EXX
        CALL CLRNUM       ; CLEAR THE NUMBER
        LD   C,A          ; SET ZERO DIGITS IN - POT ENAB
MGETN1  LD   A,(IY+CBB)   ; ENTRY COMPLETE?
        XOR  C
        AND  3FH
        RET  Z            ; QUIT IF SO
        LD   HL,MGETN1
        PUSH HL
        SYSTEM RANGED     ; RANDOMIZE WHILE WE WAIT
        SYSSUK SENTRY
        DW   NUMBAS
        SYSSUK DOIT
        DW   GNUMD0
        RET               ; NOTHIN - LOOP ON SENTRY
GNUMD0  JMPd SKYD, MGETN6, 0H
        JMPd ST0, MGETN2, 0H
        JMPd SP0, MGETN3, 0H
; TRIGGER ROUTINE
MGETN2  BIT  4,B          ; 0-1 TRANS?
        RET  Z            ; NO -IGNORE
        LD   A,C
        INC  A            ; ARE WE ALREADY IN POT MODE?
        JR   Z,MGETN9     ; YEP - JUMP TO EXIT
        BIT  7,C          ; POT LEGAL
        RET  NZ           ; NO - IGNORE
        LD   C,0FFH       ; SET POT FLAG
; POT ROUTINE
MGETN3  LD   A,C          ; QUIT IF NOT IN POT MODE
        INC  A
        RET  NZ
; HOW MANY DIGITS?
        EXX               ; TO NORMAL SET
        LD   A,B          ; SNATCH DIGITS
        EXX
        CP   1            ; 1 PRAY TELL?
        LD   B,10
        JR   Z,MGETN4     ; JUMP IF GOOD GUESS
        LD   B,100        ; WRONG!
MGETN4  IN   A,(POT0)     ; GET CURRENT POT VALUE
        LD   D,A          ; RANGE IT
        XOR  A
        LD   E,A
        LD   H,A
MGETN5  ADD  HL,DE
        ADC  A,0          ; ADD EVERY CARRY TO AC
        DAA
        DJNZ MGETN5
        EXX               ; BACK TO NORMAL SET
        LD   (HL),A
        JR   MGETN8
; KEYBOARD ROUTINE
MGETN6  INC  C            ; POT MODE?
        JR   NZ,MGETN7    ; JUMP IF NOT
        CALL CLRNUM
        INC  C            ; SET ONE DIGIT SO FAR
MGETN7  SET  7,C          ; SET POT LOCKOUT
        SYSTEM KCTASC
        CP   '='          ; EQUALS TYPED?
        JR   Z,MGETN9     ; QUIT IF EQUALS
        AND  0FH
        EXX
        SYSTEM SHIFTU     ; SHIFT DIGIT UP
MGETN8  PUSH DE
        SYSTEM DISNUM
; ENTER HERE FOR EQUAL OR TRIGGER EXIT TO THROW OUT RETURN
MGETN9  POP  DE
        EXX               ; BACK TO NORMAL
        RET
;
; SUBROUTINE TO CLEAR NUMBER
CLRNUM  PUSH BC
        EXX               ; TO NORMAL SET
        PUSH HL
        LD   A,B
        INC  A
        AND  3EH
        RRA
        EXX               ; BACK TO NORMAL SET
        LD   C,A
        XOR  A
        LD   B,A
        POP  DE
        SYSTEM FILL
        POP  BC
        RET
;
; NAME:         SHIFT UP
; INPUT:        A = DATA TO SHIFT UP
;               B = SIZE IN DIGITS
;               HL = AREA TO SHIFT ADDRESS
MSHFTU  PUSH AF
        LD   A,B
        INC  A
        AND  3EH
        LD   B,A
        POP  AF
SHFTU1  RLD
        INC  HL
        DJNZ SHFTU1
        RET
;
ENTSTG  DB   'ENTER '
        DB   0
CML     DW   CALCL
        DW   PNCM
        DW   CMSTRT       ; CHECKMATE START
SCBL    DW   0
        DW   PNSCB
        DW   SCBST
PNGF    DB   'GUNFIGHT'
        DB   0
PNCM    DB   'CHECKMATE'
        DB   0
PNCALC  DB   'CALCULATOR'
        DB   0
PNSCB   DB   'SCRIBBLING'
        DB   0
GAMSTR  DB   'SELECT GAME'
        DB   67H
        DB   8
        DB   88
        DB   1101B
        DB   '(C) BALLY '
        DB   'MFG 1978'
        DB   0
; FOUR FILLER BYTES BETWEEN ROM AND SCRIBBLING
; TO MATCH 3159 ROM
        DB   0FFH, 0FFH, 0FFH, 0FFH
;
; **********
; * SKETCH * Ver. 1.13 - Nov. 30, 2003
; **********
;
; The official name of this
; program is Scribbling
;
; zmac -i -m -o scribble.bin -x scribble.lst scribble.asm 
; Sketch equates
; Sketch packet displacements
SCPSIZ  EQU  30           ; Size of sketch packet
SCSAVA  EQU  0            ; Save area start
SCXC    EQU  26           ; X coordinate
SCYC    EQU  27           ; Y coordinate
SCSADL  EQU  28           ; Save address Lo and Hi
SCSADH  EQU  29
; Other goodies
MOVTMR  EQU  4            ; Move rate
KSCTRV  EQU  20           ; Color stepping time
        ORG  $E19         ; ** Start
BEGIN:  SYSSUK GETPAR
        DW   NOPLAY
        DB   1
        DW   NUMPLY
SCCLR:
        LD   SP,SCRSTK
        SYSTEM INTPC
        DO   FILL
        DW   NORMEM
        DW   92*BYTEPL
        DB   0
        DO   FILL
        DW   P1SCP
        DW   SCPSIZ*4
        DB   0
        DO   SETOUT
        DB   184
        DB   40
        DB   8
        DO   MOVE
        DW   COLORS
        DW   8
        DW   INICOL
        DO   COLSET
        DW   COLORS
        DO   SETW
        DB   70
        DB   36
        DW   P1SCP+SCXC
        DO   SETW
        DB   83
        DB   36
        DW   P2SCP+SCXC
        DO   SETW
        DB   70
        DB   48
        DW   P3SCP+SCXC
        DO   SETW
        DB   83
        DB   48
        DW   P4SCP+SCXC
        DO   SETB
        DB   MOVTMR
        DW   CT0
        DONT XINTC
MAINLP: LD   HL,MAINLP
        PUSH HL
        SYSSUK SENTRY
        DW   KEYMES
        SYSSUK DOIT
        DW   SCDOTB
        RET
KEYMES: DB   $2F
        DB   $F
        DB   $F
        DB   $F
; Keyboard handler
KEYB0:  DEC  B
        LD   C,3
        LD   A,B
        CP   20           ; Clear entry down?
        JR   Z,SCCLR      ; Jump to clear if so
        RRCA
        RRCA
        AND  C
        SYSSUK INDEXB
        DW   CDELTB
        EX   DE,HL
        LD   A,B
        AND  C
        LD   H,A
        LD   A,C
        SUB  H
        SYSSUK INDEXB      ; Point at color
        DW   COLORS
        LD   A,(DE)
        ADD  A,(HL)        ; Add delta factor
        BIT  3,B           ; Was key for intensity?
        JR   Z,KEYB1
        XOR  (HL)
        AND  7
        XOR  (HL)
KEYB1:  LD   (HL),A
        INC  HL            ; Change color on other side
        INC  HL
        INC  HL
        INC  HL
        LD   (HL),A
        SYSSUK COLSET
        DW   COLORS
        LD   A,KSCTRV      ; Set KEYSEX clear timer
        LD   (CT1),A
        RET
; Routine to clear KEYSEX
KLRKSX: XOR  A
        LD   (KEYSEX),A
        RET
SCDOTB: JMPd SCT0,DOWRTS,0
        JMPd SCT1,KLRKSX,0
        JMPd SKYD,KEYB0,0
; Iterate through active players subroutine
ITER4:  LD   IX,P1SCP
        LD   A,(NUMPLY)
        LD   B,A
        LD   C,A
ITER41: PUSH BC
        PUSH HL
        LD   DE,ITRET
        PUSH DE
        JP   (HL)
ITRET:  LD   DE,SCPSIZ
        ADD  IX,DE
        POP  HL
        POP  BC
        DJNZ ITER41
        RET
; Update Coordinates Routine
SCRUPD: LD   A,C
        SUB  B
        SYSSUK INDEXB
        DW   OSW0
        AND  $F
        CALL GETDLT       ; Get deltas
        LD   A,(IX+SCXC)  ; Update X
        ADD  A,D
        CP   152          ; Out of bounds?
        JR   NC,SCRUP1
        LD   (IX+SCXC),A
SCRUP1: LD   A,(IX+SCYC)  ; Same for Y
        ADD  A,H
        CP   85
        RET  NC
        LD   (IX+SCYC),A
        RET
; Restore
SCREST: PUSH IX
        POP  DE
        LD   A,(DE)
        AND  A
        RET  Z
        LD   H,(IX+SCSADH)
        LD   L,(IX+SCSADL)
        SYSTEM RESTOR
        XOR  A
        LD   (DE),A
        RET
 ; Write Routine
SCRWRT: LD   A,C
        SUB  B
        SYSSUK INDEXB
        DW   OSW0
        AND  $10
        RET  Z
SCRWR1: DEC  HL           ; Backup to pot
        DEC  HL
        DEC  HL
        DEC  HL
        DEC  HL
        LD   A,(HL)
        RLCA
        RLCA
        LD   C,A
        AND  3
        SYSSUK INDEXB     ; Set sizes
        DW   SIZTBL
        LD   D,(IX+SCYC)
        LD   E,(IX+SCXC)
        LD   B,A
        LD   A,C
        RLCA
        RLCA
        AND  3
SCRWR2: SYSSUK INDEXB
        DW   COLMSK
        LD   C,B
        SYSTEM RECTAN
        RET
COLMSK: DB   0
        DB   01010101B
        DB   10101010B
        DB   11111111B
SIZTBL: DB   1
        DB   2
        DB   4
CDELTB: DB   8
        DB   -8
        DB   1
        DB   -1
; Save Routine
SCRSAV: LD   A,B
        SYSSUK INDEXB
        DW   OSW0-1
        AND  $10
        RET  NZ
        PUSH HL
        LD   D,(IX+SCYC)
        LD   E,(IX+SCXC)
        SYSTEM RELAB1
        LD   (IX+SCSADH),D
        LD   (IX+SCSADL),E
        EX    DE,HL
        PUSH  IX
        POP   DE
        LD    BC,$0803     ; Save worst case
        SYSTEM SAVE
        POP   HL
        JR    SCRWR1
; Zero Player Game Write Handler
ZEROPL: LD    HL,ZPSTMR    ; LOAD PTR TO SIZE TIMER
        LD    DE,ZPSIZ     ; AND SIZE TRACKER
        DEC   (HL)         ; DECREMENT SIZE TIMER
        JP    P,ZPA        ; JUMP IF NO COUNTDOWN
        SYSSUK RANGED
        DB    48
        CP    8            ; 8-479 ???
        JR    C,ZP0        ; NO - ZP0
        AND   3            ; YES - HAVE MORE 1-4S
ZP0:    INC   A
        LD    (DE),A       ; GET NEW SIZE
        SYSSUK RANGED      ; GET NEW SIZE TIMER
        DB    120
        LD    (HL),A
ZPA:    INC   HL           ; ADVANCE  TO COLOR STUFF
        INC   DE
        DEC   (HL)         ; AND DEC COLOR TIMER
        JP    P,ZPB
        SYSSUK RANGED      ; GET NEW COLOR
        DB    4
        LD    (DE),A
        SYSSUK RANGED      ; GET NEW COLOR TIMER
        DB     120
        LD     (HL),A
ZPB:    INC    HL          ; TO DIRECTION STUFF
        INC    DE
        DEC    (HL)        ; DECREMENT DIRECTION TIMER
        JP     P,ZPD
ZPC:    LD     DE,DIRVAL   ; DE = DIRECTION TRACKER
        SYSSUK RANGED      ; DRAW NEW DIRECTION
        DB     10
        INC    A
        CP     3           ; REJECT ILLEGAL MOVES
        JR     Z,ZPC
        CP     7
        JR     Z,ZPC
        LD     (DE),A
        SYSSUK RANGED
        DB     40
        LD     (DIRTMR),A
ZPD:    LD     A,(DE)     ; GET DIRECTION VALUE
        CALL   GETDLT     ; GET DELTAS
        LD     BC,P1SCP+SCXC ; POINT AT COORDINATE
        LD   A,(BC)
        ADD  A,D
        CP   80
        JR   NC,ZPC       ; GET NEW DIRECTION IF AT LMT
        LD   (BC),A
        LD   E,A          ; SAVE X COORDINATE
        INC  BC
        LD   A,(BC)
        ADD  A,H
        CP   46
        JR   NC,ZPC
        LD   (BC),A
        LD   D,A          ; SET Y COORDINATE
        LD   HL,ZPSIZ     ; POINT AT SIZES AGAIN
        LD   B,(HL)
        INC  HL
        LD   A,(HL)       ; GET COLOR TOO
        CALL SCRWR2       ; DO FIRST WRITE
        LD   H,A          ; SAVE COLOR
        PUSH DE           ; AND X,Y
        LD   A,92         ; REFLECT Y
        SUB  B
        SUB  D
        LD   D,A
        LD   A,H
        SYSTEM RECTAN
        LD   A,160        ; REFLECT X
        SUB  C
        SUB  E
        LD   E,A
        LD   A,H
        SYSTEM RECTAN
        POP  HL           ; RESTART X,Y
        LD   D,H          ; RESTORE Y
        SYSTEM RECTAN
        LD   A,$FF        ; RESET TIMEOUT
        LD   (TIMOUT),A
ZERO1:  LD   A,1          ; RESET COUNTER-TIMER
        JR   ZERO2
DOWRTS: LD   A,(NUMPLY)
        DEC  A
        CP   4
        JP   NC,ZEROPL
        LD   HL,SCRUPD
        CALL ITER4
        LD   HL,SCREST
        CALL ITER4
        LD   HL,SCRWRT
        CALL ITER4
; NOW GOING BACKWARDS SAVE AND WRITE EVERYBODY WITH TRIGG
        LD   B,C
SCRB3:  LD   DE,-SCPSIZ
        ADD  IX,DE
        PUSH BC
        CALL SCRSAV
        POP  BC
        DJNZ SCRB3
        LD   A,MOVTMR
ZERO2:  LD   (CT0),A
        RET               ; DONE
; SUBROUTINE TO SCARE UP DELTAS
GETDLT: PUSH BC
        LD   B,A
        SYSSUK MSKTD
        DW   $100
        DB   0
        DW   $100
        POP  BC
        RET
; INITIAL COLORS
INICOL: DB   $8
        DB   $62          ; Value changed from $5B to match
        DB   $A5          ; exactly with "3159" ROM 
        DB   $7
        DB   $8
        DB   $62          ; Value changed from $5B to match 
        DB   $A5          ; exactly with "3159" ROM 
        DB   $7
;        ORG  $4000+3720 - Note - DEFS statements removed
; SWITCH RAM:             Original Source Syntax 
;        DEFS 96          ----------------------
SCRSTK  EQU  $4EE8        ; SCRSTK:
COLORS  EQU  $4EE8        ; COLORS: DEFS 8
ZPSTMR  EQU  $4EF0        ; ZPSTMR  EQU  $
DIRTMR  EQU  ZPSTMR+2     ; DIRTMR  EQU  ZPSTMR+2
ZPSIZ   EQU  DIRTMR+1     ; ZPSIZ   EQU  DIRTMR+1
DIRVAL  EQU  ZPSIZ+2      ; DIRVAL  EQU  ZPSIZ+2
P1SCP   EQU  $4EF0        ; P1SCP:  DEFS SCPSIZ
P2SCP   EQU  $4F0E        ; P2SCP:  DEFS SCPSIZ
P3SCP   EQU  $4F2C        ; P3SCP:  DEFS SCPSIZ
P4SCP   EQU  $4F4A        ; P4SCP:  DEFS SCPSIZ
;
;       END OF SCRIBBLING PROGRAM
;
; TWELVE FILLER BYTES BETWEEN SCRIBBLING AND
; CALCULATOR TO MATCH 3159 ROM
        DB   0FFH, 0FFH, 0FFH, 0FFH, 0FFH
        DB   0FFH, 0FFH, 0FFH, 0FFH, 0FFH
        DB   0FFH, 0FFH
;
; **************
; * CALCULATOR * (Dec 4, 2003)
; **************  
;
; CALCULATOR EQUATES
; SCREEN PARAMETERS
CALCLN  EQU  92           ; LAST DISPLAY LINE USED
; SCREEN MEMORY STACK PARAMETERS
MAXSTK  EQU  CALCLN       ; MAX NUMBER OF STACK ENTRYS
STKBOT  EQU  MAXSTK*BYTEPL+4000H-8 ; ADDR OF STK BOTTOM
; CALCULATOR MAJOR STATES
ARG1MD  EQU  0            ; ARG1 STATE
ARG2MD  EQU  1            ; ARG2 STATE
SHOWMD  EQU  2            ; SHOW STATE
;
        ORG  1020H        ; ***
CLEAR   LD   SP,CALSTK    ; BUCHWACK THE STACK
        SYSTEM INTPC      ; INTERPRET
        DO   FILL
        DW   4000H
        DW   CALCLN*BYTEPL
        DB   0
        DO   RECTAN
        DB   8
        DB   82
        DB   112
        DB   9
        DB   11111111B
        DO   SETOUT
        DB   CALCLN*2
        DB   31
        DB   8
        DO   COLSET
        DW   CALCLR       ; CALCULATOR COLORS
        DO   FILL
        DW   CALRAM
        DW   ENDCR-CALRAM
        DB   0
        DO   ACTINT
        DO   XINTC        ; QUIT INTERPRETING
        CALL DOCLRE       ; SHOW A ZERO WHEN WE COME UP
; GET NEXT KEY DEPRESSION
KEYGET  LD   HL,KEYGET
        PUSH HL
        SYSSUK SENTRY
        DW   ALKEYS
        CP   SKYD         ; KEY DOWN?
        RET  NZ           ; NO - WAIT
        LD   A,B
        SYSSUK INDEXB
        DW   TOKETB-1
        AND  0FH          ; ISOLATE 'TRANSLATE TO' FIELD
        LD   C,A
        XOR  (HL)         ; BRANCH ON TOKEN TYPE
        RRCA
        RRCA
        RRCA
        RRCA
        LD   E,A
; ARE WE WAITING FOR USER TO TYPE IN A MEMORY NUMBER?
        LD   HL,MEMSTA    ; POINT AT MEMORY STATE
        LD   A,(HL)
        AND  A
        JR   Z,KEYGT1     ; JUMP IF NOT IN THAT STATE
        LD   A,E          ; WAS   DIGIT TYPED?
        AND  A
        RET  NZ           ; NO - GO TO NEXT ENTRY
        LD   D,A          ; ZERO OUT D
        LD   A,C
        CP   0EH          ; DID OPERATOR HIT A DECIMAL PO
        RET  Z            ; YES - QUIT
; SET MEMORY DIGIT FOR DISPLAY
        INC  HL
        LD   (HL),C
        INC  (HL)         ; ZERO DESIGNATES NOTHIN  
        LD   E,A
        DEC  HL
        LD   A,(HL)       ; S-R STATUS TO A AGAIN
        RRCA              ; TEST STATUS
        JR   C,GMEM1      ; JUMP IF RECALL
; STORE PROCESSING
        CALL XMEM         ; DO POINTING AND SETUP
        LDIR              ; PERFORM STORE
        JR   SHONUM
; RECALL PROCESSING
GMEM1   CALL SHOWCK       ; CHECK FOR SHOW MODE
        CALL XMEM
        EX   DE,HL
        LDIR              ; COPY MEM TO ARG2
SHONUM  LD   HL,0
        LD   (DIGITS),HL  ; CLEAR DIGIT FLAGS
        LD   HL,SHOWBF
        PUSH HL
        CALL CONVRT
        POP  HL
        JP   ADIG7        ; JOIN DISPLAY SEQUENCE
KEYGT1
        LD   A,E
        SYSSUK INDEXW
        DW   TOKEJT
        PUSH DE
        LD   HL,STATEV
        LD   A,(HL)
        RET               ; CALL VIA RETURN
; TOKEN TYPE-TRANSLATE TABLE
; HO NIBBLE IS TYPE, LO NIBBLE IS WHAT TO TRANSLATE TO
TOKETB  DB   80H          ; 1 C
        DB   60H          ; 2 VA
        DB   61H          ; 3 DA
        DB   31H          ; 4 %
        DB   51H          ; 5 MR
        DB   52H          ; 6 MS
        DB   40H          ; 7 +-
        DB   14H          ; 8 DIVIDE
        DB   07H          ; 9 7
        DB   08H          ; 10 8
        DB   09H          ; 11 9
        DB   13H          ; 12 X
        DB   04H          ; 13 4
        DB   05H          ; 14 5
        DB   06H          ; 15 6
        DB   12H          ; 16 -
        DB   01H          ; 17 1
        DB   02H          ; 18 2
        DB   03H          ; 19 3
        DB   11H          ; 20 +
        DB   70H          ; 21 CE
        DB   00H          ; 22 0
        DB   0EH          ; 23 .
        DB   20H          ; 24 =
; TOKEN TYPE JUMP TABLE
TOKEJT  DW   ACTNUM       ; 0 NUMBER
        DW   OPROUT       ; 1 OPERATOR
        DW   EQROUT       ; 2 EQUAL SIGN
        DW   PERCEN       ; 3 PERCENT
        DW   CHGSGN       ; 4 CHANGE SIGN
        DW   MEMFUN       ; 5 MEMORY
        DW   SCRLKY       ; 6 SCROLL KEYS
        DW   CLRENT       ; 7 CLEAR ENTRY
        DW   CLEAR        ; 8 CLEAR CALC
; SUBROUTINE TO POINT DE AT A MEMORY AND LOAD HL
; INPUT   D=0,E=DIGIT (0 TO 9 MEMORY #)
; OUTPUT  DE = POINTER TO MEM HL=ARG2 BC=12
XMEM    LD   HL,CMEM0
        LD   B,11
        LD   C,B
XMEM1   ADD  HL,DE
        DJNZ XMEM1
        EX   DE,HL
        LD   HL,ARG2
        RET
; MEMORY KEY ROUTINE
MEMFUN  LD   HL,MEMSTA    ; SET MEMORY STATE
        LD   (HL),C       ; SET MEMORY STATE
        JR   EQRT1
; CHANGE SIGN ROUTINE
CHGSGN  CALL SHOWCK
        SYSSUK BCDCHS
        DB   11
        DW   ARG2
        JR   EQRT1
; OPERATOR ROUTINE
OPROUT  CP   SHOWMD       ; ARE WE IN SHOW STATE?
        JR   NZ,OP1       ; JUMP IF NOT
        LD   (HL),ARG2MD  ; GO TO ARG2 STATE
        JR   OP3
OP1     CP   ARG1MD       ; ARG1 MODE?
        EX   AF,AF'
        LD   A,C
        EX   AF,AF'
        JR   NZ,OP2       ; JUMP IF NOT
        LD   (HL),ARG2MD  ; YUP, GO TO ARG2 STATE
        CALL PUSHEN       ; PUSH ARG1 DOWN
        SYSSUK MOVE
        DW   ARG1
        DW   11
        DW   ARG2
        JR   OP2A
OP2     CALL DOOP         ; DO IT
OP2A    EX   AF,AF'
        LD   C,A
OP3     LD   A,C
        LD   (OPCOD),A
        JR   EQRT1        ; JOIN EQUAL DISPLAY CALL
; EQUALS ROUTINE
EQROUT  DEC  A            ; ARE WE IN ARG2MD?
        RET  NZ
        LD   (HL),SHOWMD  ; ENTER SHOW STATE
        CALL DOOP
        LD   A,5
        LD   (OPCOD),A
        CALL PUSHEN
EQRT1   JP   SHONUM
PERCEN  XOR  C            ; ARG2 MODE?
        RET  NZ
        SYSSUK MOVE
        DW   ARG2
        DW   9
        DW   ARG2+1
        LD   (ARG2+9),A
        LD   HL,ARG1
        LD   B,11
        LD   A,(ARG1+10)  ; SAVE SIGN OF ARG1
        SYSTEM BCDMUL
        LD   (ARG1+10),A  ; RESTORE SIGN OF ARG1
        EX   DE,HL
        LD   BC,11
        CALL OVRCHK       ; CHECK FOR OVERFLOW
        JR   EQRT1
; SUBROUTINE TO DO OPERATION
; NOTE THIS ROUTINE SETS BOTH ARG1 AND ARG2 EQUAL TO RESU
DOOP    CALL PUSHEN
        LD   A,(OPCOD)
        SYSSUK INDEXW
        DW   DOOPTB-2
        PUSH DE
        LD   HL,ARG2
        LD   DE,ARG1
        LD   B,11
        RET               ; JUMP USING RETURN
DOOPTB  DW   DOADD
        DW   DOSUB
        DW   DOMUL
        DW   DODIV
DOADD   SYSTEM BCDADD
        DB   01
DOSUB   SYSTEM BCDSUB
        DB   01
DOMUL   SYSTEM BCDMUL
        DB   01
DODIV   SYSTEM BCDDIV
NUMCHK  SYSSUK MOVE       ; ARG2 = ARG1
        DW   ARG2
        DW   11
        DW   ARG1
OVRCHK  ADD  HL,BC
        DEC  HL
        LD   A,(HL)
        AND  0FH          ; OVERFLOW SET?
        RET  Z            ; QUIT IF NOT
        LD   A,1
        LD   (HL),A
        INC  A            ; FOR SHOW MODE
        LD   (STATEV),A
        CALL PUSHEN
        POP  HL           ; THROW OUT RETURN ADDRESS
DOCLRE  LD   C,0          ; FORCE CLEAR ENTRY
        JR   CLRENT
; SUBROUTINE TO ACCEPT A DIGIT
; ONE DIGIT
ACTNUM  CALL SHOWCK
        LD   HL,DIGITS
        LD   DE,ARG2
        LD   A,(HL)       ; HOW MANY DIGITS SO FAR?
        CP   10
        RET  Z            ; QUIT IF LIMIT ATTAINED
        INC  HL           ; DECIMAL POINT ENTERED YET?
        LD   A,(HL)
        AND  A
        JR   Z,ADIG1      ; JUMP IF NO
        CP   C            ; IS INPUT A DECIMAL POINT?
                                ; (FLAG = 0E IF SET!!)
        RET  Z            ; QUIT IF EXTRA
        LD   A,C
        INC  HL
        LD   C,(HL)       ; C = DECIMAL POINT
        DEC  (HL)
        EX   DE,HL
        SYSTEM STOREN     ; STORE THE NIBBLE
        LD   C,A          ; GET DIGIT BACK IN C
        EX   DE,HL
        DEC  HL
        DEC  HL
        JR   ADIG4        ; GO ADD TO SHOW BUFFER
; PRE DECIMAL POINT
ADIG1   DEC  HL           ; BACKUP TO DIGITS
        OR   (HL)         ; HOW MANY ENTRYS?
        JR   NZ,ADIG2     ; ADIG2 IF NOT ZERO
CLRENT  CALL SHOWCK       ; PUSH DOWN IF RESULT IS THERE
        PUSH BC           ; CLEAR ARG2, SHOWBUF AND VARS
        SYSSUK FILL
        DW   ARG2
        DW   21
        DB   0
        LD   HL,DIGITS
        POP  BC
        OR   C            ; WAS DIGIT ZERO?
        JR   Z,ADIG6      ; JUMP IF SO
; CODE FOR NONZERO PREVIOUS ENTRY OR NONZERO FIRST ENTRY
ADIG2   LD   A,C
        CP   0EH          ; WAS CHAR A DECIMAL POINT?
        JR   NZ,ADIG3     ; NO   ADIG3
        INC  HL           ; TO DECIMAL FLAG
        LD   (HL),A       ; SET IT
        INC  HL
        LD   (HL),9       ; SET DECIMAL POINT POINTER
        JR   ADIG5        ; GO STUFF INTO SHOW BUFFER
ADIG3   INC  DE           ; LEADING NUMERIC - STUFF IT
        INC  DE
        INC  DE
        INC  DE
        INC  DE
        CALL ADDTB
ADIG4   INC  (HL)         ; BUMP DIGIT COUNTER
ADIG5   LD   A,C          ; GET DIGIT BACK
        LD   DE,SHOWBF    ;ADD DIGIT TO SHOW BUFFER
        CALL ADDTB
ADIG6   EX   DE,HL
ADIG7   XYRELL DE,8,83
        LD   C,0111B      ; DISPLAY ALWAYS POSITIVE
; DISPLAY RECALL OR STORE?
        PUSH HL
        LD   HL,MEMSTA
        LD   A,(HL)
        AND  A
        JR   Z,ADIG8      ; ADIG8 IF NOT WANTED
        ADD  A,'R'-21H
ADIG8   ADD  A,20H
        SYSTEM CHRDIS
; SAME THING FOR STORE-RECALL DIGIT
        INC  HL
        LD   A,(HL)
        AND  A
        JR   Z,ADIG9
        ADD  A,'0'-21H
        LD   (HL),0       ; CLEAR DIGIT AND RECALL MODE
        DEC  HL
        LD   (HL),0
ADIG9   ADD  A,20H
        SYSTEM CHRDIS
        POP  HL
        LD   A,(OPCOD)
        LD   C,0011B
        JP   NUMBA
; SUBROUTINE TO CHECK TO SEE IF WE ARE IN SHOW MODE
; IF SO - PUSH ARG2 AND GO INTO ARG1 MODE
SHOWCK  LD   HL,STATEV
        LD   A,(HL)
        CP   SHOWMD
        RET  NZ
        XOR  A
        LD   (HL),A
        DEC  HL
        LD   (HL),A       ; CLEAR OPERATOR
        RET
; SUBROUTINE TO ADD AN ENTRY TO THE STACK
PUSHEN  SYSSUK SCROLL     ; PUSH STACK UP
        DW   BYTEPL
        DB   8
        DB   MAXSTK
        DW   4020H        ; (FIRST LINE)
        LD   HL,STKBOT
        CALL CONVRT       ; CONVERT AND ADD TO STACK
        LD   HL,NLINES    ; HOW MANY ENTRYS?
        LD   A,(HL)       ; ON THE STACK?
        CP   MAXSTK       ; AT MAX?
        JR   Z,PUSHE1     ; JUMP IF SO
        INC  (HL)         ; BUMP ENTRY COUNT
; ARE WE AT TOP OR BOTTOM OF STACK?
PUSHE1  INC  HL           ; POINT AR SCHOLL POINTER
        LD   A,(HL)
        AND  A            ; JUMP IF AT BOTTOM
        JR   Z,SCRLU1
        CP   MAXSTK-10    ; OR IF AT TOP
        JR   Z,SCRLU1     ; FIX SCROLL POINTER
        INC  (HL)         ; FIX SCROLL POINTER
        RET               ; AND QUIT
; SCROLL KEY ENTRY
SCRLKY  XOR  A            ; CLEAR KEYSEX FOR SCROLL MOTOR
        LD   (KEYSEX),A
        LD   HL,NLINES
        LD   A,(HL)       ; A = NUMBER OF LINES ON STK
        BIT  0,C          ; UP OR DOWN
        JR   NZ,SCRLDN    ; JUMP IF DOWN
        AND  A            ; QUIT IF ZERO LINES ON STACK
        RET  Z
        INC  HL           ; WHERE'S THE SCROLL POINTER?
        LD   A,(HL)
        AND  A            ; QUIT IF AT BOTTOM
        RET  Z
        DEC  (HL)         ; ELSE DECREMENTETH
; PUSHEN JOINS HERE
SCRLU1  LD   A,8          ; DO IT 8 TIMES
SCRLU2  SYSSUK SCROLL
        DW   BYTEPL
        DB   32
        DB   79
        DW   4028H
        DEC  A
        JR   NZ,SCRLU2
        INC  A
        XYRELL DE,24,73
        JR   SCRLD1       ; JOIN SCRLDN DISPLAY CALL
; SCROLL DOWN ROUTINE
SCRLDN  INC  HL           ; ADVANCE THE SCROLL POINTER
        SUB  (HL)         ; SUBTRACT FROM LINES ON STACK
        CP   11           ; FAR ENOUGH APART?
        RET  C            ; QUIT IF NOT
        INC  (HL)         ; YEAH - ADVANCE SCROLL POINTER
        LD   A,8
SCRLD0  SYSSUK SCROLL
        DW   -BYTEPL
        DB   32
        DB   80
        DW   4000H+3200
        DEC  A
        JR   NZ,SCRLD0
        LD   A,10
        XYRELL DE,24,1
SCRLD1  LD   HL,SCRPTR
        ADD  A,(HL)       ; ADD SCROLL PTR
        LD   HL,STKBOT+BYTEPL
        LD   BC,-BYTEPL
SCRLD2  ADD  HL,BC
        DEC  A
        JR   NZ,SCRLD2
        LD   C,13         ; GET OPERATION
        SYSTEM INDEXN
        LD   C,0          ; WRITE WITH BLACK BACKGROUND
NUMBA   PUSH AF
        LD   B,C
        LD   C,12
        SYSTEM INDEXN
        BIT  1,A
        JR   NZ,OVFERR
        BIT  0,A
        LD   A,0100B
        JR   Z,NUMBA1
        LD   A,1000B
NUMBA1  OR   B
        LD   C,A
        LD   B,8BH
        SYSTEM DISNUM
        POP  AF
        SYSSUK INDEXB
        DW   OPTBL
        SYSTEM CHRDIS
NULKEY  RET
OVFERR
        LD   C,B
        SET  3,C
        LD   HL,OVFMSG
        SYSTEM STRDIS
        POP  AF
        RET
OVFMSG  DB   ' *OVERFLOW* '
        DB   0
OPTBL   DB   ' '          ; NULL OP
        DB   '+'
        DB   '-'
        DB   62H
        DB   63H
        DB   '='
CONVRT  LD   DE,ARG2
        LD   A,(OPCOD)
        LD   C,13
        SYSTEM STOREN     ; STORE THAT NIBBLE
; TRANSFER SIGN
        LD   A,(ARG2+10)  ; GET SIGN BYTE
        RLCA              ; PUT SIGN IN LOW BIT
        DEC  C
        SYSTEM STOREN
PACKER  EX   DE,HL        ; DE = SAVE HL = ARG
        LD   BC,6         ; CLEAR SAVE
        XOR  A
        SYSTEM FILL
        LD   C,19
CCVT1   SYSTEM INDEXN     ; GET NIBBLE
        CALL ADDTB        ; ADD INTO 'SAVE'
        JR   NZ,CCVT1A    ; JUMP IF DONE
        DEC  C            ; DECREMENT ARGPTR
        LD   A,C
        CP   9            ; IS IT NOW 9
        JR   NZ,CCVT1     ; NO - KEEP GOING
        LD   A,0EH        ; YEAH - PUT IN DECIMAL POINT
        CALL ADDTB
        JR   Z,CCVT1      ; JUMP BACK IF OKAY
; NOW REMOVE TRAILING ZEROS AND DECIMAL POINT
CCVT1A  EX   DE,HL        ; HL = SAVE
CCVT2   LD   A,(HL)       ; GET LO NIBBLE
        AND  0FH          ; IS IT NONZERO?
        JR   NZ,CCVT3     ; JUMP IF NOT ZERO
        CALL KILTRZ       ; REMOVE ONE TRAILING ZERO
        JR   CCVT2
CCVT3   CP   0EH          ; DECIMAL POINT
        RET  NZ           ; QUIT IF NOT
KILTRZ  LD   BC,6
        ADD  HL,BC
        AND  A
        LD   B,C
KILTR1  DEC  HL
        RRD
        DJNZ KILTR1
        RET
; SUBROUTINE TO ADD A DIGIT TO NUMBER POINTED AT BY DE
ADDTB   LD   B,5
ADDTD   EX   DE,HL
        PUSH HL
ADDTB1  RLD
        INC  HL
        DJNZ ADDTB1
        LD   (HL),A       ; STUFF LAST
        POP  HL
        EX   DE,HL
        RET
; COLORS FOR CALCULATOR
CALCLR  DB   76H          ; GREY
        DB   62H          ; RED
        DB   00H          ; BLACK
        DB   07H          ; WHITE
        DW   0707H
        DW   0707H
; ******************
; * CALCULATOR RAM *
; ******************
; ORG 4000H
; DEFS CALCLN*BYTEPL
; DEFS 96
        CALSTK EQU $4EC0
        CALRAM EQU $4EC0
        OPCOD  EQU $4EC0  ; OPERATION TO DO
        STATEV EQU $4EC1  ; STATE VARIABLE
        MEMSTA EQU $4EC2  ; MEMORY STATE VARIABLE
        MEMDIG EQU $4EC3  ; MEMORY DIGIT VARIABLE
        ARG1   EQU $4EC4  ; ARGUMNET 1 - 11 BYTES
; *** NOTE *** CODE EXPECTS ARG2 THRU SHOWBF TO BE IN ONE
        ARG2   EQU $4ECF  ; ARGUMENT 2 - 11 BYTES
        DIGITS EQU $4EDA  ; DIGITS COUNTER
        DPOINT EQU $4EDB  ; DECIMAL POINT ENTERED FLAG
        POINTP EQU $4EDC  ; DECIMAL POINT POINTER
        SHOWBF EQU $4EDD  ; ENTRY FEEDBACK BUFFER - 7 BYTES
        NLINES EQU $4EE4  ; NUMBER OF LINES ON SCROLL STA
        SCRPTR EQU $4EE5  ; CURRENT POS OF SCROLL SYSTEM
        CMEM0  EQU $4EE6  ; CALCULATOR MEMORYS - 110 BYTES
        ENDCR  EQU $4F54
;
;       END OF CALCULATOR PROGRAM
;
; FOUR FILLER BYTES BETWEEN SCRIBBLING AND
; CALCULATOR TO MATCH 3159 ROM
        DB   0FFH, 0FFH, 0FFH, 0FFH
        ;
        ;*****************************
        ;* H V G   C H E C K M A T E *  VER. 2.0 4-2-2003 
        ;*****************************
        ;
        ;
        ; M A C R O S
        ;
DEF4X   MACRO A4X, B4X, C4X, D4X
        DB   A4X 
        DB   B4X 
        DB   C4X 
        DB   D4X 
        ENDM 
WRECK   MACRO
        DW   (9 SHL 8)+32
        DB   0000B
        ENDM
        ;
        ;
        ; E Q U A T E S
        ;
OLDWAY  EQU 1-NWHDWR      ;1=DO OLD WAY 0=DO NEW WAY
NEWWAY  EQU 1-OLDWAY      ;OPPOSITE OF OLDWAY
        ; VARIOUS 'EQU'S
RLMOVE  EQU  1100B        ;RIGHT AND LEFT MOVES
UDMOVE  EQU  0011B        ;UP AND DOWN MOVES
NOBIT   EQU  2            ;# OF GAME BITS
NPBIT   EQU  3            ;# PLAYERS BIT
ANIMAX  EQU  3            ;MAX # TICKS PER ANIMATION FRAM
XMAX    EQU  (BYTEPL-2)*4 ;MAX X COORD - CHANGED TO MATCH ROOM
YLINES  EQU  21           ;# VERT BLOCKS
LOWY    EQU  11           ;LOWEST Y COORD
YMAX    EQU  ((YLINES-1)*4)+LOWY ;MAX Y COORD
LOWX    EQU  0            ;LOWEST X COORD
AMOVE   EQU  8H           ;AN ARBITRARY MOVE
MUSVOL  EQU  09H          ;MUSIC VOLUME
TDOPT   EQU  100100B      ;TIME DISPLAY OPTIONS
CDOPT   EQU  01000100B    ;COUNT DOWN OPT
WRITOR  EQU  010000B      ;WRITE WITH MAGIC OR
        ; PLAYER PACKER OFFSETS
LASTSW  EQU  0            ;LAST SWITCH SETTING
LASTMV  EQU  1            ;LAST ACTUAL MOVE
CURSW   EQU  2            ;CURRENT SWITCH SETTING
AROT    EQU  3            ;ARROW ROTATION SETTING
ARRX    EQU  4            ;ARROW X COORD
ARRY    EQU  5            ;ARROW Y COORD
PSTAT   EQU  6            ;PLAYER STATUS
        ; PLAYER STATUS MASKS
ACTIVE  EQU  $80
HUMAN   EQU  $40
ACTBIT  EQU  7            ;1=ACTIVE 0=DEAD
HUMBIT  EQU  6            ;1=HUMAN 0=COMPUTER
        ;SCREEN TABS
XTAB1   EQU  ((BYTEPL/4)*4)
XTAB2   EQU  XTAB1*2
XTAB3   EQU  XTAB1*3
YTAB    EQU  (((YLINES-1)/4)*4)
YTAB1   EQU  YTAB+LOWY
YTAB2   EQU  (2*YTAB)+LOWY
YTAB3   EQU  (3*YTAB)+LOWY
        ;OFFSETS FOR EACH PLAYERS ROM DATA
;NOTE0   EQU  0            ;EACH DIRECTIONS NOTES
;NOTE1   EQU  1 ; THESE FOUR LINES COMMENTED OUT BECAUSE
;NOTE2   EQU  2 ; THEY INTERFERE WITH BALLYEQU.H AND ARE
;NOTE3   EQU  3 ; NOT USED ANYWAY- ADAM, 2-16-2002
PPATL   EQU  4            ;PLAYER PAT ADDR LOW
PPATH   EQU  5            ;PLAYER PAT ADDR HIGH
PCDOP   EQU  6            ;PLAYER CHAR DISP OPT
PSPOSX  EQU  7            ;X COORD OF PLAYER SCORE
PSPOSY  EQU  8            ;Y COORD OF PLAYER SCORE
PSDOP   EQU  9            ;PLAYER SCORE DISP OPT
        ;MORE EQU'S
FORCEM  EQU  $F6          ;VAL TO FORCE RANDOM MOVE
WIDTH   EQU  $4           ;# PIXELS WIDE OF PLAYER PAT
HEIGHT  EQU  $4           ;# PIXELS HIGH OF PLAYER PAT
ALLBYT  EQU  (YLINES*4)*BYTEPL ;ALL BYTES ON SCREEN  
STARTS  EQU  (LOWY*BYTEPL)+NORMEM ; LOWEST ADDR OF PLAY FI
PATXSZ  EQU  1            ;#BYTES WIDE OF PLAYER PATTERN
PATYSZ  EQU  4            ;#BYTES HIGH OF PLAYER PATTERN
PATDIM  EQU  (PATXSZ SHL 8) OR PATYSZ ;PATTERNS DIMENSIONS
JUSJOY  EQU  $F           ;ONLY JOY STICK BITS
CBLEN   EQU  8            ;COLOR BLOCK LENGTH
SBLEN   EQU  8            ;SOUND BLOCK LENGTH
WPONOF  EQU  0
WPOPT   EQU  1
WPPAL   EQU  2
WPPAH   EQU  3
WPXSIZ  EQU  5
WPYSIZ  EQU  4
        ;
        ;
        ;
;       ORG  NORMEM+0F96H ;SHOULD BE EQUAL TO RSTART
        ;UNCLEARED RAM - THESE ARE NOW EQUS, NOT DEFS
UNCRAM  EQU  $4F96
CURSCR  EQU  $4F96           ;ALL CURRENT SCORES
        ;CLEARED
CNOPL   EQU  $4FA2           ;CURRENT # PLAYERS
PLIX    EQU  $4FA3           ;WHO IS CURRENT PLAYER
CNOHUM  EQU  $4FA4           ;CURRENT # HUMANS
TARRX   EQU  $4FA5           ;TEMP ARROW X COORD
TARRY   EQU  $4FA6           ;TEMP ARROW X COORD
RMASK   EQU  $4FA7           ;ROTATE MASK
PPACKS:                      ;START OF PLAYER PACKETS
PLAY0   EQU  $4FA8
PLAY1   EQU  $4FAF
PLAY2   EQU  $4FB6
PLAY3   EQU  $4FBD
ENDRAM  EQU  $4FC4
RSTART  EQU  BEGRAM-(ENDRAM-UNCRAM)+1 ; SHOULD BE RAM STA
        ORG  1328H
ONETIM:
        ;ONE TIME ONLY HOUSEKEEPING
        LD   SP,UNCRAM
        SYSSUK GETPAR
        DW   NOGAME
        DB   $82
        DW   CT7
        SYSSUK GETPAR
        DW   NOPLAY
        DB   1
        DW   NUMPLY
        SYSSUK FILL
        DW   CURSCR
        DW   12
        DB   0
FIREIT: 
        ;REENTRY POINT FROM END OF GAME
        DI
        LD   SP,UNCRAM
        SYSTEM INTPC
        ;OUPUT COLOR BLOCK
        DO   COLSET
        DW   CBLOCK
        DO   EMUSIC
        ; CLEAR JOYSTICKS
        DO   FILL
        DW   OSW0
        DW   4
        DB   0
        ;CLEAR ALL RAM DATA
        DO   FILL
        DW   CNOPL
        DW   (PLAY3+PSTAT)-CNOPL+1
        DB   0
        DO   SETOUT
        DB   ((YLINES*4)+LOWY)*2  ; VER BLK
        DB   67H+0         ; HOR COL BND
        DB   08H           ; INTER MODE
        ; CLEAR SCORE BLOCKS
        DO   RECTAN
        DW   0
        DW   (11 SHL 8)+160
        DB   01010101B
        DO   RECTAN
        DW   $007A
        ; WRECK - CHANGED TO MATCH ROM
        DW   $091E
        DB   0000B 
        DO   RECTAN
        DW   $0057
        ; WRECK - CHANGED TO MATCH ROM
        DW   $091D
        DB   0000B 
        DO   RECTAN
        DW   $002B
        ; WRECK - CHANGED TO MATCH ROM
        DW   $091D
        DB   0000B 
        DO   RECTAN
        DW   $0004
        ; WRECK - CHANGED TO MATCH ROM
        DW   $0921
        DB   0000B 
        DO   ACTINT
        EXIT
        ; INITIALIZE STARTING ADDRESS OF ARROWS
        LD   HL,(YTAB2 SHL 8)+XTAB1
        LD   (PLAY0+ARRX),HL
        LD   HL,(YTAB2 SHL 8)+XTAB3
        LD   (PLAY1+ARRX),HL
        LD   HL,(YTAB1 SHL 8)+XTAB2
        LD   (PLAY2+ARRX),HL
        LD   HL,(YTAB3 SHL 8)+XTAB2
        LD   (PLAY3+ARRX),HL
        ; CLEAR FIELD
        CALL CLEARF
        ; DISPLAY # GAMES
        LD   IX,FNTSML
        SYSSUK DISNUM
        DB   76
        DB   2
        DB   TDOPT
        DB   42H
        DW   CT7
DONTD:
        ; GET # HUMANS
        LD   A,(NUMPLY)
        CP   5
        JR   C,GOTNPL
        LD   A,4
GOTNPL:
        LD   (CNOHUM),A
        ; GET # PLAYERS
        ;  IF HUMANS=1 OR 0 OR > 4 THEN PLAYERS=4 ELSE PLA
        CP   2
        JR   C,FPLAY
        CP   5
        JR   C,ALLHUM
FPLAY:  LD   A,4
ALLHUM: LD   (CNOPL),A
        ; INITIALIZE THE PLAYER PACKETS
        ; B=CURR # HUMANS
        ; C=CURR # PLAYERS
        ; D=THIS PLAYER #
INTIPP: LD   A,(CNOHUM)
        LD   B,A
        LD   A,(CNOPL)
        LD   C,A
        LD   D,0
GTPLIX: LD   A,D
        CALL LDPLIX
        PUSH BC
        PUSH DE
        LD   A,D
        ADD  A,$31        ;SETUP ASCII LITERAL
        LD   E,(IX+ARRX)
        LD   D,(IX+ARRY)  ;??? IS THIS RIGHT?
        DEC  E
        DEC  E
        LD   C,(IY+PCDOP)
        SYSTEM CHRDIS     ;DISPLAY PATTERN # ON FIELD
        LD   E,(IY+PSPOSX)
        LD   D,(IY+PSPOSY)
        PUSH DE
        SYSTEM CHRDIS     ;DISPLAY# ON SCORE BLOCK
        POP  DE
        LD   A,E
        ADD  A,6
        LD   E,A 
        INC  D
        INC  D
        LD   BC,(PATYSZ SHL 8) + PATXSZ
        LD   H,(IY+PPATH)
        LD   L,(IY+PPATL)
        LD   A,00010000B
        SYSTEM WRIT ;WRIT PLAYER PAT IN SCORE BLOCK
        POP  DE
        PUSH DE
        PUSH IX
        LD   A,D
        LD   B,0
        LD   C,D
        LD   HL,CURSCR
        ADD  HL,BC
        ADD  HL,BC
        ADD  HL,BC
        CALL DISPSC       ; DISPLAY SCORES
        POP  IX
        POP  DE
        POP  BC
        XOR  A
        OR   B
        JR   Z,NOTHUM
        LD   A,ACTIVE+HUMAN
        LD   (IX+PSTAT),A
        DEC  B
        JR   CKNOPL
CKSUM3: DB   0
NOTHUM: LD   A,ACTIVE
        LD   (IX+PSTAT),A
CKNOPL: INC  D
        DEC  C
        XOR  A
        OR   C
        JR   NZ,GTPLIX
        LD   A,3
CDOWNL:
        PUSH AF
        SYSSUK PAWS
        DB   5
        LD   (PLIX),A
        CALL UPMUZK       ; MAKE SOUND FOR COUNTDOWN
        POP  AF
        PUSH AF
        ADD  A,30H
        XYRELL DE,(XTAB2-4),(YTAB2-4)
        LD   C,CDOPT
        SYSTEM CHRDIS   ; DISPLAY COUNTDOWN #
        SYSSUK PAWS
        DB   40
        SYSTEM EMUSIC
        POP  AF
        DEC  A
        JR   NZ,CDOWNL
        CALL CLEARF
        ; INT TICK COUNT
        CALL TICKIT
        XOR  A
        LD   (CNT),A
LOOPY:
        SYSSUK SENTRY
        DW   ALKEYS
        SYSSUK DOIT
        DW   THETBL
        JR   LOOPY
THETBL: RC   SCT0,ACTION,0
        RC   SJ0,MOVJOY,0
        RC   SJ1,MOVJOY,0
        RC   SJ2,MOVJOY,0
        RC   SJ3,MOVJOY,0
        RC   SKYD,CALPIZ,ENDX

ACTION: 
        CALL TICKIT
        ; INCREMENT THE CURRENT PLAYER INDEX BY 1 UNTIL
        ; AN ACTIVE PLAYER IS FOUND THEN UPDATE HIM
INCIX:  LD   A,(PLIX)
        INC  A
        AND  03H
        LD   (PLIX),A     ; CURR PLAYER IX<-CURR PL IX+1 M
        CALL LDPLIX
        BIT  ACTBIT,(IX+PSTAT) ; TEST FOR ACTIVE PLAYER
        JR   Z,INCIX
        JP   MOVEIT            ; THE MAJOR EVENT
MOVJOY:
        SUB  SJ0               ; TAKE OFF WHATEVER
        SRL  A                 ; DIVIDE BY 2
        JP   STALL
CALPIZ:
        CALL TICKIT
        SYSTEM PIZBRK
        RET
UPMUZK: LD   A,(PLIX)
        CALL LDPLIY
        LD   A,(IX+AROT)
        LD   B,3
TSTBIT:
        RRCA
        JR   C,GOTBIT
        DJNZ TSTBIT
GOTBIT:
        LD   C,B
        LD   B,0
        ADD  IY,BC
        LD   A,(IY+0)
        OUT  (TONEC),A
        LD   A,MUSVOL
        OUT  (VOLC),A
        LD   A,OA4
        OUT  (TONMO),A
        RET
CLEARF:
        ; CLEAR FIELD
        SYSSUK FILL
        DW   STARTS
        DW   ALLBYT
        DB   0
        RET
MOVEIT: 
        ; THIS ROUTINE UPDATES A PLAYER'S POSITION
        ; INPUT PARAS ARE: IX=POINTER TO PLAYERS PACKET
        ; DURING ROUTINE B=CURRENT SWITCH C=LAST SWITCH
        LD   C,(IX+LASTSW)
        LD   B,(IX+CURSW)
        BIT  HUMBIT,(IX+PSTAT)
        JR   NZ,NOCUR     ; IF NOT HUMAN
ZSW:    XOR  A            ;  CLEAR A
        LD   B,A          ; CLEAR CURRENT SWITCH
        LD   C,A          ; CLEAR LAST SWITCH ENDIF
NOCUR:  LD   A,B          ; IF CURR SW = 0
        OR   A
        JR   NZ,RANTST
        LD   B,C           ; THEN CURR SW<-LAST SW ENDIF
RANTST: LD   (IX+LASTSW),B ; SAVE LAST SW
        LD   A,B           ; IF CURR SW=0
        OR   A
        JR   NZ,GOTSW
        LD   C,0           ; LAST SW<-0
        CALL RANMOV        ; GET RANDOM MOVE ENDIF
GOTSW:
        LD   A,(IX+LASTMV) ; GET LAST MOVE
        CALL MOVTST
        JR   Z,GOTMOV
                           ; ANY MOVE AND CURR SW
        CALL MOVANY
        JR   Z,GOTMOV
        LD   B,C           ; TRY LAST SW
                           ; ANY MOVE
        CALL MOVANY
        JR   Z,GOTMOV
        LD   B,(IX+LASTMV)  ; TRY LAST MOVE
                            ; ANY MOVE
        CALL MOVANY
        JR   NZ,CRASH
GOTMOV:
        ; A LEGIT MOVE HAS BEEN FOUND SO UPDATE THE GUY
        LD   (IX+LASTMV),A ; SAVE ACTUAL MOVE FOR LATER
        LD   (IX+AROT),A  ;ARROW ROTATION AMOUNT<- THE MOV
        LD   D,(IX+ARRY)
        LD   E,(IX+ARRX)
        CALL ERASE
        LD   H,(IY+PPATH)
        LD   L,(IY+PPATL)
        LD   BC,(PATYSZ SHL 8) + PATXSZ
        LD   A,WRITOR
        SYSTEM WRIT       ; WRITE PLAYER PATTERN OVER ARRO
        LD   A,(TARRX)
        LD   (IX+ARRX),A  ;SAVE NEW ARROW X
        LD   A,(TARRY)
        LD   (IX+ARRY),A  ;SAVE NEW ARROW Y
        CALL ANIARR       ;ANIMATE THE ARROW
        JP   UPMUZK
ERASE:
        PUSH DE
        SYSSUK RELAB1
        DB   0
        EX   DE,HL
        LD   B,0
        LD   DE,[(PATYSZ SHL 8) + PATXSZ]
        SYSTEM BLANK
        POP  DE
        RET
CRASH:
        ; A PLAYER HAS CRASHED.  DESTROY HIS ARROW AND
        ; ELIMINATE HIM FROM THE GAME.
        LD   BC,EXPATS
        LD   DE,EXCOLS    ;DE<-EXPLODE COLOR TABLE ADDR
        LD   A,5
        LD   HL,EXPSND
EXLOOP: PUSH AF           ;PUSH LOOP COUNT
        PUSH BC           ;PUSH EXT PAT ADDRESS
        PUSH DE           ;PUSH EXPLODE COLOR TABLE ADDR
        PUSH HL           ;PUSH EXPLODE SOUND ADDRESS
        LD   A,(DE)       ;A<-EXPLODE COLOR
        OUT  (COL0L),A     ; CHANGED TO MATCH ROM
        PUSH BC
        LD   D,(IX+ARRY)
        LD   E,(IX+ARRX)
        CALL ERASE
        POP  HL            ;PATTERN ADDRESS
        LD   A,WRITOR
        LD   BC,(PATYSZ SHL 8) + PATXSZ
        SYSTEM WRIT        ;WRIT EXPLOSION
        SYSSUK PAWS
        DB   7
        POP  HL            ;GET EXPLODE SOUND ADDR
        LD   BC,(SBLEN SHL 8) + SNDBX
        OTIR
        POP  DE
        POP  BC
        POP  AF
        DEC  A
        JR   Z,EXPFIN      ;LOOP COUNT EXPIRED
        INC  DE            ;INC TO NEXT COLOR
        INC  BC            ;BUMP UP TO NEXT PAT ADDR
        INC  BC
        INC  BC
        INC  BC
        JR   EXLOOP
EXPFIN:
        LD   D,(IY+PPATH)
        LD   E,(IY+PPATL)  ;DE<-PLAYER PAT ADDR
        LD   IY,0
        ADD  IY,DE         ;IY<-PLAYER PAT ADDR
        LD   DE,4 SHL 8 + 0  ;D<-LOOP COUNT
STOMP:  LD   A,(IY+0)      ;A<-BYTE OF PLAYER PATTERN
        LD   HL,STARTS
        LD   BC,ALLBYT
STLOOP: CPIR
        JR   NZ,RESTOM
        INC  BC
        DEC  HL
        LD   (HL),E
        JR   STLOOP
RESTOM: INC  IY
        DEC  D
        JR   NZ,STOMP
        BIT  HUMBIT,(IX+PSTAT)
        JR   Z,KILLST      ;IF HUMAN
        LD   HL,CNOHUM
        DEC  (HL)          ;DEC CURRENT # HUMANS END IF
KILLST: RES  ACTBIT,(IX+PSTAT) ;KILL STATUS
        ; INC ALL ACTIVE PLAYERS SCORES
        LD   C,4
BUMPEM:
        DEC  C
        LD   A,C
        CALL LDPLIX
        BIT  ACTBIT,(IX+PSTAT)
        JR   Z,BUMPCK
        LD   B,0
        PUSH BC
        LD   A,C
        LD   HL,CURSCR
        ADD  HL,BC
        ADD  HL,BC
        ADD  HL,BC
        SCF
        CALL DISPSC
        POP  BC
        SYSTEM INCSCR
        LD   A,C
        PUSH BC
        OR   A
        CALL DISPSC
        POP  BC
BUMPCK:
        SYSSUK PAWS
        DB   30
        LD   A,C
        OR   A
        JR   NZ,BUMPEM
        ;DEC CURR # PLAYERS
        ;IF CURR # PLAYERS LEQ  GO TO END OF GAME
        LD   HL,CNOPL 
        DEC  (HL)
        DEC  (HL)
        JR   Z,ENDCHK
        INC  (HL)
        RET
ENDCHK:
        LD   A,(CT7)
        DEC  A
        DAA
        LD   (CT7),A
        JP   NZ,FIREIT
        SYSTEM QUIT
DISPSC:
        ;DISPLAY SCORE
        ;A=PLAYER #
        ;HL->LAST BYTE OF SCORE
        LD   C,(IY+PSDOP)
        JR   NC,NOTXOR
        RES  MROR,C
        SET  MRXOR,C
NOTXOR:
        LD   E,(IY+PSPOSX)
        LD   D,(IY+PSPOSY)
        LD   A,12
        ADD  A,E
        LD   E,A
        INC  D
        LD   B,$43
        LD   IX,FNTSML
        SYSTEM DISNUM
        RET
ANIARR:
        ;ANIMATE THE ARROW
        ;INPUT AND OUTPUT IS IX WHO STAYS THE SAME
        ;DESTROYS ALL OTHER REGISTERS
        BIT  ACTBIT,(IX+PSTAT)
        RET  Z             ;EXIT IF NOT ACTIVE
        LD   A,(IX+AROT)
        CALL GETROT        ;HL<-ARROW PAT ADDR
        LD   D,(IX+ARRY)
        LD   E,(IX+ARRX)
        PUSH HL
        CALL ERASE
        POP  HL
        LD   BC,(PATYSZ SHL 8) + PATXSZ
        LD   A,WRITOR
        SYSTEM WRIT
        RET
STALL: 
        ;THIS ROUTINE TAKES CARE OF ARROW ANIMATION
        ;AND SHOWING A PLAYER HIS CURRENT JOYSTICK POSIT
        ;A=WHICH PLAYER
        ;B=JOYSTICK BITS
        CALL LDPLIX        ;IX<-ADDR OF PLAYER PACKET
        XOR  A
        OR   B
        JR   NZ,STORIT
        LD   A,(IX+CURSW)
STORIT: LD   (IX+CURSW),A
        XOR  (IX+LASTMV)   ;A<-DIFFERENCE FROM LAST MOVE
        JR   Z,GETLM       ;IF DIFFERENCE=0 USE LAST MOVE
        XOR  RLMOVE
        JR   Z,GETLM
        XOR  RLMOVE
        XOR  UDMOVE
        JR   Z,GETLM
        XOR  UDMOVE
HUMCHK: BIT  HUMBIT,(IX+PSTAT)
        JR   NZ,GOTIT      ;IF HUMAN WE'VE GOT IT
GETLM   LD   A,(IX+LASTMV) ;GET LAST MOVE
GOTIT:  LD   (IX+AROT),A   ;STORE ARROW ROTATION
        JR   ANIARR
TICKIT:
        ;TICK COUNT<-(8-CURR # PLAYERS)
        LD   A,(CNOHUM)
        OR   A
        LD   A,2
        JR   Z,STICK
        LD   HL,CNOPL
        LD   A,8
        SUB  (HL)
STICK:  LD   (CT0),A
        RET
LDPLIX:
LDPLIY:
        ;LOAD IY WITH POINTER TO CURR PLAYER ROM DATA
        ;LOAD IX WITH A POINTER TO CURRENT PLAYER PACKET
        ;A=PLAYER# MUST BE GEQ 0 $ LEQ 3
        PUSH DE
        PUSH HL
        SYSSUK INDEXW
        DW   ROMTBL
        PUSH DE
        POP  IY
        SYSSUK INDEXW
        DW   RAMTBL
        PUSH DE
        POP  IX
        POP  HL
        POP  DE
        RET
ROMTBL: DW   PLROM0
        DW   PLROM1
        DW   PLROM2
        DW   PLROM3
RAMTBL: DW   PLAY0
        DW   PLAY1
        DW   PLAY2
        DW   PLAY3
RANMOV:
        ;GENERATE A RANDOM MOVE FOR THE PLAYER PACKET POI
        ;INPUT AND OUTPUT:
        ;B=CURRENT SWITCH C=LAST SWITCH
        SYSSUK(RANGED)
        DB   32
        OR   A             ;TIME TO CHANGE DIRECTION?
        JR   Z,NEWMOV
        LD   B,(IX+LASTMV) ;USE LAST MOVE
        LD   A,B
        CALL MOVTST
        RET  Z             ;LAST MOVE IS GOOD ENOUGH
NEWMOV: SYSSUK RANGED
        DB   4
        LD   B,A
        INC  B
        LD   A,$80
SHFTIT: RLCA
        DJNZ SHFTIT
        LD   B,A
RANFIN: LD   A,$8
        CALL MOVTST
        JR   NZ,ANYMOV
        LD   B,A
        RET
ANYMOV: LD   B,$F          ;TRY ALL MOVES
        LD   A,$8
        CALL MOVTST
        LD   B,A
        RET
MOVANY:
        LD   A,AMOVE
MOVTST:
        ;TEST THE NEW MOVE FOR VALIDITY
        ;THE INPUTS AND OUTPUTS:
        ;B=A SET ??? MOVES TO BE TESTED (IS UNCHANGED)
        ;C=UNCHANGED
        ;A=FIRST MOVE TO TEST,VALUE OF GOOD MOVE ON OUTPU
        ;DE,HL=RETURNED UNCHANGED
        ;D=# ROTATES
        ;Z FLAG=Z IF GOOD MOVE FOUND(A CONTAINS FIRST GOO
        ;Z FLAG=??? IF NO GOOD MOVES FOUND(IN B)
        PUSH DE
        LD   D,8           ;INIT # ROTATES
ROTMSK: RRCA               ;ROTATE TO NEXT MOVE
        LD   E,A
        AND  B
        CALL CHKMOV        ;CHECK MOVE
        LD   A,E
        JR   Z,MOVEXT      ;FOUND ONE
        DEC  D             ;DEC # ROTATES
        JR   NZ,ROTMSK
        SCF                ;NO GOOD MOVES
        RL   D             ;SET Z FLAG=NZ
MOVEXT: POP  DE
        RET
CHKMOV: 
        ;CHECK THE MOVE IN A FOR BEING UNOCCUPIED
        ;INPUT AND OUTPUT:
        ;A=UP, DOWN, RIGHT OR LEFT BIT(RETURN UNCHANGED)
        ;Z FLAG=Z IF MOVE IN A IS TO AN EMPTY POSITION
        ;Z FLAG=NZ IF MOVE IN A IS BAD
        ;BC,DE,HL RETURNED UNTOUCHED
        ;IX=POINTER TO CURRENT PLAYER PACKET
        ;LOCAL TO THIS ROUTINE:
        ;   D=TEMP X COORD OF ARROW
        ;   E=TEMP Y COORD OF ARROW
        PUSH BC
        PUSH DE
        PUSH HL
        PUSH AF
        LD   D,(IX+ARRX)   ;GET X COORD OF ARROW
        LD   E,(IX+ARRY)   ;GET Y COORD OF ARROW
TLEFT:  BIT  CHLEFT,A
        JR   Z,TRIGHT
        LD   A,D           ;GET A LEFT MOVE
        CP   LOWX
        JR   Z,BADMOV      ;ALREADY AT LOWEST X
        SUB  WIDTH         ;DEC TEMP X BY 1 POSITION
        LD   D,A
        JR   LOOKSQ
TRIGHT: BIT  CHRIGH,A
        JR   Z,TUP
        LD   A,D           ;GOT A RIGHT MOVE
        CP   XMAX
        JR   NC,BADMOV     ;ALREADY GEQ MAX X
        ADD  A,WIDTH
        LD   D,A
        JR   LOOKSQ
TUP:    BIT  CHUP,A
        JR   Z,TDOWN
        LD   A,E           ;GOT AN UP MOVE
        CP   LOWY
        JR   Z,BADMOV      ;ALREADY AT LOWEST Y
        SUB  HEIGHT        ;DEC TEMP Y BY 1 POSITION
        LD   E,A
        JR   LOOKSQ
TDOWN:  BIT  CHDOWN,A
        JR   Z,BADMOV
        LD   A,E           ;GOT A DOWN MOVE
        CP   YMAX
        JR   Z,BADMOV      ;ALREADY AT HIGHEST Y
        ADD  A,HEIGHT      ;INC TEMP Y BY 1 POSITION
        LD   E,A
        JR   LOOKSQ
BADMOV: POP  AF
        SCF
        RL   D             ;SET ??? FLAG = NZ
        JR   MOVEND
LOOKSQ:
        ;SEE IF THE NEW SQUARE IS OCCUPIED
        PUSH DE
        PUSH DE
        POP  BC
        LD   D,C           ;REVERSE X,Y FOR SYSTEM
        LD   E,B
        SYSSUK RELAB1
        DB   0
        POP  HL
        EX   DE,HL
        LD   A,(HL)
        OR   A             ;TEST SQUARE
        JR   NZ,BADMOV
        LD   BC,BYTEPL
        ADD  HL,BC
        LD   A,(HL)
        OR   A
        JR   NZ,BADMOV
        LD   A,D
        LD   (TARRX),A     ;STORE TEMP ARROW X COORD
        LD   A,E
        LD   (TARRY),A     ;STORE TEMP ARROW Y COORD
        POP  AF
        LD   D,0
        SRL  D             ;SET Z FLAG=Z
MOVEND: POP  HL
        POP  DE
        POP  BC
        RET
GETROT:
        ;HL<-BASE ADDR OF ROTATED PATTERN
        ;A<-DIRECTION OF ROTATION
        ;IF A HAS MORE THAN 1 BIT SET THEN ONLY ONE IS US
        LD   HL,AUP
        BIT  CHUP,A
        RET  NZ
        LD   HL,ADOWN
        BIT  CHDOWN,A
        RET  NZ
        LD   HL,ARIGHT
        BIT  CHRIGH,A
        RET  NZ
        LD   HL,ALEFT
        RET
        ;START OF ROM DATA FOR EACH PLAYER.
        ;CONTAINS 4 PLAYER NOTES, PLAYER PATTERN ADDR
        ; , PLAYER CHAR DISP OPT
        ;PLAYER SCORE DISP OPT
        ;AND PLAYER SCORE POSITION
PLROM0:
PNOTE0: DB   G0,GS0,A0,AS0
PPADR0: DW   PPAT0
PCDOP0: DB   011000B
PSPOS0: DB   12            ; CHANGED TO MATCH ROM 
        DB   1
PSDOP0: DB   011000B
PLROM1:
PNOTE1: DB   B0,C1,CS1,D1
PPADR1: DW   PPAT1
PCDOP1: DB   011100B
PSPOS1: DB   125           ; CHANGED TO MATCH ROM 
        DB   1
PSDOP1: DB   011100B
PLROM2:
PNOTE2: DB   DS1,E1,F1,FS1
PPADR2: DW   PPAT2
PCDOP2: DB   011100B
PSPOS2: DW   45 + (1 SHL 8)
PSDOP2: DB   011100B
PLROM3:
PNOTE3: DB   G1,GS1,A1,AS1
PPADR3: DW   PPAT3
PCDOP3: DB   011000B
PSPOS3: DW   $0159         ; CHANGED TO MATCH ROM 
PSDOP3: DB   011000B
        ;EXPLOSION PATTERNS
EXPATS:
EXPAT1: DB   0,00010100B,00010100B,0
EXPAT2: DB   0,01000101B,01010001B,0
EXPAT3: DB   00000101B,01000000B,00000001B,01010000B
EXPAT4: DB   00010001B,01000000B,00000001B,01000100B
EXPAT5: DB   0,0,0,0
        ;EXPLOSION COLORS
EXCOLS:
        DB   7
        DB   3
        DB   7
        DB   3
        DB   $87           ; CHANGED TO MATCH ROM 
        ;COUNT DOWN DISPLAY PACKET
CDCOLR: DW   0100B + (0 SHL 8)
        ;TIMER DISPLAY PACKET
TDPACK: DW   0001B + (10000000B SHL 8)
        ;ARROW ANIMATION PATTERNS FOR EACH ROTATION
AUP:
        DB   00010100B,01010101B,01000001B,0
ARIGHT:
        DB   00010100B,00000101B,00000101B,00010100B
ADOWN:
        DB   0,01000001B,01010101B,00010100B
ALEFT:
        DB   00010100B,01010000B,01010000B,00010100B
        ;PLAYER PATTERNS
PPAT0:  DB   00001000B,10101000B,00101010B,00100000B
PPAT1:  DB   11111111B,11000011B,11000011B,11111111B
PPAT2:  DB   00001100B,11111100B,00111111B,00110000B
PPAT3:  DB   10101010B,10000010B,10000010B,10101010B
        ;COLOR BOCK
CBLOCK:
        DB   $B3           ; CHANGED TO MATCH ROM
        DB   $62           ; CHANGED TO MATCH ROM 
        DB   $F8
        DB   $87           ; CHANGED TO MATCH ROM 
        DB   $F8           ; CHANGED TO MATCH ROM 
        DB   $F8           ; CHANGED TO MATCH ROM 
        DB   $F8
        DB   $F8           ; CHANGED TO MATCH ROM 
        ;EXPLOSION SOUNDS
EXPSND: DB $EF,$FF,$3F,$00,$FF,$FD,$F5,$F5
        DB $8F,$EE,$3E,$00,$FF,$FD,$F5,$F5
        DB $4E,$88,$38,$00,$FF,$FD,$F5,$F5
        DB $48,$44,$34,$00,$FF,$FD,$F5,$F5
        DB 0,0,0,0,0,0,0,0
        
;
;       END OF CHECKMATE PROGRAM
;
; FIVE FILLER BYTES BETWEEN CHECKMATE AND
; GUNFIGHT TO MATCH 3159 ROM
        DB   0FFH, 0FFH, 0FFH, 0FFH, 0FFH
        ;
        ;
        ; ***************************
        ; * H V G   G U N F I G H T * ; VER. 1.0 - DEC. 15, 2003
        ; ***************************
        ;
        ;
; Program Name - Gun Fight
;                for the Bally Astrocade
; Retyped by Adam Trionfo

        JP    INIT 

; *********************
; * GUN FIGHT EQUATES *
; *********************
 ; GUNFIGHT BACKGROUND JOB
 ; CONSISTING OF INITIALIZATION, PRE-ROUND DISPLAY,
 ; MONITORING OF CONTROLS AND VECTOR DELTA CHANGING
 ; DEATH, POST ROUND STUFF AND END GAME

  ;EQUATES
LNX     EQU  8            ; LEFT NUMBER X
BSY     EQU  2            ; BANNER STRINGS Y
RNX     EQU  136          ; RIGHT NUMBER X
LBULX   EQU  32           ; LEFT BULLETS X
RBULX   EQU  104          ; RIGHT BULLETS X
STMRX   EQU  76           ; SUB TIMER X
GRX     EQU  44           ; GET READY X
GRY     EQU  1            ; GET READY Y
DRX     EQU  64           ; DRAW X
TCACY   EQU  20           ; TOP CACTUS Y
TTREEY  EQU  TCACY-5
MCACY   EQU  42           ; MID CACTUS Y
BCACY   EQU  70           ; BOTTOM CACTUS Y
BTREEY  EQU  BCACY-5
LCACX   EQU  64           ; LEFT CACTUS X
RCACX   EQU  88           ; RIGHT CACTUS X
CCACX   EQU  76           ; CENTER CACTUS X
WAGX    EQU  72           ; WAGON X
COWX    EQU  RCACX+8      ; OTHER COWBOYS WINDOW X
;
TLINE   EQU  10           ; TOP LINE OF GUNSPACE
ALINE   EQU  TLINE-1
BLINE   EQU  92           ; BOTTOM LINE OF GUNSPACE
;
BULVSZ  EQU  18           ; BULLET VECTOR SIZE
GFVSIZ  EQU  23
WAGVSZ  EQU  18           ; WAGON VECTOR SIZE
;
WINBND  EQU  50           ; TOP BOTTOM WINDOW BOUNDARY
TOPLIN  EQU  53*2         ; TOP WINDOWS LINE
BOTLIN  EQU  00           ; BOTTOM WINDOW LINE
LFRLIN  EQU  100*2        ; LOW PRIORITY FOREGROUND LINE
;
NEXT    EQU  -1           ; NEXT LINE FOR QUEUES
VBARM   EQU  VBOAH+1      ; ARM STATE
VBOARM  EQU  VBARM+1      ; LAST ARM PATTERN WRITTEN
VBLEGT  EQU  VBOARM+1     ; LEG TIMER
VBLEG   EQU  VBLEGT+1     ; LEG LINK
VBCOMP  EQU  VBLEG+1      ; TIMER FOR COMPUTER CONTROL
; BITS
VBSWAG  EQU  0            ; WAGON BIT
VBSCHG  EQU  3            ; CHANGE STATUS BIT
VBSNOM  EQU  4            ; NOT MOVING STATUS
VBSINT  EQU  5            ; INTERCEPTED/DEAD STATUS

; ***************
; * SUBROUTINES *
; ***************
; DISPLAY CLOCK AND UPDATE CT4
DCLOCK  DI
        SYSSUK DECCTS
        DB    10000000B
        LD    IX,FNTSML
        LD    A,(CT7)
        OR    A
        JR    Z,DCOUT
        SYSSUK DISNUM
        DB    STMRX
        DB    BSY
        DB    TIME
        DB    42H
        DW    CT7
DCOUT   XOR   A
        OUT   (MAGIC),A
        LD    (WASTE),A
        EI
        RET
; FIRE BULLETS
; LEFT COWBOY
FIRE0   SYSSUK SUCK
        DB    11011100B
        DW    LCOWB
        DW    LBULS
        DW    BULV1+1
        JR    ZORE
FIRE1   SYSSUK SUCK
        DEFB  11011100B
        DW    RCOWB
        DW    RBULS
        DW    BULV3+1
ZORE    LD    A,(IY+CBB)
        OR    A
        RET   Z
        LD    A,(BC)        ; GET BULLET COUNT
        OR    A
        RET   Z
        LD    A,(HL)       ; CHECK IF BULLET IS AVAILABLE
        OR    A
        JR    Z,ZOK
        LD    DE,BULVSZ    ; DELTA TO NEXT BULLET
        ADD   HL,DE
        LD    A,(HL)
        OR    A
        JR    Z,ZOK
        RET
; N?W HL->BULLET
;     IX->COWBOY
; SUB 1 FROM BULLET COUNT
ZOK     LD    A,(BC)
        DEC   A
        LD    (BC),A
; SET SUB TIMER IF OUT OF BULLETS
        JR    NZ,BERASE
        LD    A,(CT7)
        OR    A
        LD    A,10H
        JR    Z,STSEC
        LD    A,2
STSEC   LD    (CT7),A
BERASE  PUSH  HL
        PUSH  IX
        LD    A,(BC)
        LD    L,A
        LD    H,0
        ADD   HL,HL
        ADD   HL,HL        ;*4
        LD    DE,BSY*256+RBULX
        BIT   MRFLOP,(IX+VBMR)
        LD    A,40H        ; FLOPPED MR
        JR    Z,RITB
        XOR   A            ; NORMAL MR
; NOW POSITION AND ERASE
RITB    ADD   HL,DE
        EX    DE,HL
        SYSTEM RELAB1
        EX    DE,HL
        LD    B,5
        LD    DE,40        ; INC TO NEXT LINE
BELP    LD    (HL),0FFH    ; ERASE LINE
        ADD   HL,DE        ; GO DOWN A LINE
        DJNZ  BELP
        LD    D,0
        LD    E,(IX+VBARM) ; GET CURRENT ARM POS
        LD    H,D
        LD    L,E
        ADD   HL,HL        ; *2
        ADD   HL,DE        ; *3
        LD    DE,BULTAB
        ADD   HL,DE        ; -> BULTAB(ARM)
        EX    DE,HL
        POP   BC           ; BC<==IX
        POP   HL           ; BUL [STAT]
        PUSH  HL           ; SAVE FOR ACTIVATE
        INC   HL           ; BUL [DEL TIME]
        LD    (HL),1       ; MAKE BULIT JUMP OUT
        INC   HL           ; BUL [DEL XLOW]
        INC   BC           ; COW [STAT]
        INC   BC           ; COW [DEL TIME]
        INC   BC           ; COW [DX LO]
        CALL PUTVEC
        INC   BC           ; COW [XCHK]
        INC   BC           ; COW [DY LO]
        INC   HL           ; BUL [XCHK]
        LD    (HL),1       ; LIMIT CHECK
        INC   HL           ; BUL [DY LO]
        CALL PUTVEC
        POP   HL           ; BUL [STAT]
        LD    (HL),80H     ; ACTIVE
        SYSSUK BMUSIC
        DW    MSTACK
        DB    00000001B    ; JUST NOISE
        DW    GUNSHOT
        RET
; TAKE A COFFEE BREAK
NBRK    DONT  PIZBRK       ; SEE IF I CARE
        DO    MRET
; CONVERT JOYSTICKS
JOY0    LD    IX,LCOWB
        JR    PJOY
JOY1    LD    IX,RCOWB
;   CONVERT   JOYSTICKS
PJOY    LD    C,(IX+VBMR)
        LD    DE,128
        LD    HL,128
        SYSTEM MSKTD       ; COMPUTE DELTAS
        LD    (IX+VBDYH),H
        LD    (IX+VBDYL),L
        LD    (IX+VBDXH),D
        LD    (IX+VBDXL),E
        RET
PPOT1:  LD    IX,RCOWB
        LD    A,B          ; POT MUST BE FLOPPED BECAUSE
        CPL                ; ARM IS FLOPPED
        JR    PPOT
PPOT0   LD    IX,LCOWB
        LD    A,B
; CONVERT POT AND STORE
PPOT    AND   0E0H
        RRCA
        RRCA
        RRCA
        RRCA
        CP    0EH
        JR    NZ,KART
        LD    A,0CH              ; IF KNOB=7 THEN SET TO 6
KART    LD    (IX+VBARM),A       ; SET ARM POSITION
        RET
; CHECK IF BULLET HIT ANYTHING
HITCHK  LD    A,(IX+VBSTAT)
        AND   060H
        CP    20H                ; CHECK ONLY IF BLANKED
        JR    Z,HIT
        RET   NC                 ; RETURN IF NOT BLANKED YET
        BIT   VBCLAT,(IX+VBXCHK)
        RET   Z
        LD    (IX+ VBSTAT),0     ; BULLET HIT WALL
        LD    (IX+VBXCHK),1      ; SET LIMIT CHECK
        RET
HIT     LD    A,(IX+VBXH)   ; CHECK WHAT PART OF SCR ITS ON
        CP    WAGX
        JR    NC,HIT1
        LD    (IX+VBTIMB),2      ;MAKE IT JUMP OUT
        LD    (IX+VBSTAT),80H    ; RE ACTIVATE
        LD    HL,BULLMT
        SYSTEM VECT
        RET
HIT1    LD    (IX+VBSTAT),0 ; BULLET DIES FROM WAGON ON
        CP    RCACX
        JR    NC,HIT2
        LD    A,(WAGON)
        OR    A            ; IS IT A CACTII?
        RET   NZ           ; NOPE ITS A WAGON
        LD    E,CCACX      ; LOAD X
; ERASE OBJECT BULLET HITS
ERASE_  LD    D,(IX+VBYH)  ; LOAD Y
        DEC   D
        SYSSUK RELAB1
        DB    0
        EX    DE,HL
        LD    DE,-41
        LD    B,0
ELOP    LD    A,(HL)
        LD    (HL),B       ; ZERO THE SCREEN BYTE
        INC   HL
        OR    (HL)
        LD    (HL),B
        ADD   HL,DE
        JR    NZ,ELOP
        RET
HIT2    CP    RCACX+8      ; GUNFTR SPACE
        JR    NC,DIE
        LD    E,LCACX
        BIT   MRFLOP,(IX+VBMR)
        JR    NZ,ERASE_
        LD    E,RCACX
        JR    ERASE_
DIE     BIT   MRFLOP,(IX+VBMR) ; WHO DIED?
        JR    Z,DLEFT
        SYSSUK SUCK
        DB    11011101B
        DW    LCOWB
        DB    8
        DW    TAPS
        DW    RSCORE
        JR    DIE1
DLEFT   SYSSUK SUCK
        DB    11011101B
        DW    RCOWB
        DB    100
        DW    FUNERL
        DW    LSCORE
DIE1    LD    (IX+VBLEGT),6      ; SET FIRST CELL TIME
        LD    (IX+VBLEG),KIL1 AND 0FFH ; ??
        LD    (IX+VBSTAT),068H   ; KILL HIM
        LD    A,(IX+VBYH)        ; WHERE TO WRITE GOT ME
        SUB   8
        CP    TLINE+9
        JR    NC,DIE4
        ADD   A,32
DIE4    LD    D,A
        SYSTEM INCSCR
        DEC   HL
        LD    A,(HL)             ; FIELD
        CP    5
        ADC   A,0
        LD    (HL),A
; PLAY DEATH SONG
        LD    H,B
        LD    L,C
        LD    IX,MSTACK
        LD    A,11000000B
        SYSTEM BMUSIC
        LD    C,LARG2
        LD    HL,GOTME
        DI
        SYSTEM STRDIS
        SYSSUK PAWS
        DB    250
        LD    A,1
        LD    (SEMI4S),A   ; SET FLAG0
        RET
; FIELD PUTS UP THE SACTII APPROP TO SCORE
; A=SCORE OF OPP PLAYER UPTO 6
; BC -> ARRAY OF Y POSITIONS
FIELD   LD    HL,CACTUS    ; -> CACTUS PATTERN
        PUSH  AF
        LD    A,1000B
        OUT   (XPAND),A
        POP   AF
        CP    1
        RET   C
        CP    4
        JR    NC,TCAC
        CALL  CACW
TCAC    INC   BC
        CP    2
        RET   C
        CP    5
        JR    NC,MCAC
        CALL  CACW
MCAC    CP    3
        RET   C
        INC   BC
        EX    AF,AF'
        LD    A,81H        ; ACIVATE WAGON
        LD    (WAGON),A
        EX    AF,AF'
        CALL  CACW
        CP    4
        RET   C
        INC   BC
        LD    HL,TREE
        PUSH  AF
        LD    A,1100B
        OUT   (XPAND),A
        POP   AF
        CALL  CACW
        CP    5
        RET   C
        INC   BC
CACW    PUSH  AF
        PUSH  DE
        LD    A,(BC)
        LD    D,A
        LD    A,8          ; EXPAND
        SYSTEM WRITP
        POP   DE
        POP   AF
        RET
; PUT DEL X,Y INTO BULLET VECTORS
PUTVEC  LD    A,(DE)        ; TABLE [D LO]
        LD    (HL),A       ; BUL [D LO]
        INC   DE           ; TAB [D HI]
        INC   BC           ; COW [D HI]
        INC   HL           ; BUL [D HI]
        LD    A,(DE)
        LD    (HL),A
        INC   HL           ; BUL [LO]
        INC   DE           ; TAB [HI]
        INC   BC           ; COW [LO]
        LD    (HL),0
        INC   BC           ; COW [HI]
        INC   HL           ; BUL [HI]
        LD    A,(BC)
        EX    DE,HL
        ADD   A,(HL)
        EX    DE,HL
        LD    (HL),A       ; BUL [HI]=COW [HI]+TAB [HI]
        INC   DE           ; TAB [D HI]
        RET
; GUNFIGHT START UP ROUTINE (ONCE PER GAME)
INIT    SYSSUK GETPAR
        DW    MXSCR
        DB    84H
        DW    ENDSCR
        LD    SP,STACK
        SYSTEM INTPC
        DO    FILL
        DW    STACK
        DW    CT7-STACK
        DB    0
        DO    SETB
        DB    2*GSBSCR
        DW    GAMSTB
        DO    SETOUT       ; SET UP GAME PORTS
        DB    BLINE*2      ; BOTTOM LINE - VRT BLK
        DB    214D         ; HORZ BOUNDS
        DB    8            ; INMOD
        DO    COLSET
        DW    GFCOLS
        DO    BMUSIC       ; PLAYS STREETS OF LOREDO
        DW    MSTACK
        DB    11000000B    ; ON VOICE A
        DW    HOME
        EXIT
; *****
; ONCE A ROUND START UP ROUTINE
; *****
STRND   DI
        SYSTEM INTPC
 ; INIT HANDLES, BULLETS, TIMERS
        DO    MOVE
        DW    CT5
        DW    12
        DW    SINIT
 ; COLOR BANNER
        FILLq NORMEM,BYTEPL*ALINE,0FFH
 ; ERASE SCREEN
        FILLq NORMEM+BYTEPL*ALINE,BYTEPL*(BLINE-ALINE),0
 ; RESET VECTORS
        FILLq STRRAM,ENDRAM_-STRRAM,0
 ; SHOW SCORES
        DO    SUCK
        DB    00010000B
        DW    FNTSML
        DO    DISNUM
        DB    LNX
        DB    BSY
        DB    TIME
        DB    0C4H         ; ZERO SUPRS,SMALL
        DW    LSCORE
        DO    DISNUM
        DB    RNX
        DB    BSY
        DB    TIME
        DB    0C4H
        DW    RSCORE
 ; CHECK FOR END GAME
        DO    RCALL
        DW    ENDGAM
        TEXTD GETRDY,GRX,GRY,LARGE
        EXIT
        XOR   A            ; SET UP WAGON
        LD    (WAGON),A    ; STOP WAGON
 ; PUT UP PLAY FIELD
        LD    A,(RFIELD)   ; NUMBER OF CACTII
        LD    E,RCACX      ; RIGHT CAC COLUMN
        LD    BC,RFTAB     ; POSITIONS OF TABLE OF CACTII
        CALL FIELD         ; PUT THE CATCII UP
        LD    A,(LFIELD)
        LD    E,LCACX
        LD    BC,LFTAB
        CALL  FIELD
 ; INITIALIZE Q POINTERS
INITQ   LD    A,LCOWB SHR 8
        LD    (WRITQ+2),A
        LD    (VECQ+2),A
 ; SETUP VECTORS SO COWBOYS WALK OUT
        LD    IX,LCOWB     ; LEFT COWBOY VECTOR
        LD    (IX+VBMR),10H
        LD    HL,VECQ
        CALL  COWINT
        LD    IX,RCOWB     ; RIGHT COWBOY VECTOR
        LD    (IX+VBMR),50H
        CALL  COWINT
        LD    A,(WAGON)    ; IF WAGON IS ON
        OR    A
        JR    Z,MIDC
        LD    IX,WAGVEC    ; THEN ACTIVATE WAGON
        LD    (IX+VBMR),10H
        LD    (IX+VBYCHK),3
        LD    (IX+VBDYL),40H
        LD    (IX+VBXH),72
        LD    (IX+VBYH),TLINE
        CALL  ADDTQ
        JR    BORG
MIDC    LD    A,8
        OUT   (XPAND),A
        SYSSUK WRITP       ; ELSE PUT UP A CACTUS
        DB    CCACX
        DB    MCACY
        DB    8            ; EXPAND
        DW    CACTUS
; INITIALIZE BULLET VECTORS
BORG    LD    DE,BULVSZ
        LD    IX,BULV1
        LD    BC,4*256+20H
        LD    A,2
BULLP   CP    B
        JR    NZ,TIYU
        LD    C,60H
TIYU    LD    (IX+VBMR),C
        LD    (IX+VBXCHK),1
        LD    (IX+VBYCHK),3
        ADD   IX,DE
        DJNZ  BULLP
; FIRE UP INTERRUPTS
        LD    A,INTTBL SHR 8
        LD    I,A
;       IM       2         ; DONE IN MENU
        LD    A,LFRVEC AND 0FFH
        OUT   (INFBK),A
; ***
; LET COWBOYS WALK OUT
; ***
WALK    SYSSUK PAWS
        DB    100
        DI
        LD    IX,FNTSML
        SYSTEM INTPC
 ; ERASE GET READY
        DO    BLANK
        DB    18           ; X SIZE
        DB    8            ; Y SIZE
        DB    0FFH         ; DATA TO BLANK TO
        DW    4033H        ; BLANK ADDRESS
        TEXTD DRAW,DRX,GRY,LARGE
        DO    CHRDIS
        DB    LBULX
        DB    BSY
        DB    BULT
        DB    0BBH         ; BULLET
        DO    MCALL        ; 5 MORE
        DW    BULRIT
        DO    SUCK
        DB    00000001B
        DB    RBULX        ; DO THE RIGHT ONES
        DONT  CHRDIS       ; DISPAY THE FIRST ONE
        DO    MCALL        ; DISPLAY THR OTHER FIVE
        DW    BULRIT
        DO    PAWS
        DB    60
        DO    BLANK
        DB    8            ; X SIZE
        DB    8            ; Y SIZE
        DB    0FFH         ; DATA TO BLANK TO
        DW    4038H        ; BLANK ADDRESS
        EXIT
;
; *****
; MAIN LOOP DURING ROUND
; GETS HANDLES, SETS VECTORS AND CHECKS BULLETS
LOOP    SYSTEM INTPC
        DO    SENTRY
        DW    ALKEYS
        DO    DOIT
        DW    DTAB
        EXIT
;
; CHECK FOR DEATHS
DEATH   LD    IX,BULV1
        LD    DE,BULVSZ
        LD    B,4
LPPP2   PUSH  BC
        PUSH  DE
        CALL  HITCHK
        POP   DE
        POP   BC
        ADD   IX,DE
        LD    A,(SEMI4S)   ; CHECK IF DEATH MODE
        DEC   A
        JR    Z,LOOP
        DJNZ  LPPP2
        JR    LOOP
;
ENDRND  EXIT
        JP    STRND
;
ENDGAM  LD    A,(GAMSTB)
        BIT   GSBEND,A
        RET   Z
        SYSTEM QUIT
;
DTAB    JMPd  SCT7, ENDRND, 0
        JMPd  SF0, ENDRND, 0
        RC    SP0, PPOT0, 0
        RC    SP1, PPOT1, 0
        RC    SJ0, JOY0, 0
        RC    SJ1, JOY1, 0
        MC    SKYD, NBRK, 0
        RC    ST0, FIRE0, 0
        RC    ST1, FIRE1, 0
        RC    SSEC, DCLOCK, ENDx
BULRIT  DONT  CHRDIS
        DONT  CHRDIS
        DONT  CHRDIS
        DONT  CHRDIS
        DONT  CHRDIS
        DONT  MRET

; ************************************
; * GUNFIGHT WRITE INTERRUPT ROUTINE *
; ************************************
GFWRIT  EX    AF,AF'
              EXX
        PUSH  IX
BEGINT  LD    A,LFRVEC AND 0FFH  ; ESTABLISH TICKS INT
        OUT   (INFBK),A
        LD    A,LFRLIN
        OUT   (INLIN),A
        LD    HL,WRITQ           ; GET FIRST WRITE Q ENTRY
        CALL  FIRST
        CALL  DELQ               ; DROP FROM WRITE Q
        XOR   A
        LD    (WASTE),A
        BIT   VBSWAG,(IX+VBSTAT) ; WAGON?
        JR    NZ,GFWRT1          ; JUMP IF YES
; GUNFIGHTER - BLANKETH HIM
        LD    DE,1405H           ; LOAD BLANKING PARMS
        SYSTEM VBLANK            ; CALL BLANKER
        LD    H,LEG0 SHR 8       ; WRITE LEG PATTERN
        LD    L,(IX+VBLEG)
        INC   L                  ; SKIP OVER LINK AND TIME
        INC   L
        SYSTEM VWRITR            ; AND WRITE LEG
; IS GUNFIGHTER DEAD?
        BIT   VBSINT,(IX+VBSTAT)
        JR    NZ,GFWRT5          ; JUMP IF SO
        LD    HL,ARMTBL          ; LOOKUP ARM PATTERN
        LD    D,0
        LD    E,(IX+VBARM)
        ADD   HL,DE
        LD    E,(HL)
        INC   HL
        LD    D,(HL)
        EX    DE,HL
        SYSTEM VWRITR            ; WRITE ARM PATTERN
        LD    HL,GFBODY          ; LOAD BODY PATTERN
        JR    GFWRT2
; BLANK THE WAGON
GFWRT1  LD    DE,1604H           ; LOAD WAGON SIZE
        SYSTEM VBLANK
        LD    HL,WAGPAT
GFWRT2  SYSTEM VWRITR            ; NOW WRITE
GFWRT4  LD    (IX+VBOAH),D
        LD    (IX+VBOAL),E
GFWRT3 LD    HL,VECQ             ; ADD VECTOR TO VECTOR Q
        CALL  ADDTQ
        POP   IX
        EX    AF,AF'
        EXX
EIRE    EI
        RET
GFWRT5  LD    HL,NULPAT
        JR    GFWRT2
; ***********************************
; * GUNFIGHT LOW FOREGROUND ROUTINE *
; ***********************************
GFLFR   PUSH  AF
        PUSH  BC
        PUSH  DE
        PUSH  HL
        PUSH  IX
; BUMP TIME BASES OF ACTIVE OR INTERCEPTED VECTORS
        LD    HL,BULV1+VBSTAT
        LD    DE,BULVSZ-1
        LD    B,4
        CALL  TBUMP
        INC   HL                 ; SKIP LINK FIELD
        LD    DE,GFVSIZ-1
        LD    B,3
        CALL  TBUMP
; LOOP TO UNWRITE, THE WRITE ALL FOUR BULLETS
; BUT FIRST, A WORD TO OUR SHIFTER
        XOR   A
        LD    (WASTE),A
        LD    B,4
        LD    IX,BULV1
; UNWRITE THIS GUY?
WRBUL1  BIT   VBBLNK,(IX+VBSTAT)
        JR    Z,WRBUL2           ; JUMP IF NOT
        LD    H,(IX+VBOAH)
        LD    L,(IX+VBOAL)
        LD    A,(IX+VBARM)       ; GET LAST MR
        OUT   (MAGIC),A
        LD    (HL),0C0H          ; UNWRITE BULLET
        RES   VBBLNK,(IX+VBSTAT) ; CLEAR BLANK BIT
; SHALL WE WRITE THIS GUY?
WRBUL2  BIT   VBSACT,(IX+VBSTAT)
        JR    Z,WRBUL4
        LD    D,(IX+VBYH)
        LD    E,(IX+VBXH)
        LD    A,(IX+VBMR)
        SYSTEM RELABS
        LD    (IX+VBOAH),D
        LD    (IX+VBOAL),E
        LD    (IX+VBARM),A
        LD    HL,NORMEM-SCREEN
        ADD   HL,DE
DIFER   EQU   WASTE-SCREEN+NORMEM
        LD    A,(HL)
        EX    DE,HL
        LD    (HL),0C0H
        OR    A
        JR    Z,WRBUL3            ; JUMP IF NOT
        RES   VBSACT,(IX+VBSTAT)  ; KILL ACTIVE BIT
        SET   VBSINT,(IX+VBSTAT)  ; SET INTERCEPT LINE
WRBUL3  SET   VBBLNK,(IX+VBSTAT)  ; SET BLANK BIT
; STEP TO NEXT BULLET VECTOR   LOOP BACK IF DONE
WRBUL4  LD    DE,BULVSZ
        ADD   IX,DE
        DJNZ  WRBUL1
; GET NEXT PATTERN TO WRITE, AND SCHEDULE HIM
        LD    HL,WRITQ
        CALL  FIRST
        JR    Z,WRBL5A            ; JUMP IF EMPTY Q
        LD    A,WRTVEC AND 0FFH   ; SET FEEDBACK REG
        OUT   (INFBK),A
        LD    A,(IX+VBYH)         ; WHICH WINDOW TO USE?
        CP    WINBND              ; COMPARE TO WINDOW BOUNDRY
        LD    A,BOTLIN            ; ASSUME BOTTOM LINE
        JR    NC,WRBUL5           ; JUMP IF GOOD GUESS
        LD    A,TOPLIN            ; WRONG - USE TOP
WRBUL5  OUT   (INLIN),A           ; SET LINE REGISTER
        EI
; LOOP THRU VECTORING THOSE BULLETS
WRBL5A  LD    IX,BULV1
        LD    B,4
        LD    HL,BULLMT    ; HL = BULLET LIMITS TABLE
        LD    DE,BULVSZ
WRBUL6  BIT   VBSACT,(IX+VBSTAT) ; ACTIVE BULLET?
        JR    Z,WRBUL7
        SYSTEM VECT
        BIT   VBCLAT,(IX+VBXCHK) ; DID Y HIT EDGE?
        JR    Z,WRBUL7           ; NOPE
        RES   VBSACT,(IX+VBSTAT) ; DEACTIVATE BULLET
WRBUL7  ADD   IX,DE
        DJNZ  WRBUL6             ; LOOP BACK
; NOW PUT SOMETHING ON THE WRITE Q
        LD    B,2                ; MAX 2 TIMES THRU
        LD    HL,VECQ
GVECT   CALL  FIRST              ; FIRST VECTOR Q ENTRY
        JP    Z,GVECT4           ; JUMP IS Q EMPTY
        CALL  DELQ               ; DROP FROM VECTOR Q
        EI
; WAGON?
        BIT   VBSWAG,(IX+VBSTAT)
        JP    NZ,GVECT5          ; JUMP ON WAGON
; DEAD?
        BIT   VBSINT,(IX+VBSTAT)
        JR    NZ,GVECT1
; ZERO VELOCITY?
        LD    A,(IX+VBDXL)
        OR    (IX+VBDXH)
        OR    (IX+VBDYL)
        OR    (IX+VBDYH)
        JR    NZ,GVECT1          ; GVECT1 IF NONZERO
        LD    (IX+VBTIMB),A      ; ZERO TIME BASE
        BIT   VBSNOM,(IX+VBSTAT) ; ALREADY STATIONARY
        JR    NZ,GVEC3A
; SET STATIONARY LEGS
        LD    (IX+VBLEG),LEG0 AND 0FFH
        SET   VBSCHG,(IX+VBSTAT) ; SET CHANGED
        SET   VBSNOM,(IX+VBSTAT) ; AND STATIONARY
        JR    GVEC3A             ; JUMP TO ARM CHECK
; MOVING GUNFIGHTER
; VECTOR
GVECT1  LD    HL,GUNLMT          ; LOAD GF LIMITS
        SYSTEM VECT
        JR    Z,GVECT2
        SET   VBSCHG,(IX+VBSTAT) ; SET CHANGED BIT
        RES   VBSNOM,(IX+VBSTAT) ; CLEAR NOT MOVING STATUS
; NEED WE GO TO NEXT CELL IN ANIMATION SEQUENCE?
GVECT2  LD    A,(IX+VBLEGT)      ; A = ANIMATION TIMER
        SUB   C                  ; SUBTRACT TIME BASE
        JP    P,GVECT3           ; JUMP IF NOT COUNTED DOWN
; GET NEXT CELL
        LD    E,(IX+VBLEG)       ; GET LINK
        LD    D,LEG0 SHR 8       ; SET H.0. PART
        LD    A,(DE)             ; A = NEXT
        LD    (IX+VBLEG),A
        INC   DE                 ; STEP TO TIMER
        LD    A,(DE)             ; GET NEW TIMER
        SET   VBSCHG,(IX+VBSTAT) ; SET CHANGED BIT
GVECT3  LD    (IX+VBLEGT),A      ; STORE BACK TIMER
; DID ARM CHANGE?
GVEC3A  LD    A,(IX+VBARM)
        CP    (IX+VBOARM)        ; COMPARE TO OLD ARM
        JR    Z,GVEC3B           ; JUMP IF NO CHANGE
        SET   VBSCHG,(IX+VBSTAT) ; SET CHANGED BIT
        LD    (IX+VBOARM),A
; ADD ITEM TO WRITE Q?
GVEC3B  BIT   VBSCHG,(IX+VBSTAT)
        JR    NZ,GVECT6          ; YES GVECT6
; NO CHANGE - LINK TO VECTOR Q
        LD    HL,VECQ
        CALL  ADDTQ
        DEC   B
        JP    NZ,GVECT           ; SUB FOR DJNZ
GVECT4  EI
        CALL  STIMER
        POP   IX
        POP   HL
        POP   DE
        POP   BC
        POP   AF
        RET
; VECTOR AND Q WAGON
GVECT5  LD    HL,WAGLMT
        SYSTEM VECT
        LD    HL,VECQ
        CALL  DELQ               ; REMOVE FROM VECTOR Q
GVECT6  RES   VBSCHG,(IX+VBSTAT)
        LD    HL,WRITQ
        CALL  ADDTQ
        JR    GVECT4             ; JUMP BACK TO QUIT
; ROUTINE TO BUMP TIME BASE OF VECTORS
TBUMP   LD    A,(HL)             ; GET STATUS
        INC   HL
        AND   0A0H               ; ACTIVE OR INTERCEPTED?
        JR    Z,TBUMP1           ; NO - TBUMP1
        INC   (HL)               ; BUMP THE TIME BASE
TBUMP1  ADD   HL,DE
        DJNZ  TBUMP
        RET
; SUBROUTINE TO DELETE ENTRY AT FRONT OF Q
; ENTRY:        HL = HEAD-TAIL, IX = OBJECT, A = CLOBBERE
DELQ    DI
        LD    A,(IX+NEXT)        ; HEAD = NEXT(OBJECT)
        LD    (HL),A
        AND   A                  ; IS HEAD NOW NIL?
        RET   NZ                 ; QUIT IF NOT
        INC   HL                 ; YES - SET TAIL = NIL TOO
        LD    (HL),A
        DEC   HL
        RET
COWINT  LD    (IX+VBDXL),50      ; SLOW WALK OUT
        LD    (IX+VBSTAT),80H    ; ACTIVE
        LD    (IX+VBXCHK),1
        LD    (IX+VBYCHK),1
        LD    (IX+VBXH),4
        LD    (IX+VBYH),40
        LD    (IX+VBARM),6       ; SET ARM STRAIGHT
        LD    (IX+VBLEG),LEG0 AND 0FFH
;       JP    ADDTQ
; SUBROUTINE TO APPEND ENTRY TO END OF Q
; ENTRY:        HL = HEAD-TAIL BYTES, IX = OBJECT, A,DE C
ADDTQ   PUSH  IX                 ; DE = ENTRY
        POP   DE
        DI
        LD    (IX+NEXT),0        ; NEXT(OBJ)=NIL
        INC   HL
        LD    A,(HL)             ; A= OLD TAIL
        LD    (HL),E             ; SET TAIL = .OBJ
        AND   A                  ; WAS OLD TAIL NIL?
        JR    Z,ADDTQ1           ; JUMP IF SO
; NONNIL OLD TAIL  SET NEXT(OLDTAIL)=.OBJ
        LD    E,A                ; DE = .NEXT(OLDTAIL)
        LD    A,(HL)             ; A = OBJ (FROM NEW TAIL)
        DEC   HL
        DEC   DE
        LD    (DE),A
        RET
; NIL OLD TAIL CASE
ADDTQ1  DEC   HL                 ; BACKUP TO HEAD
        LD    (HL),E             ; HEAD = .OBJ
        RET
; SUBROUTINE TO POINT IX AT FIRST ENTRY ON A Q
; ENTRY:        HL = Q HEAD-TAIL
; EXIT:         IX,DE = OBJECT, A = L.O. BYTE OF OBJECT
;               NONZERO STATUS SET IF Q NOT EMPTY
FIRST   DI
        LD    E,(HL)
        INC   HL
        INC   HL
        LD    D,(HL)             ; D = H.O. ADDR. BYTE
        DEC   HL
        DEC   HL
        LD    A,E                ; E = HEAD OF Q
        AND   A
        PUSH  DE
        POP   IX
        RET
;
; **********************
; * GUNFIGHT CONSTANTS *
; **********************
        ORG   ($+1) AND 0FFFEH
INTTBL
LFRVEC  DW    GFLFR
WRTVEC  DW    GFWRIT
; WAGON LIMITS TABLE
WAGLMT  DB    TLINE
        DB    BLINE-24
GETRDY  DB    'GET READY'
; GUNFIGHTER LIMITS
GUNLMT  DB    0
        DB    LCACX-17
        DB    TLINE
        DB    BLINE-20
DRAW    DB    'DRAW'
; BULLET LIMITS
BULLMT  DB    0
        DB    159
        DB    ALINE
        DB    BLINE-1
BN      MACRO DX, ARMX, DY, ARMY
        DW    DX
        DB    ARMX
        DW    DY
        DB    ARMY 
        ENDM 
BULTAB  BN    768, 15, 768, 15
        BN    1024, 15, 512, 12
        BN    1024, 15, 256, 11
        BN    1024, 15, 0, 8
        BN    1024, 15, -256, 6
        BN    1024, 15, -512, 4
        BN    768, 15, -768, 3
LFTAB   DEF5  72, 22, 44, 67, 14
RFTAB   DEF5  18, 68, 40, 13, 63
GFCOLS  DB    0A2H   ; WAS 9DH - NOW MATCHES 3159 ROM
        DB    7DH    ; WAS 76H - NOW MATCHES 3159 ROM 
        DB    0BH    ; WAS FCH - NOW MATCHES 3159 ROM 
        DB    87H
        DB    0A2H   ; WAS 9DH - NOW MATCHES 3159 ROM 
        DB    7DH    ; WAS 76H - NOW MATCHES 3159 ROM 
        DB    6CH
        DB    87H
SINIT   DEF8  6, 6, 0, 0, 0, 30H, 30H, 0
        DEF4  0, 80H, 0FH, 0FH
NUMB    EQU   00000111B          ; COLOR MASK
BULT    EQU   00001011B
TIME    EQU   00001011B
LARGE   EQU   00001011B
LARG2   EQU   00001100B
;
; **********************
; * GUN FIGHT PATTERNS *
; **********************
;
; PATTERN TABLES
ARMTBL  DW    ARM0
        DW    ARM1
        DW    ARM2
        DW    ARM3
        DW    ARM4
        DW    ARM5
        DW    ARM6
; PATTERN DEFINITION MACROS
DEF02   MACRO AA, BB
        DB    $AA
        DB    $BB
        ENDM
DEF03   MACRO AA, BB, CC
        DB    $AA
        DB    $BB
        DB    $CC
        ENDM
DEF04   MACRO AA, BB, CC, DD
        DB    $AA
        DB    $BB
        DB    $CC
        DB    $DD
        ENDM
TREE    DEF2  1, 17
        DB    00001000B
        DB    00011100B
        DB    00111110B
        DB    01101011B
        DB    00001000B
        DB    00001000B
        DB    00111100B
        DB    01111110B
        DB    10101001B
        DB    00001000B
        DB    00111100B
        DB    01111110B
        DB    11101011B
        DB    10001001B
        DB    00001000B
        DB    00011100B
        DB    10101110B 
ARM0    DEF04 0A, 0A, 2, 5
        DEF02 40, 00
        DEF02 51, 00
        DEF02 04, 00
        DEF02 01, 00
        DEF02 00, 40
ARM1    DEF04 0A, 0A, 2, 3
        DEF02 50, 00
        DEF02 14, 00
        DEF02 01, 40
ARM2    DEF04 0A, 0A, 2, 2
        DEF02 54, 00
        DEF02 55,40
ARM3    DEF04 0A, 7, 2, 4
        DEF02 10, 00
        DEF02 05, 40
        DEF02 54, 00
        DEF02 50, 00
ARM4    DEF04 0A, 6, 2, 5
        DEF02 00, 40
        DEF02 45, 00
        DEF02 10, 00
        DEF02 50, 00
        DEF02 40, 00
ARM5    DEF04 0A, 5, 2, 6
        DEF02 00, 40
        DEF02 01, 00
        DEF02 05, 00
        DEF02 14, 00
        DEF02 54, 00
        DEF02 50, 00
ARM6    DEF04 0A, 5, 1, 5
        DB    01H
        DB    44H
        DB    10H
        DB    40H
        DB    40H
; **** NOTE ****
; THE FOLLOWING PATTERNS ARE CONSTRAINED TO EXIST ON THE
; PAGE.  THE FOLLOWING 'ORG' WILL DO IT FOR EXPERIMENTAL
; PATTERNS ARE: LEG0, LEG1, LEG2, KIL1, KIL2
;        ORG   ($+255) AND 0FF00H ; ***
LEG0    DB    LEG1 AND 0FFH
        DB    4
        DEF04 0, 0F, 3, 5
        DEF03 01, 55, 0
        DEF03 05, 45, 40
        DEF03 15, 01, 40
        DEF03 50, 01, 40
        DEF03 15, 00, 54
LEG1    DB    LEG2 AND 0FFH
        DB    4
        DEF04 2, 0F, 2, 5
        DEF02 15, 50
        DEF02 54, 50
        DEF02 50, 50
        DEF02 50, 50
        DEF02 55, 15
LEG2    DB    LEG0 AND 0FFH
        DB    4
        DEF04 3, 0F, 2, 5
        DEF02 55, 00
        DEF02 15, 00
        DEF02 15, 00
        DEF02 14, 00
        DEF02 05, 40
KIL1    DB    KIL2 AND 0FFH
        DB    20
        DEF04 0, 1, 4, 13
        DEF04 01, 10, 00, 00
        DEF04 45, 54, 40, 00
        DEF04 55, 55, 40, 00
        DEF04 0A, A8, 00, 00
        DEF04 0A, A2, 00, 01
        DEF04 0A, AA, 80, 14
        DEF04 02, AA, 00, 50
        DEF04 00, A8, 05, 40
        DEF04 05, 55, 54, 00
        DEF04 15, 55, 50, 00
        DEF04 54, 55, 50, 00
        DEF04 50, 05, 54, 00
        DEF04 50, 01, 55, 00
        DEF04 10, 01, 55, 40
        DEF04 10, 00, 05, 50
        DEF04 00, 00, 01, 50
        DEF04 00, 00, 00, 40
        DEF04 00, 00, 01, 40
        DEF04 00, 00, 00, 54
KIL2    DB    KIL2 AND 0FFH
        DB    60
        DEF04 0, D, 4, 7 
        DEF04 01, 10, 00, 00
        DEF04 45, 54, 40, 00
        DEF04 55, 55, 40, 00
        DEF04 0A, A8, 00, 00
        DEF04 0A, 88, 15, 01
        DEF04 16, A5, 55, 41
        DEF04 15, 55, 55, 55
CACTUS  DEF2  1, 12
        DB    00100000B
        DB    00110000B
        DB    00111000B
        DB    00110000B
        DB    10110010B
        DB    11110010B
        DB    11110110B
        DB    00111100B
        DB    00111100B
        DB    00110000B
        DB    00110000B
        DB    00110000B
GOTME   DB    'GOT ME'
NULPAT  DB    0
        DB    0
        DB    1
        DB    1
GFBODY  DEF04 0, 0, 3, F
        DEF03 00, 44, 00
        DEF03 11, 55, 10
        DEF03 15, 55, 50
        DEF03 02, AA, 00
        DEF03 02, A2, 00
        DEF03 02, AA, 80
        DEF03 00, AA, 00
        DEF03 00, A8, 00
        DEF03 15, 55, 00
        DEF03 55, 55, 50
        DEF03 51, 55, 50
        DEF03 41, 55, 00
        DEF03 41, 55, 00
        DEF03 45, 55, 00
        DB    01H 
        DB    55H
WAGPAT  DEF04 0, 0, 4, 16
        DEF04 00, 05, 50, 00
        DEF04 00, 55, 55, 00
        DEF04 01, 55, 55, 40
        DEF04 05, 55, 55, 50
        DEF04 15, 54, 15, 54
        DEF04 15, 50, 05, 54
        DEF04 15, 40, 01, 54
        DEF04 15, 40, 01, 54
        DEF04 15, 50, 05, 54
        DEF04 05, 54, 15, 50
        DEF04 01, 55, 55, 40
        DEF04 00, 55, 55, 00 
        DEF04 00, 15, 54, 00
        DEF04 02, AA, AA, 80
        DEF04 00, AA, AA, 00
        DEF04 12, AA, AA, 84
        DEF04 10, A8, 2A, 04
        DEF04 10, 20, 08, 04
        DEF04 52, AA, AA, 85
        DEF04 10, 20, 08, 04
        DEF04 10, 00, 00, 04
        DEF04 10, 00, 00, 04
FUDG4   DB    0
MSET    MASTER 11H
        VOLUME 09H, 0H
        RET
  ; HOME ON THE RANGE
HOME    CALL  MSET
        NOTE1 36, G1
        NOTE1 12, F1
        NOTE1 18, E1
        NOTE1 6, D1
        NOTE1 36, E1
        QUIET
  ; TAPS
TAPS
        CALL  MSET
        NOTE1 18, C1
        NOTE1 06, C1
        NOTE1 36, F1
        NOTE1 18, C1
        NOTE1 6, F1
        NOTE1 36, A1
        QUIET
  ; FUNERAL
FUNERL  CALL  MSET
        NOTE1 24, A0
        NOTE1 18, A0
        NOTE1 6, A0
        NOTE1 24, A0
        NOTE1 18, C1
        NOTE1 6, B0
        NOTE1 18, B0
        NOTE1 6, A0
        NOTE1 18, A0
        NOTE1 6, GS0
        NOTE1 18, A0
        QUIET
GUNSHOT OUTPUT 18H, 0F0H, 0F5H, 0FDH, 0FFH, 0, 3FH, 0FFH, 0EFH
        LEGSTA
        VOLUME 0FFH, 03FH
        REST  5
        NOTE1 5, 8FH
        NOTE1 5, 4CH
        QUIET
LASTB   EQU   $
;
; *************
; * RAM CELLS *
; *************
;       ORG   NORMEM+0E70H
;       DEFS  150                ; ALLLOW BIG STACK
STACK   EQU   $4F06              ; START STACK HERE
;       DEFS  12
MSTACK  EQU   $4F12
STRRAM  EQU   $4F12
WRITQ   EQU   $4F12              ; WRUTE Q HEADER
VECQ    EQU   $4F15              ; VECTOR Q HEADER
VECSTR  EQU   $4F18
BULV1   EQU   $4F18              ; BULLET VECTOR 1
BULV2   EQU   $4F2A              ; BULLET VECTOR 2
BULV3   EQU   $4F3C              ; BULLET VECTOR 3
BULV4   EQU   $4F4E              ; BULLET VECTOR 4
;       DEFS  1                  ; LEFT COMWBOY LINK
LCOWB   EQU   $4F61              ; LEFT GUNFIGHTER
;       DEFS  1                  ; RIGHT COWBOY LINK
RCOWB   EQU   $4F78              ; RIGHT GUNFIGHTER
WAGVEC  EQU   $4F8F              ; WAGON VECTOR
WAGON   EQU   $4F90              ; WAGVEC+VBSTAT
ENDRAM_ EQU   $4FA1
LBULS   EQU   $4FDA              ; CT5
RBULS   EQU   $4FDB              ; CT6
RFIELD  EQU   $4FA1
LSCORE  EQU   $4FA2
LFIELD  EQU   $4FA5
RSCORE  EQU   $4FA6
LEND    EQU   $1FEF              ; LASTB
;
;       END OF GUNFIGHT PROGRAM
;
; TWENTY-ONE FILLER BYTES BETWEEN GUNFIGHT AND
; END OF ROM TO MATCH 3159 ROM
        DB   0FFH, 0FFH, 0FFH, 0FFH, 0FFH
        DB   0FFH, 0FFH, 0FFH, 0FFH, 0FFH
        DB   0FFH, 0FFH, 0FFH, 0FFH, 0FFH
        DB   0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH

